@@ -32,7 +32,12 @@ Card: E-mu APS\n\
 PCI: 0x1102=0x0002\n\
 ");
 
-int snd_index[SND_CARDS] = SND_DEFAULT_IDX;	/* Index 1-MAX */
+#if defined(CONFIG_SND_SEQUENCER) && defined(CONFIG_SND_SYNTH_EMU10K1)
+#define ENABLE_SYNTH
+#include "../include/emu10k1_synth.h"
+#endif
+
+int snd_index[SND_CARDS] = SND_DEFAULT_IDX;	/* Index 0-MAX */
 char *snd_id[SND_CARDS] = SND_DEFAULT_STR;	/* ID for this card */
 int snd_dac_frame_size[SND_CARDS] = {[0 ... (SND_CARDS - 1)] = 128};
 int snd_adc_frame_size[SND_CARDS] = {[0 ... (SND_CARDS - 1)] = 64};
@@ -58,6 +63,9 @@ struct snd_emu10k1 {
 	snd_kmixer_t *mixer;
 	snd_rawmidi_t *midi_uart;
 	snd_hwdep_t *synth;
+#ifdef ENABLE_SYNTH
+	snd_seq_device_t *wave;
+#endif	
 };
 
 static struct snd_emu10k1 *snd_emu10k1_cards[SND_CARDS] = SND_DEFAULT_PTR;
@@ -135,6 +143,9 @@ static int __init snd_card_emu10k1_probe(int dev, struct snd_emu10k1 *scard)
 	snd_kmixer_t *mixer = NULL;
 	snd_rawmidi_t *midi_uart = NULL;
 	snd_hwdep_t *synth = NULL;
+#ifdef ENABLE_SYNTH
+	snd_seq_device_t *wave = NULL;
+#endif
 
 	card = snd_card_new(snd_index[dev], snd_id[dev],
 			    snd_card_emu10k1_use_inc, snd_card_emu10k1_use_dec);
@@ -179,6 +190,19 @@ static int __init snd_card_emu10k1_probe(int dev, struct snd_emu10k1 *scard)
 			 OPL3_HW_OPL3_FM801, -1, &synth) < 0)
 		goto __nodev;
 #endif
+#ifdef ENABLE_SYNTH
+	if (snd_seq_device_new(card, 1, SND_SEQ_DEV_ID_EMU10K1_SYNTH,
+			       sizeof(snd_emu10k1_synth_arg_t), &wave) < 0 ||
+	    wave == NULL) {
+		snd_printk("can't initialize Emu10k1 wavetable synth\n");
+	} else {
+		snd_emu10k1_synth_arg_t *arg;
+		arg = SND_SEQ_DEVICE_ARGPTR(wave);
+		strcpy(wave->name, "Emu-10k1 Synth");
+		arg->hwptr = scard->emu;
+		arg->index = 1;
+	}
+#endif
  
 	strcpy(card->abbreviation, "EMU10K1");
 	if (scard->emu->APS) {
@@ -203,6 +227,9 @@ static int __init snd_card_emu10k1_probe(int dev, struct snd_emu10k1 *scard)
 		scard->pcm_efx = pcm_efx;
 		scard->midi_uart = midi_uart;
 		scard->synth = synth;
+#ifdef ENABLE_SYNTH
+		scard->wave = wave;
+#endif
 		return 0;
 	}
 	goto __nodev;
