@@ -29,10 +29,24 @@
 #include "../include/opl3.h"
 #include "../include/initval.h"
 
-int snd_index[SND_CARDS] = SND_DEFAULT_IDX;	/* Index 1-MAX */
+MODULE_DESCRIPTION("\
+Driver: ESS AudioDrive ES18xx\n\
+Card: ESS ES1868\n\
+Card: ESS ES1869\n\
+Card: ESS ES1878\n\
+Card: ESS ES1879\n\
+ISAPNP: ESS1868=ESS1868,ESS0000\n\
+ISAPNP: ESS1868=ESS8611,ESS8610\n\
+ISAPNP: ESS0003=ESS1869,ESS0006\n\
+ISAPNP: ESS1869=ESS1869,ESS0006\n\
+ISAPNP: ESS1878=ESS1878,ESS0004\n\
+ISAPNP: ESS1879=ESS1879,ESS0009\n\
+");
+
+int snd_index[SND_CARDS] = SND_DEFAULT_IDX;	/* Index 0-MAX */
 char *snd_id[SND_CARDS] = SND_DEFAULT_STR;	/* ID for this card */
 int snd_port[SND_CARDS] = SND_DEFAULT_PORT;	/* 0x220,0x240,0x260,0x280 */
-#ifndef CONFIG_ISAPNP
+#ifndef __ISAPNP__
 int snd_mpu_port[SND_CARDS] = {[0 ... (SND_CARDS - 1)] = -1};
 #else
 int snd_mpu_port[SND_CARDS] = SND_DEFAULT_PORT;
@@ -43,7 +57,7 @@ int snd_dma1[SND_CARDS] = SND_DEFAULT_DMA;	/* 0,1,3 */
 int snd_dma1_size[SND_CARDS] = SND_DEFAULT_DMA_SIZE;	/* 8,16,32,64,128 */
 int snd_dma2[SND_CARDS] = SND_DEFAULT_DMA;	/* 0,1,3 */
 int snd_dma2_size[SND_CARDS] = SND_DEFAULT_DMA_SIZE;	/* 8,16,32,64,128 */
-#ifdef CONFIG_ISAPNP
+#ifdef __ISAPNP__
 int snd_isapnp[SND_CARDS] = {[0 ... (SND_CARDS - 1)] = 1};
 #endif
 MODULE_PARM(snd_index, "1-" __MODULE_STRING(SND_CARDS) "i");
@@ -51,24 +65,24 @@ MODULE_PARM_DESC(snd_index, "Index value for ES18xx soundcard.");
 MODULE_PARM(snd_id, "1-" __MODULE_STRING(SND_CARDS) "s");
 MODULE_PARM_DESC(snd_id, "ID string for ES18xx soundcard.");
 MODULE_PARM(snd_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_port, "Port # for ES18xx driver.");
+MODULE_PARM_DESC(snd_port, "Port # for ES18xx driver. [PORT12]");
 MODULE_PARM(snd_mpu_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_mpu_port, "MPU-401 port # for ES18xx driver.");
+MODULE_PARM_DESC(snd_mpu_port, "MPU-401 port # for ES18xx driver. [PORT12]");
 MODULE_PARM(snd_fm_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_fm_port, "FM port # for ES18xx driver (optional).");
+MODULE_PARM_DESC(snd_fm_port, "FM port # for ES18xx driver (optional). [PORT12]");
 MODULE_PARM(snd_irq, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_irq, "IRQ # for ES18xx driver.");
+MODULE_PARM_DESC(snd_irq, "IRQ # for ES18xx driver. [IRQ]");
 MODULE_PARM(snd_dma1, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma1, "DMA 1 # for ES18xx driver.");
+MODULE_PARM_DESC(snd_dma1, "DMA 1 # for ES18xx driver. [DMA]");
 MODULE_PARM(snd_dma1_size, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma1_size, "Size of DMA 1 # for ES18xx driver.");
+MODULE_PARM_DESC(snd_dma1_size, "Size of DMA 1 # for ES18xx driver. [DMA_SIZE]");
 MODULE_PARM(snd_dma2, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma2, "DMA 2 # for ES18xx driver.");
+MODULE_PARM_DESC(snd_dma2, "DMA 2 # for ES18xx driver. [DMA]");
 MODULE_PARM(snd_dma2_size, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma2_size, "Size of DMA 2 # for ES18xx driver.");
-#ifdef CONFIG_ISAPNP
+MODULE_PARM_DESC(snd_dma2_size, "Size of DMA 2 # for ES18xx driver. [DMA_SIZE]");
+#ifdef __ISAPNP__
 MODULE_PARM(snd_isapnp, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_isapnp, "Skip ISA PnP detection for specified soundcard.");
+MODULE_PARM_DESC(snd_isapnp, "Skip ISA PnP detection for specified soundcard. [BOOL]");
 #endif
 
 static struct snd_audiodrive {
@@ -81,13 +95,13 @@ static struct snd_audiodrive {
 	snd_rawmidi_t *rmidi;
 	snd_hwdep_t *synth;
 	es18xx_t *codec;
-#ifdef CONFIG_ISAPNP
+#ifdef __ISAPNP__
 	struct isapnp_dev *dev;
 	struct isapnp_dev *devc;
 #endif
 } *snd_audiodrive_cards[SND_CARDS] = SND_DEFAULT_PTR;
 
-#ifdef CONFIG_ISAPNP
+#ifdef __ISAPNP__
 static unsigned int snd_audiodrive_pnpids[] = {
 	/* ESS 1868 (integrated on Compaq dual P-Pro motherboard and Genius 18PnP 3D) */
 	(ISAPNP_VENDOR('E','S','S')<<16)|ISAPNP_DEVICE(0x1868),	  /* DEVICE */
@@ -129,20 +143,20 @@ static void snd_audiodrive_use_dec(snd_card_t * card)
 	MOD_DEC_USE_COUNT;
 }
 
-static es18xx_t *snd_audiodrive_detect(snd_card_t * card,
-				       int port,
-				       int mpu_port,
-				       int fm_port,
-				       snd_irq_t * irqptr,
-				       snd_dma_t * dma1ptr,
-				       snd_dma_t * dma2ptr)
+static es18xx_t * __init snd_audiodrive_detect(snd_card_t * card,
+					       int port,
+					       int mpu_port,
+					       int fm_port,
+					       snd_irq_t * irqptr,
+					       snd_dma_t * dma1ptr,
+					       snd_dma_t * dma2ptr)
 {
 	es18xx_t *codec;
-	int ctrl_port;
+	int ctrl_port, err;
 
-	codec = snd_es18xx_new_device(card, port, mpu_port, fm_port,
-				      irqptr, dma1ptr, dma2ptr);
-	if(!codec)
+	err = snd_es18xx_new_device(card, port, mpu_port, fm_port,
+				    irqptr, dma1ptr, dma2ptr, &codec);
+	if(err < 0)
 		return NULL;
 
 	/* Audio ports registration */
@@ -176,7 +190,7 @@ static es18xx_t *snd_audiodrive_detect(snd_card_t * card,
 
  giveup:
 	snd_unregister_ioports(card);
-	snd_kfree(codec);
+	snd_magic_kfree(codec);
 	return NULL;
 }
 
@@ -245,9 +259,9 @@ static void snd_audiodrive_interrupt(int irq, void *dev_id, struct pt_regs *regs
 	}
 }
 
-static int snd_audiodrive_resources(int dev,
-				    struct snd_audiodrive *acard,
-				    snd_card_t * card)
+static int __init snd_audiodrive_resources(int dev,
+					   struct snd_audiodrive *acard,
+					   snd_card_t * card)
 {
 	static int possible_irqs[] = {5, 9, 10, 7, 11, 12, -1};
 	static int possible_dmas[] = {1, 0, 3, -1};
@@ -271,8 +285,8 @@ static int snd_audiodrive_resources(int dev,
 	return 0;
 }
 
-#ifdef CONFIG_ISAPNP
-static int snd_audiodrive_isapnp(int dev, struct snd_audiodrive *acard)
+#ifdef __ISAPNP__
+static int __init snd_audiodrive_isapnp(int dev, struct snd_audiodrive *acard)
 {
 	static int idx = 0;
 	static struct isapnp_card *card = NULL;
@@ -288,19 +302,19 @@ static int snd_audiodrive_isapnp(int dev, struct snd_audiodrive *acard)
 			break;
 		idx += step;
 	}
-	if (!card) {
+	if (card == NULL) {
 		snd_printdd("isapnp failed for ES18xx\n");
 		return -ENODEV;
 	}
 	tmp = snd_audiodrive_pnpids[idx+1];
 	acard->dev = pdev = isapnp_find_dev(card, tmp >> 16, tmp & 0xffff, NULL);
-	if (!pdev) {
+	if (pdev == NULL) {
                 snd_printdd("isapnp failed for ES18xx - %i\n", idx);
                 goto __again;           /* maybe we have another config */
         }
 	tmp = snd_audiodrive_pnpids[idx+2];
 	acard->devc = isapnp_find_dev(card, tmp >> 16, tmp & 0xffff, NULL);
-	if (!acard->devc) {
+	if (acard->devc == NULL) {
                 snd_printdd("isapnp failed for ES18xx (Control) - %i\n", idx);
                 goto __again;           /* maybe we have another config */
         }
@@ -311,7 +325,7 @@ static int snd_audiodrive_isapnp(int dev, struct snd_audiodrive *acard)
 		snd_printk("ES18xx isapnp control configure failure (out of resources?)\n");
 		return -EAGAIN;
 	}
-	snd_printdd("isapnp ES18xx: port=0x%x\n", cfg.port[0]);
+	snd_printdd("isapnp ES18xx: port=0x%x\n", acard->devc->resource[0].start);
 	/* PnP initialization */
 	if (pdev->prepare(pdev)<0) {
 		acard->devc->deactivate(acard->devc);
@@ -360,11 +374,11 @@ static void snd_audiodrive_deactivate(struct snd_audiodrive *acard)
 	if (acard->dev)
 		acard->dev->deactivate(acard->dev);
 }
-#endif /* CONFIG_ISAPNP */
+#endif /* __ISAPNP__ */
 
-static int snd_audiodrive_probe(int dev, struct snd_audiodrive *acard)
+static int __init snd_audiodrive_probe(int dev, struct snd_audiodrive *acard)
 {
-#ifndef CONFIG_ISAPNP
+#ifndef __ISAPNP__
 	static int possible_ports[] =
 		{0x220, 0x240, 0x260, 0x280, -1};
 	int *ports = possible_ports;
@@ -375,13 +389,14 @@ static int snd_audiodrive_probe(int dev, struct snd_audiodrive *acard)
 	snd_rawmidi_t *rmidi = NULL;
 	snd_hwdep_t *synth = NULL;
 	es18xx_t *codec;
+	int err;
 
 	card = snd_card_new(snd_index[dev], snd_id[dev],
 			 snd_audiodrive_use_inc, snd_audiodrive_use_dec);
-	if(!card)
+	if(card == NULL)
 		return -ENOMEM;
 	card->type = SND_CARD_TYPE_ESS_ES18XX;
-#ifdef CONFIG_ISAPNP
+#ifdef __ISAPNP__
 	if (snd_isapnp[dev] && snd_audiodrive_isapnp(dev, acard) < 0) {
 		snd_card_free(card);
 		return -ENODEV;
@@ -392,7 +407,7 @@ static int snd_audiodrive_probe(int dev, struct snd_audiodrive *acard)
 		return -EBUSY;
 	}
 	codec = NULL;
-#ifndef CONFIG_ISAPNP
+#ifndef __ISAPNP__
 	if(snd_port[dev] == SND_AUTO_PORT) {
 		for(ports = possible_ports; *ports >= 0; ports++) {
 			codec = snd_audiodrive_detect(card, *ports,
@@ -401,7 +416,7 @@ static int snd_audiodrive_probe(int dev, struct snd_audiodrive *acard)
 			if(codec)
 				break;
 		}
-		if (!codec) {
+		if (codec == NULL) {
 			snd_card_free(card);
 			return -ENODEV;
 		}
@@ -410,58 +425,30 @@ static int snd_audiodrive_probe(int dev, struct snd_audiodrive *acard)
 		codec = snd_audiodrive_detect(card, snd_port[dev],
 			snd_mpu_port[dev], snd_fm_port[dev],
 			acard->irqptr, acard->dma1ptr, acard->dma2ptr);
-		if (!codec) {
+		if (codec == NULL) {
 			snd_card_free(card);
 			return -ENODEV;
 		}
-#ifndef CONFIG_ISAPNP
+#ifndef __ISAPNP__
 	}
 #endif
-	pcm = snd_es18xx_pcm(codec);
-	if (!pcm)
+	err = snd_es18xx_pcm(codec, 0, &pcm);
+	if (err < 0)
 		goto __nodev;
 
-	mixer = snd_es18xx_mixer(codec);
-	if(!mixer)
+	err = snd_es18xx_mixer(codec, 0, pcm, &mixer);
+	if(err < 0)
 		goto __nodev;
 
-	synth = snd_opl3_new_device(card, codec->port, codec->port + 2, OPL3_HW_OPL3, -1);
+	snd_opl3_new(card, 0, codec->port, codec->port + 2, OPL3_HW_OPL3, -1, &synth);
 
 	if(snd_mpu_port[dev] > SND_AUTO_PORT) {
-		snd_printk("ES%x: midi isn't working!!!\n", codec->version);
-		rmidi = snd_mpu401_uart_new_device(card, MPU401_HW_ES18XX,
-						codec->mpu_port,
-						acard->irqptr->irq);
-		if(!rmidi)
+		err = snd_mpu401_uart_new(card, 0, MPU401_HW_ES18XX,
+					  codec->mpu_port,
+					  acard->irqptr->irq,
+					  &rmidi);
+		if(err < 0)
 			goto __nodev;
-		if(snd_rawmidi_register(rmidi, 0) < 0)
-			goto __nodev;
-	}
-	if(synth && snd_hwdep_register(synth, 0) < 0) {
-		if(rmidi)
-			snd_rawmidi_unregister(rmidi);
-		rmidi = NULL;
-		goto __nodev;
-	}
-	if(snd_mixer_register(mixer, 0) < 0) {
-		if(synth)
-			snd_hwdep_unregister(synth);
-		synth = NULL;
-		if(rmidi)
-			snd_rawmidi_unregister(rmidi);
-		rmidi = NULL;
-		goto __nodev;
-	}
-	if(snd_pcm_register(pcm, 0) < 0) {
-		if(synth)
-			snd_hwdep_unregister(synth);
-		synth = NULL;
-		if(rmidi)
-			snd_rawmidi_unregister(rmidi);
-		rmidi = NULL;
-		snd_mixer_unregister(mixer);
-		mixer = NULL;
-		goto __nodev;
 	}
 	sprintf(card->abbreviation, "ES%x", codec->version);
 	sprintf(card->shortname, "ESS AudioDrive ES%x", codec->version);
@@ -480,31 +467,17 @@ static int snd_audiodrive_probe(int dev, struct snd_audiodrive *acard)
 		acard->codec = codec;
 		return 0;
 	}
-	if (synth)
-		snd_hwdep_unregister(synth);
-	if (rmidi)
-		snd_rawmidi_unregister(rmidi);
-	if (mixer)
-		snd_mixer_unregister(mixer);
-	if (pcm)
-		snd_pcm_unregister(pcm);
-	snd_card_free(card);
-	return -ENOMEM;
 
       __nodev:
-	if(synth)
-		snd_hwdep_free(synth);
-	if(rmidi)
-		snd_rawmidi_free(rmidi);
-	if(mixer)
-		snd_mixer_free(mixer);
-	if(pcm)
-		snd_pcm_free(pcm);
 	snd_card_free(card);
 	return -ENXIO;
 }
 
-int init_module(void)
+#ifdef MODULE
+int __init init_module(void)
+#else
+int __init alsa_card_es18xx_init(void)
+#endif
 {
 	int dev, cards;
 	struct snd_audiodrive *acard;
@@ -512,31 +485,37 @@ int init_module(void)
 	for(dev = cards = 0; dev < SND_CARDS && snd_port[dev] > 0; dev++) {
 		acard = (struct snd_audiodrive *)
 				snd_kcalloc(sizeof(struct snd_audiodrive), GFP_KERNEL);
-		if(!acard)
+		if(acard == NULL)
 			continue;
 		if(snd_audiodrive_probe(dev, acard) < 0) {
-#ifdef CONFIG_ISAPNP
+#ifdef __ISAPNP__
 			snd_audiodrive_deactivate(acard);
 #endif
 			snd_kfree(acard);
 			if(snd_port[dev] == SND_AUTO_PORT)
 				break;
+#ifdef MODULE
 			snd_printk("ESS AudioDrive ES18xx soundcard #%i not found at 0x%x or device busy\n",
 				dev + 1, snd_port[dev]);
+#endif
 			continue;
 		}
 		snd_audiodrive_cards[dev] = acard;
 		cards++;
 	}
 	if(!cards) {
+#ifdef MODULE
 		snd_printk("ESS AudioDrive ES18xx soundcard #%i not found or device busy\n",
 			dev + 1);
+#endif
 		return -ENODEV;
 	}
 	return 0;
 }
 
-void cleanup_module(void)
+#ifdef MODULE
+
+void __exit cleanup_module(void)
 {
 	int idx;
 	struct snd_audiodrive *acard;
@@ -545,23 +524,12 @@ void cleanup_module(void)
 		acard = snd_audiodrive_cards[idx];
 		if(acard) {
 			snd_card_unregister(acard->card);
-			if(acard->synth)
-				snd_hwdep_unregister(acard->synth);
-			if(acard->rmidi)
-				snd_rawmidi_unregister(acard->rmidi);
-			if(acard->mixer)
-				snd_mixer_unregister(acard->mixer);
-			if(acard->pcm)
-				snd_pcm_unregister(acard->pcm);
-			if (acard->codec) {
-				snd_kfree(acard->codec);
-				acard->codec = NULL;
-			}
-			snd_card_free(acard->card);
-#ifdef CONFIG_ISAPNP
+#ifdef __ISAPNP__
 			snd_audiodrive_deactivate(acard);
 #endif
 			snd_kfree(acard);
 		}
 	}
 }
+
+#endif
