@@ -20,17 +20,21 @@
  */
 
 #define SND_MAIN_OBJECT_FILE
-#include "driver.h"
-#include "fm801.h"
-#include "mpu401.h"
-#include "opl3.h"
-#include "initval.h"
+#include "../include/driver.h"
+#include "../include/fm801.h"
+#include "../include/mpu401.h"
+#include "../include/opl3.h"
+#include "../include/initval.h"
 
-int snd_index[SND_CARDS] = SND_DEFAULT_IDX;	/* Index 1-MAX */
+MODULE_DESCRIPTION("\
+Driver: ForteMedia FM801\n\
+PCI: 0x1319=0x0801\n\
+");
+
+int snd_index[SND_CARDS] = SND_DEFAULT_IDX;	/* Index 0-MAX */
 char *snd_id[SND_CARDS] = SND_DEFAULT_STR;	/* ID for this card */
 int snd_dac_frame_size[SND_CARDS] = {[0 ... (SND_CARDS - 1)] = 128};
 int snd_adc_frame_size[SND_CARDS] = {[0 ... (SND_CARDS - 1)] = 128};
-#ifdef MODULE_PARM
 MODULE_PARM(snd_index, "1-" __MODULE_STRING(SND_CARDS) "i");
 MODULE_PARM_DESC(snd_index, "Index value for the FM801 soundcard.");
 MODULE_PARM(snd_id, "1-" __MODULE_STRING(SND_CARDS) "s");
@@ -39,10 +43,9 @@ MODULE_PARM(snd_dac_frame_size, "1-" __MODULE_STRING(SND_CARDS) "i");
 MODULE_PARM_DESC(snd_dac_frame_size, "DAC frame size in kB for the FM801 soundcard.");
 MODULE_PARM(snd_adc_frame_size, "1-" __MODULE_STRING(SND_CARDS) "i");
 MODULE_PARM_DESC(snd_adc_frame_size, "ADC frame size in kB for the FM801 soundcard.");
-#endif
 
 struct snd_fm801 {
-	struct snd_pci_dev pci;
+	struct pci_dev *pci;
 	snd_irq_t *irqptr;
 	snd_dma_t *dma1ptr;	/* DAC frame */
 	snd_dma_t *dma2ptr;	/* ADC frame */
@@ -51,7 +54,7 @@ struct snd_fm801 {
 	snd_pcm_t *pcm;
 	snd_kmixer_t *mixer;
 	snd_rawmidi_t *midi_uart;
-	snd_synth_t *synth;
+	snd_hwdep_t *synth;
 };
 
 static struct snd_fm801 *snd_fm801_cards[SND_CARDS] = SND_DEFAULT_PTR;
@@ -66,13 +69,17 @@ static void snd_card_fm801_use_dec(snd_card_t * card)
 	MOD_DEC_USE_COUNT;
 }
 
-static int snd_card_fm801_detect(snd_card_t * card, struct snd_fm801 *scard, int pci_index)
+static int __init snd_card_fm801_detect(snd_card_t * card, struct snd_fm801 *scard)
 {
-	if (snd_pci_find_device(PCI_VENDOR_ID_FORTEMEDIA,
-			         PCI_DEVICE_ID_FORTEMEDIA_FM801,
-			         pci_index, &scard->pci))
+	if ((scard->pci = pci_find_device(PCI_VENDOR_ID_FORTEMEDIA,
+					  PCI_DEVICE_ID_FORTEMEDIA_FM801,
+					  scard->pci)) == NULL)
 		return -ENODEV;
-	if (snd_register_ioport(card, scard->pci.base_address[0] & ~PCI_BASE_ADDRESS_SPACE, 0x80, "FM801", NULL) < 0)
+#ifdef NEW_PCI
+	if (snd_register_ioport(card, scard->pci->resource[0].start, 0x80, "FM801", NULL) < 0)
+#else
+	if (snd_register_ioport(card, scard->pci->base_address[0] & ~PCI_BASE_ADDRESS_SPACE, 0x80, "FM801", NULL) < 0)
+#endif
 		return -ENODEV;
 	return 0;
 }
@@ -83,10 +90,10 @@ static void snd_card_fm801_interrupt(int irq, void *dev_id, struct pt_regs *regs
 	fm801_t *codec;
 	unsigned int status;
 
-	if (!scard || !(codec = scard->codec))
+	if (scard == NULL || (codec = scard->codec) == NULL)
 		return;
-	status = FM801_REG(codec, IRQ_STATUS);
-	if (status & (FM801_IRQ_PLAYBACK|FM801_IRQ_CAPTURE|FM801_IRQ_VOLUME)) {
+	status = inw(FM801_REG(codec, IRQ_STATUS));
+	if (status & (FM801_IRQ_PLAYBACK|FM801_IRQ_CAPTURE|FM801_IRQ_MPU|FM801_IRQ_VOLUME)) {
 		if (status & (FM801_IRQ_PLAYBACK|FM801_IRQ_CAPTURE))
 			snd_fm801_interrupt(codec, status);
 		if (status & FM801_IRQ_MPU) {
@@ -101,13 +108,14 @@ static void snd_card_fm801_interrupt(int irq, void *dev_id, struct pt_regs *regs
 	}
 }
 
-static int snd_card_fm801_resources(snd_card_t * card,
-                                  struct snd_fm801 *scard, int dev)
+static int __init snd_card_fm801_resources(snd_card_t * card,
+					   struct snd_fm801 *scard,
+					   int dev)
 {
 	int err;
 
 	if ((err = snd_register_interrupt(card,
-			"FM801", scard->pci.irq,
+			"FM801", scard->pci->irq,
 			SND_IRQ_TYPE_PCI, snd_card_fm801_interrupt,
 			scard, NULL, &scard->irqptr)) < 0)
 		return err;
@@ -124,26 +132,26 @@ static int snd_card_fm801_resources(snd_card_t * card,
 	return 0;
 }
 
-static int snd_card_fm801_probe(int dev, struct snd_fm801 *scard)
+static int __init snd_card_fm801_probe(int dev, struct snd_fm801 *scard)
 {
 	snd_card_t *card;
 	snd_pcm_t *pcm = NULL;
 	snd_kmixer_t *mixer = NULL;
 	snd_rawmidi_t *midi_uart = NULL;
-	snd_synth_t *synth = NULL;
-	int pci_index;
+	snd_hwdep_t *synth = NULL;
 
 	card = snd_card_new(snd_index[dev], snd_id[dev],
 			    snd_card_fm801_use_inc, snd_card_fm801_use_dec);
-	if (!card)
+	if (card == NULL)
 		return -ENOMEM;
 	card->static_data = scard;
 	card->type = SND_CARD_TYPE_CS461X;
-	for (pci_index = 0; pci_index < SND_CARDS; pci_index++) {
-		if (!snd_card_fm801_detect(card, scard, pci_index))
+	scard->pci = NULL;
+	do {
+		if (!snd_card_fm801_detect(card, scard))
 			break;
-	}
-	if (pci_index >= SND_CARDS) {
+	} while (scard->pci);
+	if (scard->pci == NULL) {
 		snd_card_free(card);
 		return -ENODEV;
 	}
@@ -151,57 +159,34 @@ static int snd_card_fm801_probe(int dev, struct snd_fm801 *scard)
 		snd_card_free(card);
 		return -ENODEV;
 	}
-	scard->codec = snd_fm801_create(card, &scard->pci,
-				         scard->dma1ptr,
-				         scard->dma2ptr,
-				         scard->irqptr);
-	if (!scard->codec)
+	if (snd_fm801_create(card, scard->pci,
+			     scard->dma1ptr,
+			     scard->dma2ptr,
+			     scard->irqptr,
+			     &scard->codec) < 0)
 		goto __nodev;
-	mixer = snd_fm801_mixer(scard->codec, 0);
-	if (!mixer)
+	if (snd_fm801_pcm(scard->codec, 0, &pcm) < 0)
 		goto __nodev;
-	pcm = snd_fm801_pcm(scard->codec);
-	if (!pcm)
+	if (snd_fm801_mixer(scard->codec, 0, pcm, &mixer) < 0)
 		goto __nodev;
-	midi_uart = snd_mpu401_uart_new_device(card, MPU401_HW_FM801,
-						 FM801_REG(scard->codec, MPU401_DATA),
-						 scard->irqptr->irq);
-	if (!midi_uart)
+	if (snd_mpu401_uart_new(card, 0, MPU401_HW_FM801,
+				FM801_REG(scard->codec, MPU401_DATA),
+				scard->irqptr->irq, &midi_uart) < 0)
 		goto __nodev;
-	synth = snd_opl3_new_device(card, FM801_REG(scard->codec, OPL3_BANK0),
-					  FM801_REG(scard->codec, OPL3_BANK1),
-					  OPL3_HW_OPL3, -1);
-	if (!synth)
+	if (snd_opl3_new(card, 0, FM801_REG(scard->codec, OPL3_BANK0),
+			 FM801_REG(scard->codec, OPL3_BANK1),
+			 OPL3_HW_OPL3_FM801, -1, &synth) < 0)
 		goto __nodev;
  
-	if (snd_mixer_register(mixer, 0) < 0)
-		goto __nodev;
-	if (snd_pcm_register(pcm, 0) < 0) {
-		snd_mixer_unregister(mixer);
-		mixer = NULL;
-		goto __nodev;
-	}
-	if (snd_rawmidi_register(midi_uart, 0) < 0) {
-		snd_pcm_unregister(pcm);
-		pcm = NULL;
-		snd_mixer_unregister(mixer);
-		mixer = NULL;
-		goto __nodev;
-	}
-	if (snd_synth_register(synth) < 0) {
-		snd_pcm_unregister(pcm);
-		pcm = NULL;
-		snd_mixer_unregister(mixer);
-		mixer = NULL;
-		snd_rawmidi_unregister(midi_uart);
-		midi_uart = NULL;
-		goto __nodev;
-	}
 	strcpy(card->abbreviation, "FM801");
 	strcpy(card->shortname, "ForteMedia FM801");
 	sprintf(card->longname, "%s at 0x%lx, irq %i",
 		card->shortname,
-		scard->pci.base_address[0] & ~PCI_BASE_ADDRESS_SPACE,
+#ifdef NEW_PCI
+		scard->pci->resource[0].start,
+#else
+		scard->pci->base_address[0] & ~PCI_BASE_ADDRESS_SPACE,
+#endif
 		scard->irqptr->irq);
 
 	if (!snd_card_register(card)) {
@@ -215,68 +200,43 @@ static int snd_card_fm801_probe(int dev, struct snd_fm801 *scard)
 	goto __nodev;
 
       __nodev:
-	if (synth)
-		snd_synth_free(synth);
-	if (midi_uart)
-		snd_rawmidi_free(midi_uart);
-	if (mixer)
-		snd_mixer_free(mixer);
-	if (pcm)
-		snd_pcm_free(pcm);
-	if (scard->codec) {
-		snd_fm801_free(scard->codec);
-		scard->codec = NULL;
-	}
 	snd_card_free(card);
 	return -ENXIO;
 }
 
-static int snd_card_fm801_free(int dev)
+#ifdef MODULE
+
+static int __exit snd_card_fm801_free(int dev)
 {
 	struct snd_fm801 *scard;
-	snd_pcm_t *pcm;
 
 	scard = snd_fm801_cards[dev];
 	snd_fm801_cards[dev] = NULL;
 	if (scard) {
 		snd_card_unregister(scard->card);
-		if (scard->synth)
-			snd_synth_unregister(scard->synth);
-		if (scard->midi_uart)
-			snd_rawmidi_unregister(scard->midi_uart);
-		if (scard->mixer)
-			snd_mixer_unregister(scard->mixer);
-		if (scard->pcm) {
-			pcm = scard->pcm;
-			scard->pcm = NULL;	/* turn off interrupts */
-			snd_pcm_unregister(pcm);
-		}
-		if (scard->codec) {
-			snd_fm801_free(scard->codec);
-			scard->codec = NULL;
-		}
-		snd_card_free(scard->card);
-		snd_free(scard, sizeof(struct snd_fm801));
+		snd_kfree(scard);
 	}
 	return 0;
 }
 
-int init_module(void)
+#endif
+
+#ifdef MODULE
+int __init init_module(void)
+#else
+int __init alsa_card_fm801_init(void)
+#endif
 {
 	int dev, cards;
 	struct snd_fm801 *scard;
 
-#ifndef LINUX_2_1
-	register_symtab(NULL);
-#endif
 	for (dev = cards = 0; dev < SND_CARDS; dev++) {
 		scard = (struct snd_fm801 *)
-				snd_malloc(sizeof(struct snd_fm801));
-		if (!scard)
+				snd_kcalloc(sizeof(struct snd_fm801), GFP_KERNEL);
+		if (scard == NULL)
 			continue;
-		memset(scard, 0, sizeof(struct snd_fm801));
 		if (snd_card_fm801_probe(dev, scard) < 0) {
-			snd_free(scard, sizeof(struct snd_fm801));
+			snd_kfree(scard);
 			break;
 		}
 		snd_fm801_cards[dev] = scard;
@@ -289,10 +249,14 @@ int init_module(void)
 	return 0;
 }
 
-void cleanup_module(void)
+#ifdef MODULE
+
+void __exit cleanup_module(void)
 {
 	int dev;
 
 	for (dev = 0; dev < SND_CARDS; dev++)
 		snd_card_fm801_free(dev);
 }
+
+#endif
