@@ -37,7 +37,7 @@
 //      MIDI Synchronization to Video, ADAT, SMPTE and other Clock sources
 //      128 'scene' memories, recallable from MIDI program change
 ///////////////////////////////////////////////////////////////////////////
-//      remember to set SND_RAWMIDI_PORTS in rawmidi.h to >= 8 for 1 mtp or 16 for 2 mtps
+//      remember to set SND_RAWMIDI_PORTS in rawmidi.h to >= 18
 ///////////////////////////////////////////////////////////////////////////
 
 #define SND_MAIN_OBJECT_FILE
@@ -57,19 +57,16 @@ static int snd_mtpav_input_close(snd_rawmidi_t * rmidi);
 static void snd_mtpav_input_trigger(snd_rawmidi_t * rmidi, int up);
 static int snd_mtpav_output_open(snd_rawmidi_t * rmidi);
 static int snd_mtpav_output_close(snd_rawmidi_t * rmidi);
-static void snd_mtpav_output_trigger(snd_rawmidi_t * rmidi, int up);
 static void snd_mtpav_mputreg(U16 reg, U8 val);
 
-static void snd_mtpav_inmidi_h(TSmtp * mcrd, U8 inbyte);
+static void snd_mtpav_inmidi_h(U8 inbyte);
 
 /////////////////////////////////////////////////////////////////
 //      globals
 
-EXPORT_NO_SYMBOLS;
 MODULE_DESCRIPTION("\
 Driver: MOTU MidiTimePiece AV multiport MIDI\n\
 ");
-MODULE_LICENSE("GPL");
 
 int snd_index = SND_DEFAULT_IDX1;
 char *snd_id = SND_DEFAULT_STR1;
@@ -85,19 +82,29 @@ MODULE_PARM(snd_irq, "i");
 MODULE_PARM_DESC(snd_irq, "Parallel IRQ # for MotuMTPAV MIDI.");
 
 static TSmtp *mtp_card;
-//static U8 outmsg_lastport;
-//static U8 inmsg_lastport;
+static U8 outmsg_lastport;
+static U8 inmsg_lastport;
 
 static struct snd_stru_rawmidi_channel_hw snd_mtpav_output = {
-	open:		snd_mtpav_output_open,
-	close:		snd_mtpav_output_close,
-	trigger:	snd_mtpav_output_trigger,
+	SND_RAWMIDI_HW_POLL,	// flags
+	NULL,			/// private_data
+	NULL,			/// private_free
+	snd_mtpav_output_open,	/// open
+	snd_mtpav_output_close,	/// close
+	NULL,			/// trigger
+	{snd_mtpav_output_write},	// io.write
+	NULL,			// abort
 };
 
 static struct snd_stru_rawmidi_channel_hw snd_mtpav_input = {
-	open:		snd_mtpav_input_open,
-	close:		snd_mtpav_input_close,
-	trigger:	snd_mtpav_input_trigger,
+	0,			// flags
+	NULL,			// private_data
+	NULL,			// private_free
+	snd_mtpav_input_open,	// open
+	snd_mtpav_input_close,	// close
+	snd_mtpav_input_trigger,	// trigger
+	{NULL},			// io.read
+	NULL,			// abort
 };
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -130,24 +137,26 @@ static void snd_mtpav_mputreg(U16 reg, U8 val)
 
 //////////////////////////////////////////////////////////////////////////
 
-static void snd_mtpav_wait_rfdhi(void)
-{
-	U8 sbyte;
+static U32 mtp_lastoutport = 0xffffffff;
 
-	sbyte = snd_mtpav_getreg(SREG);
-	while (!(sbyte & SIGS_RFD)) {
-		sbyte = snd_mtpav_getreg(SREG);
-	}
-
-}
-
-static void snd_mtpav_send_byte(U8 byte)
+static void snd_mtpav_output_write(snd_rawmidi_t * rmidi)
 {
+	U8 outbyte;
+	U8 ifsent;
 	U8 tcbyt;
 	U8 clrwrite;
 	U8 setwrite;
+	U8 sbyt;
+	U32 port;
 
-	snd_mtpav_wait_rfdhi();
+	/////////////////
+	//      wait for RFD to go high 
+
+	sbyt = snd_mtpav_getreg(SREG);
+
+	while (!(sbyt & SIGS_RFD)) {
+		sbyt = snd_mtpav_getreg(SREG);
+	}
 
 	/////////////////
 
@@ -155,37 +164,23 @@ static void snd_mtpav_send_byte(U8 byte)
 	clrwrite = tcbyt & (SIGC_WRITE ^ 0xff);
 	setwrite = tcbyt | SIGC_WRITE;
 
-	snd_mtpav_mputreg(DREG, byte);
-	snd_mtpav_mputreg(CREG, clrwrite);	// clear write bit
-
-	snd_mtpav_mputreg(CREG, setwrite);	// set write bit
-
-}
-
-//
-
-static void snd_mtpav_output_write(snd_rawmidi_t * rmidi)
-{
-	U8 outbyte;
-	U8 ifsent;
-	U32 port;
-	unsigned long flags;
-
-
 	/////////////////
 	// send port change command if necessary
 
 	port = (U32) rmidi->device;
 
-	spin_lock_irqsave(&mtp_card->spinlock, flags);
+	if (port != mtp_lastoutport) {
+		mtp_lastoutport = port;
 
-	if (port != mtp_card->outmidiport) {
-		mtp_card->outmidiport = port;
+		snd_printk("new outport: 0x%x\n", (unsigned int) port + 1);
 
-		snd_mtpav_send_byte(0xf5);
-		snd_mtpav_send_byte(port);
-		//snd_printk("new outport: 0x%x\n", (unsigned int) port);
+		snd_mtpav_mputreg(DREG, 0xf5);
+		snd_mtpav_mputreg(CREG, clrwrite);	// clear write bit
+		snd_mtpav_mputreg(CREG, setwrite);	// set write bit
 
+		snd_mtpav_mputreg(DREG, (port + 1));
+		snd_mtpav_mputreg(CREG, clrwrite);	// clear write bit
+		snd_mtpav_mputreg(CREG, setwrite);	// set write bit
 	}
 	///////////////////
 	//      send data
@@ -194,24 +189,11 @@ static void snd_mtpav_output_write(snd_rawmidi_t * rmidi)
 	while (ifsent == 1) {
 		ifsent = snd_rawmidi_transmit(rmidi, &outbyte, 1);
 
-		if (ifsent == 1)
-			snd_mtpav_send_byte(outbyte);
-	}
-
-	spin_unlock_irqrestore(&mtp_card->spinlock, flags);
-}
-
-/////////////////////////////////////////////////////
-//      mtpav control
-
-static void snd_mtpav_portscan(void)	// put mtp into smart routing mode
-{
-	U8 port;
-
-	for (port = 0; port < 8; port++) {
-		snd_mtpav_send_byte(0xf5);
-		snd_mtpav_send_byte(port);
-		snd_mtpav_send_byte(0xfe);
+		if (ifsent == 1) {
+			snd_mtpav_mputreg(DREG, outbyte);
+			snd_mtpav_mputreg(CREG, clrwrite);	// clear write bit
+			snd_mtpav_mputreg(CREG, setwrite);	// set write bit
+		}
 	}
 }
 
@@ -221,11 +203,10 @@ static int snd_mtpav_input_open(snd_rawmidi_t * rmidi)
 {
 	U32 flags;
 	U32 port = (U32) rmidi->private_data;
-	//printk("mtpav port: %d opened\n", (int) port);
+	printk("mtpav port: %d opened\n", (int) port);
 	spin_lock_irqsave(&mtp_card->spinlock, flags);
 	mtp_card->mode[port] |= MTPAV_MODE_INPUT_OPENED;
-	snd_mtpav_mputreg(CREG, (SIGC_INTEN | SIGC_WRITE));	// enable pport interrupts
-
+	snd_mtpav_mputreg(CREG, SIGC_INTEN);	// enable pport interrupts
 	spin_unlock_irqrestore(&mtp_card->spinlock, flags);
 	return 0;
 }
@@ -236,23 +217,11 @@ static int snd_mtpav_input_close(snd_rawmidi_t * rmidi)
 {
 	U32 flags;
 	U32 port = (U32) rmidi->private_data;
-	UBOOL killirq = TRUE;
-	U32 tport;
-
-	//printk("mtpav port: %d closed\n", (int) port);
+	printk("mtpav port: %d closed\n", (int) port);
 
 	spin_lock_irqsave(&mtp_card->spinlock, flags);
-
 	mtp_card->mode[port] &= (~MTPAV_MODE_INPUT_OPENED);
-
-	for (tport = 0; tport < NUMPORTS; tport++) {
-		if (mtp_card->mode[tport] & MTPAV_MODE_INPUT_OPENED)
-			killirq = FALSE;
-	}
-
-	if (killirq == TRUE)
-		snd_mtpav_mputreg(CREG, 0);	// disable pport interrupts
-
+	snd_mtpav_mputreg(CREG, 0);	// enable pport interrupts
 	spin_unlock_irqrestore(&mtp_card->spinlock, flags);
 	return 0;
 }
@@ -269,6 +238,7 @@ static void snd_mtpav_input_trigger(snd_rawmidi_t * rmidi, int up)
 		mtp_card->mode[port] |= MTPAV_MODE_INPUT_TRIGGERED;
 	else
 		mtp_card->mode[port] &= ~MTPAV_MODE_INPUT_TRIGGERED;
+
 	spin_unlock_irqrestore(&mtp_card->spinlock, flags);
 
 }
@@ -294,92 +264,43 @@ static int snd_mtpav_output_close(snd_rawmidi_t * rmidi)
 	U32 port = (U32) rmidi->private_data;
 
 	spin_lock_irqsave(&mtp_card->spinlock, flags);
-	mtp_card->mode[port] &= (~MTPAV_MODE_OUTPUT_OPENED);
+	mtp_card->mode[port] &= (~MTPAV_MODE_INPUT_OPENED);
 	spin_unlock_irqrestore(&mtp_card->spinlock, flags);
 	return 0;
 };
 
 //////////////////////////////////////////////////////////////////
 
-static void snd_mtpav_output_trigger(snd_rawmidi_t * rmidi, int up)
-{
-	U32 flags;
-	U32 port = (U32) rmidi->private_data;
+static U32 mtp_inmidistate;
+static U8 mtp_incable;
 
-	spin_lock_irqsave(&mtp_card->spinlock, flags);
-	if (up) {
-#if 0	/* FIXME */
-		if ((mtp_card->mode[port] & MTPAV_MODE_OUTPUT_TRIGGERED) == 0) {
-			if (!mtp_card->istimer) {
-				mtp_card->timer.function = snd_mtpav_output_timer;
-				mtp_card->timer.data = (unsigned long) mtp_card;
-				mtp_card->timer.expires = 1 + jiffies;
-				add_timer(&mtp_card->timer);
-			}
-			mtp_card->istimer++;
-		}
-#endif
-		mtp_card->mode[port] |= MTPAV_MODE_OUTPUT_TRIGGERED;
-	} else {
-		mtp_card->mode[port] &= ~MTPAV_MODE_OUTPUT_TRIGGERED;
-	}
-	spin_unlock_irqrestore(&mtp_card->spinlock, flags);
-
-	if (up)
-		snd_mtpav_output_write(rmidi);
-}
-
-//////////////////////////////////////////////////////////////////
-
-static void snd_mtpav_inmidi_h(TSmtp * mcrd, U8 inbyte)
+static void snd_mtpav_inmidi_h(U8 inbyte)
 {
 	snd_rawmidi_t *rmidi;
 	//char ibyte = inbyte;
-	int tesdev;
 
-	if (mcrd == NULL)
-		return;
-
-	if (inbyte == 0xfe)
-		return;
-
-	if (mcrd->inmidistate == 0)	// awaiting command
-	 {
+	if (mtp_inmidistate == 0)	// awaiting command
+	{
 		if (inbyte == 0xf5)	// MTP port #
-		 {
-			mcrd->inmidistate = 1;
-
+		{
+			mtp_inmidistate = 1;
 		} else {
-
-			if (mcrd->inmidiport >= NUMPORTS)
-				return;
-
-			rmidi = mcrd->rmidi[mcrd->inmidiport];
-
-			if (rmidi == NULL)
-				return;
-
-			tesdev = rmidi->device;
-
+			rmidi = mtp_card->rmidi[mtp_incable];
 			if (
-				   (mcrd->
-			mode[mcrd->inmidiport] & MTPAV_MODE_INPUT_OPENED)
-				   != 0) {
-				snd_rawmidi_receive(rmidi, &inbyte, 1);
-				//printk( "rmidi: 0x%x tesdev: %d inbyte: 0x%x\n", rmidi, tesdev, inbyte );
-			}
+			    (mtp_card->
+			     mode[mtp_incable] & MTPAV_MODE_INPUT_OPENED)
+			    != 0) snd_rawmidi_receive(rmidi, &inbyte, 1);
 		}
-	} else if (mcrd->inmidistate == 1) {
-		mcrd->inmidiport = inbyte;
-		if (mcrd->inmidiport > (NUMPORTS - 1))
-			mcrd->inmidiport = (NUMPORTS - 1);	// limit input ports
-
-		mcrd->inmidistate = 0;
-		//snd_printk("cable %d\n", (int) mcrd->inmidiport);
+	} else if (mtp_inmidistate == 1) {
+		mtp_incable = inbyte;
+		if (mtp_incable > 8)
+			mtp_incable = 8;	// limit input ports
+		mtp_inmidistate = 0;
+		snd_printk("cable %d\n", mtp_incable);
 	}
 }
 
-static void snd_mtpav_read_bytes(TSmtp * mcrd)
+void snd_mtpav_read_bytes(void)
 {
 	U8 clrread, setread;
 	U8 mtp_read_byte;
@@ -388,16 +309,12 @@ static void snd_mtpav_read_bytes(TSmtp * mcrd)
 
 	U8 sbyt = snd_mtpav_getreg(SREG);
 
-	//printk("snd_mtpav_read_bytes() sbyt: 0x%x\n", sbyt);
+	printk("snd_mtpav_read_bytes() sbyt: 0x%x\n", sbyt);
 
-	if (!(sbyt & SIGS_BYTE))
-		return;
-
-	cbyt = snd_mtpav_getreg(CREG);
-	clrread = cbyt & (SIGC_READ ^ 0xff);
-	setread = cbyt | SIGC_READ;
-
-	do {
+	while (sbyt & SIGS_BYTE) {
+		cbyt = snd_mtpav_getreg(CREG);
+		clrread = cbyt & (SIGC_READ ^ 0xff);
+		setread = cbyt | SIGC_READ;
 
 		snd_mtpav_mputreg(CREG, setread);
 		read_sr0 = snd_mtpav_getreg(SREG);
@@ -423,37 +340,36 @@ static void snd_mtpav_read_bytes(TSmtp * mcrd)
 		mtp_read_byte |=
 		    (((read_sr3 & (SIGS_IN0 | SIGS_IN1)) / 16) << 6);
 
-		snd_mtpav_inmidi_h(mcrd, mtp_read_byte);
+		snd_mtpav_inmidi_h(mtp_read_byte);
 
 		sbyt = snd_mtpav_getreg(SREG);
-
-	} while (sbyt & SIGS_BYTE);
+	}
 }
 
 static void snd_mtpav_irqh(int irq, void *dev_id, struct pt_regs *regs)
 {
 	U32 flags = 0;
-	TSmtp *mcard = (TSmtp *) dev_id;
+	TSmtp *mcard = (TSmtp *)dev_id;
 
-	//printk("irqh()\n");
+	printk("irqh()\n");
 	spin_lock_irqsave(&mcard->spinlock, flags);
-	snd_mtpav_read_bytes(mcard);
+	snd_mtpav_read_bytes();
 	spin_unlock_irqrestore(&mcard->spinlock, flags);
 }
 
 /////////////////////////////////////////////////////////////////
 // get ISA resources
 
-static int snd_mtpav_get_ISA(TSmtp * mcard)
+static int snd_mtpav_get_ISA(TSmtp *mcard)
 {
-	static long possible_irqs[] = { -1 };
+	int possible_irqs[] = { -1 };
 
 	if (snd_register_ioport(mcard->card, snd_port, 3,
 				"MotuMTPAV MIDI", &mcard->rioport) < 0) {
 		snd_printk("MTVAP port 0x%x is busy\n", snd_port);
 		return -EBUSY;
 	}
-	//snd_printk("snd_mtpav_register_resources requesting irq=%d\n", snd_irq);
+	snd_printk("snd_mtpav_register_resources requesting irq=%d\n", snd_irq);
 	if (snd_register_interrupt(mcard->card, "MOTU MTPAV", snd_irq,
 				   SND_IRQ_TYPE_ISA, snd_mtpav_irqh,
 				   mcard, possible_irqs,
@@ -465,17 +381,18 @@ static int snd_mtpav_get_ISA(TSmtp * mcard)
 /////////////////////////////////////////////////////////////////
 // get RAWMIDI resources
 
-static int snd_mtpav_get_RAWMIDI(TSmtp * mcard)
+
+static int snd_mtpav_get_RAWMIDI(TSmtp *mcard)
 {
 	U32 i;
 	int rval = 0;
 	snd_rawmidi_t *rawmidi;
 	char rmidinames1[80];
 
-	//printk("entering snd_mtpav_get_RAWMIDI\n");
+	printk("entering snd_mtpav_get_RAWMIDI\n");
 
 	for (i = 0; i < NUMPORTS; i++) {
-		sprintf(rmidinames1, "MTPAV Midiport %ld", i);
+		sprintf(rmidinames1, "MTPAV Midiport %d", i);
 
 		if ((rval = snd_rawmidi_new(mcard->card, rmidinames1,
 					    i, &mcard->rmidi[i])) < 0)
@@ -485,19 +402,17 @@ static int snd_mtpav_get_RAWMIDI(TSmtp * mcard)
 		if (rawmidi == NULL)
 			return 0;
 
-		rawmidi->private_data = (void *) i;
-
 		memcpy(&rawmidi->chn[SND_RAWMIDI_CHANNEL_OUTPUT].hw,
 		       &snd_mtpav_output, sizeof(snd_mtpav_output));
 		memcpy(&rawmidi->chn[SND_RAWMIDI_CHANNEL_INPUT].hw,
 		       &snd_mtpav_input, sizeof(snd_mtpav_input));
 		rawmidi->info_flags |=
-		    SND_RAWMIDI_INFO_OUTPUT | SND_RAWMIDI_INFO_INPUT |
-		    SND_RAWMIDI_INFO_DUPLEX;
+			SND_RAWMIDI_INFO_OUTPUT | SND_RAWMIDI_INFO_INPUT |
+			SND_RAWMIDI_INFO_DUPLEX;
 		sprintf(rawmidi->name, "MotuMIDI %d", (int) i);
 	}
 
-	//printk("exiting snd_mtpav_get_RAWMIDI() \n");
+	printk("exiting snd_mtpav_get_RAWMIDI() \n");
 	return 0;
 }
 
@@ -508,16 +423,12 @@ static TSmtp *new_TSmtp(void)
 	U32 i;
 	TSmtp *ncrd = (TSmtp *) snd_kcalloc(sizeof(TSmtp), GFP_KERNEL);
 	if (ncrd != NULL) {
-		spin_lock_init(&ncrd->spinlock);
-
+		ncrd->spinlock = SPIN_LOCK_UNLOCKED;
+	
 		ncrd->card = NULL;
 		ncrd->rioport = NULL;
 		ncrd->irq = NULL;
 
-		ncrd->inmidiport = 0xffffffff;
-		ncrd->inmidistate = 0;
-		ncrd->outmidiport = 0xffffffff;
-
 		for (i = 0; i < NUMPORTS; i++) {
 			ncrd->rmidi[i] = NULL;
 			ncrd->mode[i] = 0;
@@ -536,17 +447,15 @@ static void free_TSmtp(TSmtp * crd)
 
 /////////////////////////////////////////////////////////////////
 
-static int __init alsa_card_mtpav_init(void)
+int init_module(void)
 {
 	int err = 0;
 	char longname_buffer[80];
 
 	mtp_card = NULL;
 
-	if (NUMPORTS > SND_RAWMIDI_DEVICES) {
-		snd_printk("snd-card-mtpav ERROR: NUMPORTS in alsa-driver/include/mtpav.h (%d) is > SND_RAWMIDI_DEVICES in alsa-driver/include/rawmidi.h (%d)!\n", NUMPORTS, SND_RAWMIDI_DEVICES);
-		return -1;
-	}
+	snd_printk("init_module() for card-mtpav.c\n");
+
 	mtp_card = new_TSmtp();
 	if (mtp_card == NULL)
 		return -ENOMEM;
@@ -566,25 +475,24 @@ static int __init alsa_card_mtpav_init(void)
 	sprintf(longname_buffer, "mtpav on parallel port at");
 
 	err = snd_mtpav_get_ISA(mtp_card);
-	//printk("snd_mtpav_get_ISA returned: %d\n", err);
+	printk("snd_mtpav_get_ISA returned: %d\n", err);
 	if (err < 0)
 		goto __error;
 
 	err = snd_mtpav_get_RAWMIDI(mtp_card);
-	//snd_printk("snd_mtapv_get_RAWMIDI returned: %d\n", err);
+	snd_printk("snd_mtapv_get_RAWMIDI returned: %d\n", err);
 	if (err < 0)
 		goto __error;
 
 	err = snd_card_register(mtp_card->card);	// dont snd_card_register until AFTER all cards reources done!
-
-	//printk("snd_card_register returned %d\n", err);
+	printk("snd_card_register returned %d\n", err);
 	if (err < 0)
 		goto __error;
 
-
-	snd_mtpav_portscan();
-
-	printk("Motu MidiTimePiece on parallel port irq: %d ioport: 0x%x\n", snd_irq, snd_port);
+	mtp_inmidistate = 0;
+	mtp_incable = 0;
+	inmsg_lastport = 0xff;
+	outmsg_lastport = 0xff;
 
 	return 0;
 
@@ -596,7 +504,7 @@ static int __init alsa_card_mtpav_init(void)
 
 /////////////////////////////////////////////////////////////////
 
-static void __exit alsa_card_mtpav_exit(void)
+void __exit cleanup_module(void)
 {
 	if (mtp_card == NULL)
 		return;
@@ -618,8 +526,3 @@ static void snd_mtpav_use_dec(snd_card_t * card)
 {
 	MOD_DEC_USE_COUNT;
 }
-
-/////////////////////////////////////////////////////////////////
-
-module_init(alsa_card_mtpav_init)
-module_exit(alsa_card_mtpav_exit)
