@@ -1,6 +1,7 @@
+
 /*
     card-opti92x-ad1848.c - driver for OPTi 82c92x based soundcards.
-    Copyright (C) 1998-2000 by Massimo Piccioni <dafastidio@libero.it>
+    Copyright (C) 1998-99 by Massimo Piccioni <dafastidio@libero.it>
 
     Part of this code was developed at the Italian Ministry of Air Defence,
     Sixth Division (oh, che pace ...), Rome.
@@ -39,28 +40,26 @@
 #include "../include/mpu401.h"
 #include "../include/opl3.h"
 
-EXPORT_NO_SYMBOLS;
 #ifdef OPTi93X
 MODULE_DESCRIPTION("\
 Driver: OPTi93X\n\
 ISAPNP: OPT0931=OPT9310,OPT0002\n\
 ");
-#else	/* OPTi93X */
+#else
 #ifdef CS4231
 MODULE_DESCRIPTION("\
 Driver: OPTi92X - CS4231\n\
 ISAPNP: OPT0924=OPT0000,OPT0002\n\
 ISAPNP: OPT0925=OPT9250,OPT0002\n\
 ");
-#else	/* CS4231 */
+#else
 MODULE_DESCRIPTION("\
 Driver: OPTi92X - AD1848\n\
 ISAPNP: OPT0924=OPT0000,OPT0002\n\
 ISAPNP: OPT0925=OPT9250,OPT0002\n\
 ");
-#endif	/* CS4231 */
-#endif	/* OPTi93X */
-MODULE_LICENSE("GPL");
+#endif
+#endif
 
 int snd_index = SND_DEFAULT_IDX1;		/* Index 0-MAX */
 char *snd_id = SND_DEFAULT_STR1;		/* ID for this card */
@@ -75,38 +74,28 @@ int snd_dma1_size = SND_DEFAULT_DMA_SIZE1;	/* 8,16,32,64,128 */
 int snd_dma2 = SND_DEFAULT_DMA1;		/* 0,1,3 */
 int snd_dma2_size = SND_DEFAULT_DMA_SIZE1;	/* 8,16,32,64,128 */
 #endif	/* CS4231 || OPTi93X */
-#ifdef __ISAPNP__
-int snd_isapnp = 1;
-#endif
 
 MODULE_PARM(snd_index, "i");
 MODULE_PARM_DESC(snd_index, "Index value for opti9xx based soundcard.");
 MODULE_PARM(snd_id, "s");
 MODULE_PARM_DESC(snd_id, "ID string for opti9xx based soundcard.");
 MODULE_PARM(snd_port, "i");
-MODULE_PARM_DESC(snd_port, "WSS port # for opti9xx driver. [list=0x530,0xe80,0xf40,0x604]");
+MODULE_PARM_DESC(snd_port, "WSS port # for opti9xx driver.");
 MODULE_PARM(snd_mpu_port, "i");
-MODULE_PARM_DESC(snd_mpu_port, "MPU-401 port # for opti9xx driver. [list=0x300,0x310,0x320,0x330]");
+MODULE_PARM_DESC(snd_mpu_port, "MPU-401 port # for opti9xx driver.");
 MODULE_PARM(snd_fm_port, "i");
-MODULE_PARM_DESC(snd_fm_port, "FM port # for opti9xx driver. [list=0x388]");
+MODULE_PARM_DESC(snd_fm_port, "FM port # for opti9xx driver.");
 MODULE_PARM(snd_irq, "i");
-MODULE_PARM_DESC(snd_irq, "WSS irq # for opti9xx driver. [IRQ]");
+MODULE_PARM_DESC(snd_irq, "WSS irq # for opti9xx driver.");
 MODULE_PARM(snd_mpu_irq, "i");
-MODULE_PARM_DESC(snd_mpu_irq, "MPU-401 irq # for opti9xx driver. [IRQ]");
+MODULE_PARM_DESC(snd_mpu_irq, "MPU-401 irq # for opti9xx driver.");
 MODULE_PARM(snd_dma1, "i");
-MODULE_PARM_DESC(snd_dma1, "1st dma # for opti9xx driver. [DMA8]");
-MODULE_PARM(snd_dma1_size, "i");
-MODULE_PARM_DESC(snd_dma1_size, "Size of 1st dma for opti9xx driver. [DMA8_SIZE]");
+MODULE_PARM_DESC(snd_dma1, "1st dma # for opti9xx driver.");
 #if defined(CS4231) || defined(OPTi93X)
 MODULE_PARM(snd_dma2, "i");
-MODULE_PARM_DESC(snd_dma2, "2nd dma # for opti9xx driver. [DMA8]");
-MODULE_PARM(snd_dma2_size, "i");
-MODULE_PARM_DESC(snd_dma2_size, "Size of 2nd dma for opti9xx driver. [DMA8_SIZE]");
+MODULE_PARM_DESC(_DESCsnd_dma2, "2nd dma # for opti9xx driver.");
 #endif	/* CS4231 || OPTi93X */
-#ifdef __ISAPNP__
-MODULE_PARM(snd_isapnp, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_isapnp, "ISA PnP detection for opti9xx driver. [BOOL]");
-#endif
+
 
 static struct snd_card_opti9xx {
 	snd_irq_t *irqptr;
@@ -132,15 +121,18 @@ static unsigned int snd_card_opti9xx_pnpids[] = {
 #ifndef OPTi93X
 	/* OPTi 82C924 */
 	(ISAPNP_VENDOR('O','P','T')<<16|ISAPNP_DEVICE(0x0924)),    /* device */
+	OPTi9XX_HW_82C924,
 	(ISAPNP_VENDOR('O','P','T')<<16|ISAPNP_FUNCTION(0x0000)),  /* audio */
 	(ISAPNP_VENDOR('O','P','T')<<16|ISAPNP_FUNCTION(0x0002)),  /* mpu401 */
 	/* OPTi 82C925 */
 	(ISAPNP_VENDOR('O','P','T')<<16|ISAPNP_DEVICE(0x0925)),    /* device */
+	OPTi9XX_HW_82C925,
 	(ISAPNP_VENDOR('O','P','T')<<16|ISAPNP_FUNCTION(0x9250)),  /* audio */
 	(ISAPNP_VENDOR('O','P','T')<<16|ISAPNP_FUNCTION(0x0002)),  /* mpu401 */
 #else
 	/* OPTi 82C931/3 */
 	(ISAPNP_VENDOR('O','P','T')<<16|ISAPNP_DEVICE(0x0931)),    /* device */
+	OPTi9XX_HW_82C931,
 	(ISAPNP_VENDOR('O','P','T')<<16|ISAPNP_FUNCTION(0x9310)),  /* audio */
 	(ISAPNP_VENDOR('O','P','T')<<16|ISAPNP_FUNCTION(0x0002)),  /* mpu401 */
 #endif	/* OPTi93X */
@@ -154,7 +146,7 @@ static unsigned int snd_card_opti9xx_pnpids[] = {
 #define DRIVER_NAME	"snd-card-opti92x"
 #endif	/* OPTi93X */
 
-#define snd_card_opti9xx_printk(format, args...) snd_printk(__FILE__": " format, ##args)
+#define snd_card_opti9xx_printk(args...)	snd_printk(__FILE__": " ##args)
 
 
 static void snd_card_opti9xx_use_inc(snd_card_t *card)
@@ -223,12 +215,9 @@ static opti9xx_t * __init snd_card_opti9xx_detect(snd_card_t *card)
 		}
 
 		value = snd_opti9xx_read(chip, OPTi9XX_MC_REG(1));
-		if (value != 0xff) {
-			if (value != inb(chip->mc_base + 1))
-				snd_printk("warning: opti92x: port didn't close after read\n");
+		if ((value != 0xff) && (value != inb(chip->mc_base + 1)))
 			if (value == snd_opti9xx_read(chip, OPTi9XX_MC_REG(1)))
 				break;
-		}
 
 		snd_unregister_ioports(card);
 		snd_kfree(chip);
@@ -288,20 +277,19 @@ __again:
 	while ((tmp = snd_card_opti9xx_pnpids[idx]) != 0) {
 		if ((card = isapnp_find_card(tmp >> 16, tmp & 0xffff, card)))
 			break;
-		idx += 3;
+		idx += 4;
 	}
 	if (card == NULL)
 		return -ENODEV;
 
-	hw = tmp & 0xffff;
-
-	tmp = snd_card_opti9xx_pnpids[idx+1];
+	hw = snd_card_opti9xx_pnpids[idx+1];
+	tmp = snd_card_opti9xx_pnpids[idx+2];
 	pdev = isapnp_find_dev(card, tmp >> 16, tmp & 0xffff, NULL);
 	if (pdev == NULL)
 		goto __again;
 
 	acard->dev = pdev;
-	tmp = snd_card_opti9xx_pnpids[idx+2];
+	tmp = snd_card_opti9xx_pnpids[idx+3];
 	acard->devmpu = isapnp_find_dev(card, tmp >> 16, tmp & 0xffff, NULL);
 
 	pdev = acard->dev;
@@ -312,7 +300,7 @@ __again:
 	if (snd_port != SND_AUTO_PORT)
 		isapnp_resource_change(&pdev->resource[0], snd_port + 4, 4);
 #else
-	if ((hw != ISAPNP_DEVICE(0x0924)) && (snd_port != SND_AUTO_PORT))
+	if ((hw != OPTi9XX_HW_82C924) && (snd_port != SND_AUTO_PORT))
 		isapnp_resource_change(&pdev->resource[1], snd_port, 4);
 #endif	/* OPTi93X */
 	if (snd_irq != SND_AUTO_IRQ)
@@ -335,7 +323,7 @@ __again:
 	snd_port = pdev->resource[0].start - 4;
 	snd_fm_port = pdev->resource[1].start;
 #else
-	if (hw != ISAPNP_DEVICE(0x0924))
+	if (hw != OPTi9XX_HW_82C924)
 		snd_port = pdev->resource[1].start;
 	snd_fm_port = pdev->resource[2].start;
 #endif	/* OPTi93X */
@@ -381,18 +369,18 @@ static int __init snd_card_opti9xx_resources(struct snd_card_opti9xx *acard,
 					     snd_card_t *card)
 {
 	int error, i, pnp = 0;
-	static long no_alternatives[] = {-1};
-	static long possible_ports[] = {0x530, 0xe80, 0xf40, 0x604, -1};
-	static long possible_mpu_ports[] = {0x300, 0x310, 0x320, 0x330, -1};
+	static int no_alternatives[] = {-1};
+	static int possible_ports[] = {0x530, 0xe80, 0xf40, 0x604, -1};
+	static int possible_mpu_ports[] = {0x300, 0x310, 0x320, 0x310, -1};
 #ifdef OPTi93X
-	static long possible_irqs[] = {5, 9, 10, 11, 7, -1};
+	static int possible_irqs[] = {5, 9, 10, 11, 7, -1};
 #else
-	static long possible_irqs[] = {9, 10, 11, 7, -1};
+	static int possible_irqs[] = {9, 10, 11, 7, -1};
 #endif	/* OPTi93X */
-	static long possible_mpu_irqs[] = {5, 9, 10, 7, -1};
-	static long possible_dma1s[] = {3, 1, 0, -1};
+	static int possible_mpu_irqs[] = {5, 9, 10, 7, -1};
+	static int possible_dma1s[] = {3, 1, 0, -1};
 #if defined(CS4231) || defined(OPTi93X)
-	static long possible_dma2s[][2] = {{1,-1}, {0,-1}, {-1,-1}, {0,-1}};
+	static int possible_dma2s[][2] = {{1,-1}, {0,-1}, {-1,-1}, {0,-1}};
 #endif	/* CS4231 || OPTi93X */
 
 #ifdef __ISAPNP__
@@ -508,22 +496,7 @@ static int __init snd_card_opti9xx_probe(struct snd_card_opti9xx *acard)
 #endif	/* OPTi93X */
 
 #ifdef __ISAPNP__
-	if (snd_isapnp && (hw = snd_card_opti9xx_isapnp(acard)) > 0) {
-		switch (hw) {
-		case ISAPNP_DEVICE(0x0924):
-			hw = OPTi9XX_HW_82C924;
-			break;
-		case ISAPNP_DEVICE(0x0925):
-			hw = OPTi9XX_HW_82C925;
-			break;
-		case ISAPNP_DEVICE(0x0931):
-			hw = OPTi9XX_HW_82C931;
-			break;
-		default:
-			error = -ENODEV;
-			goto __on_error;
-		}
-
+	if ((hw = snd_card_opti9xx_isapnp(acard)) >= 0) {
 		if (!(chip = snd_opti9xx_new_device(hw))) {
 			error = -ENOMEM;
 			goto __on_error;
@@ -577,7 +550,7 @@ static int __init snd_card_opti9xx_probe(struct snd_card_opti9xx *acard)
 		if ((error = snd_mpu401_uart_new(card, 0, MPU401_HW_MPU401,
 				acard->chip->mpu_port, acard->chip->mpu_irq,
 				&rmidi)))
-			snd_card_opti9xx_printk("no MPU-401 device at 0x%lx?\n",
+			snd_card_opti9xx_printk("no MPU-401 device at 0x%x?\n",
 				acard->chip->mpu_port);
 
 	if (acard->chip->fm_port <= 0)
@@ -585,22 +558,18 @@ static int __init snd_card_opti9xx_probe(struct snd_card_opti9xx *acard)
 	else
 		if ((error = snd_opl3_new(card, 0,
 				acard->chip->fm_port, acard->chip->fm_port + 2,
-#ifdef CS4231
-				OPL3_HW_AUTO, 1, &synth)))
-#else
 				OPL3_HW_AUTO, 0, &synth)))
-#endif	/* CS4231 */
-			snd_card_opti9xx_printk("no OPL device at 0x%lx-0x%lx?\n",
+			snd_card_opti9xx_printk("no OPL device at 0x%x-0x%x?\n",
 				acard->chip->fm_port, acard->chip->fm_port + 2);
 
 	strcpy(card->abbreviation, acard->chip->name);
 	sprintf(card->shortname, "OPTi %s", card->abbreviation);
 #if defined(CS4231) || defined(OPTi93X)
-	sprintf(card->longname, "%s soundcard, %s at 0x%lx, irq %li, dma %li&%li",
+	sprintf(card->longname, "%s soundcard, %s at 0x%x, irq %i, dma %i&%i",
 		card->shortname, pcm->name, acard->chip->wss_base + 4,
 		acard->irqptr->irq, acard->dma1ptr->dma, acard->dma2ptr->dma);
 #else
-	sprintf(card->longname, "%s soundcard, %s at 0x%lx, irq %li, dma %li",
+	sprintf(card->longname, "%s soundcard, %s at 0x%x, irq %i, dma %i",
 		card->shortname, pcm->name, acard->chip->wss_base + 4,
 		acard->irqptr->irq, acard->dma1ptr->dma);
 #endif	/* CS4231 || OPTi93X */
@@ -635,7 +604,17 @@ static void snd_card_opti9xx_free(struct snd_card_opti9xx *acard)
 	snd_kfree(acard);
 }
 
-static int __init alsa_card_opti9xx_init(void)
+#ifdef MODULE
+int __init init_module(void)
+#else
+#ifdef OPTi93X
+int __init alsa_card_opti93x_init(void)
+#elif defined(CS4231)
+int __init alsa_card_opti92x_cs4231_init(void)
+#else
+int __init alsa_card_opti92x_ad1848_init(void)
+#endif
+#endif
 {
 	int error;
 	struct snd_card_opti9xx *acard;
@@ -662,10 +641,11 @@ static int __init alsa_card_opti9xx_init(void)
 	return 0;
 }
 
-static void __exit alsa_card_opti9xx_exit(void)
+#ifdef MODULE
+
+void __exit cleanup_module(void)
 {
 	snd_card_opti9xx_free(snd_card_opti9xx_card);
 }
 
-module_init(alsa_card_opti9xx_init)
-module_exit(alsa_card_opti9xx_exit)
+#endif
