@@ -29,23 +29,14 @@
 #include "../include/emu8000.h"
 #include "../include/seq_device.h"
 
-#ifndef CARD_NAME
-#define CARD_NAME	"Sound Blaster 16"
-#define DRIVER_NAME	"SB16"
-#endif
-
-EXPORT_NO_SYMBOLS;
 #ifndef SND_SBAWE
 MODULE_DESCRIPTION("\
 Driver: Sound Blaster 16\n\
-ISAPNP: CTL0024=CTL0031\n\
+ISAPNP: CTL0024=CTL0041\n\
 ISAPNP: CTL0026=CTL0031\n\
 ISAPNP: CTL0027=CTL0031\n\
-ISAPNP: CTL0028=CTL0031\n\
 ISAPNP: CTL0029=CTL0031\n\
-ISAPNP: CTL002a=CTL0031\n\
 ISAPNP: CTL002b=CTL0031\n\
-ISAPNP: CTL002c=CTL0031\n\
 ISAPNP: CTL0051=CTL0001\n\
 ISAPNP: CTL0070=CTL0001\n\
 ISAPNP: CTL0080=CTL0041\n\
@@ -56,31 +47,22 @@ MODULE_DESCRIPTION("\
 Driver: Sound Blaster AWE\n\
 Card: Sound Blaster AWE 32\n\
 Card: Sound Blaster AWE 64\n\
-ISAPNP: CTL0035=CTL0031,CTL0021\n\
 ISAPNP: CTL0039=CTL0031,CTL0021\n\
 ISAPNP: CTL0042=CTL0031,CTL0021\n\
 ISAPNP: CTL0043=CTL0031,CTL0021\n\
 ISAPNP: CTL0044=CTL0031,CTL0021\n\
-ISAPNP: CTL0045=CTL0031,CTL0021\n\
-ISAPNP: CTL0047=CTL0031,CTL0021\n\
 ISAPNP: CTL0048=CTL0031,CTL0021\n\
 ISAPNP: CTL0054=CTL0031,CTL0021\n\
-ISAPNP: CTL009a=CTL0041,CTL0021\n\
 ISAPNP: CTL009c=CTL0041,CTL0021\n\
 ISAPNP: CTL009d=CTL0042,CTL0022\n\
 ISAPNP: CTL009e=CTL0044,CTL0023\n\
-ISAPNP: CTL009f=CTL0041,CTL0021\n\
-ISAPNP: CTL00b2=CTL0044,CTL0023\n\
 ISAPNP: CTL00c1=CTL0042,CTL0022\n\
 ISAPNP: CTL00c3=CTL0045,CTL0022\n\
 ISAPNP: CTL00c5=CTL0045,CTL0022\n\
 ISAPNP: CTL00c7=CTL0045,CTL0022\n\
 ISAPNP: CTL00e4=CTL0045,CTL0022\n\
-ISAPNP: CTL00e9=CTL0045,CTL0022\n\
-ISAPNP: CTL00ed=CTL0041,CTL0070\n\
 ");
 #endif
-MODULE_LICENSE("GPL");
 
 #if 0
 #define SND_DEBUG_IRQ
@@ -94,7 +76,7 @@ int snd_index[SND_CARDS] = SND_DEFAULT_IDX;	/* Index 0-MAX */
 char *snd_id[SND_CARDS] = SND_DEFAULT_STR;	/* ID for this card */
 int snd_port[SND_CARDS] = SND_DEFAULT_PORT;	/* 0x220,0x240,0x260 */
 int snd_mpu_port[SND_CARDS] = {0x330, 0x300,[2 ... (SND_CARDS - 1)] = -1};
-int snd_fm_port[SND_CARDS] = {0x388, [1 ... (SND_CARDS - 1)] = -1};
+int snd_fm_port[SND_CARDS] = SND_DEFAULT_PORT;	/* optional, not used */
 #ifdef SND_SBAWE_EMU8000
 int snd_awe_port[SND_CARDS] = {0x620,[1 ... (SND_CARDS - 1)] = -1};
 #endif
@@ -110,46 +92,39 @@ int snd_csp[SND_CARDS] = {[0 ... (SND_CARDS - 1)] = 0};
 #ifdef __ISAPNP__
 int snd_isapnp[SND_CARDS] = {[0 ... (SND_CARDS - 1)] = 1};
 #endif
-#ifdef SND_SBAWE_EMU8000
-int snd_seq_ports[SND_CARDS] = {[0 ... (SND_CARDS - 1)] = 4};
-#endif
 MODULE_PARM(snd_index, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_index, "Index value for " CARD_NAME " soundcard.");
+MODULE_PARM_DESC(snd_index, "Index value for SoundBlaster 16 soundcard.");
 MODULE_PARM(snd_id, "1-" __MODULE_STRING(SND_CARDS) "s");
-MODULE_PARM_DESC(snd_id, "ID string for " CARD_NAME " soundcard.");
+MODULE_PARM_DESC(snd_id, "ID string for SoundBlaster 16 soundcard.");
 MODULE_PARM(snd_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_port, "Port # for " DRIVER_NAME " driver. [list=0x220,0x240,0x260]");
+MODULE_PARM_DESC(snd_port, "Port # for SB16 driver. [list=0x220,0x240,0x260]");
 MODULE_PARM(snd_mpu_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_mpu_port, "MPU-401 port # for " DRIVER_NAME " driver. [list=0x330,0x300]");
+MODULE_PARM_DESC(snd_mpu_port, "MPU-401 port # for SB16 driver. [list=0x330,0x300]");
 MODULE_PARM(snd_fm_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_fm_port, "FM port # for " DRIVER_NAME " driver. [list=0x388]");
+MODULE_PARM_DESC(snd_fm_port, "FM port # for SB16 driver. [list=0x388]");
 #ifdef SND_SBAWE_EMU8000
 MODULE_PARM(snd_awe_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_awe_port, "AWE port # for " DRIVER_NAME " driver. [list=0x620]");
+MODULE_PARM_DESC(snd_awe_port, "AWE port # for SB16 driver. [list=0x620]");
 #endif
 MODULE_PARM(snd_irq, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_irq, "IRQ # for " DRIVER_NAME " driver. [IRQ]");
+MODULE_PARM_DESC(snd_irq, "IRQ # for SB16 driver. [IRQ]");
 MODULE_PARM(snd_dma8, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma8, "8-bit DMA # for " DRIVER_NAME " driver. [DMA8]");
+MODULE_PARM_DESC(snd_dma8, "8-bit DMA # for SB16 driver. [DMA8]");
 MODULE_PARM(snd_dma16, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma16, "16-bit DMA # for " DRIVER_NAME " driver. [DMA16]");
+MODULE_PARM_DESC(snd_dma16, "16-bit DMA # for SB16 driver. [DMA16]");
 MODULE_PARM(snd_dma8_size, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma8_size, "Size of 8-bit DMA # for " DRIVER_NAME " driver. [DMA8_SIZE]");
+MODULE_PARM_DESC(snd_dma8_size, "Size of 8-bit DMA # for SB16 driver. [DMA8_SIZE]");
 MODULE_PARM(snd_dma16_size, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma16_size, "Size of 16-bit DMA # for " DRIVER_NAME " driver. [DMA16_SIZE]");
+MODULE_PARM_DESC(snd_dma16_size, "Size of 16-bit DMA # for SB16 driver. [DMA16_SIZE]");
 MODULE_PARM(snd_mic_agc, "1-" __MODULE_STRING(SND_CARDS) "i");
 MODULE_PARM_DESC(snd_mic_agc, "Mic Auto-Gain-Control switch. [BOOL]");
 #ifdef CONFIG_SND_SB16_CSP
 MODULE_PARM(snd_csp, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_csp, "ASP/CSP chip support. [BOOL]");
+MODULE_PARM_DESC(snd_csp, "CSP chip support. [BOOL]");
 #endif
 #ifdef __ISAPNP__
 MODULE_PARM(snd_isapnp, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_isapnp, "ISA PnP detection for " CARD_NAME " soundcard. [BOOL]");
-#endif
-#ifdef SND_SBAWE_EMU8000
-MODULE_PARM(snd_seq_ports, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_seq_ports, "Number of sequencer ports for WaveTable synth. [range=2-32]");
+MODULE_PARM_DESC(snd_isapnp, "ISA PnP detection for specified soundcard. [BOOL]");
 #endif
 
 static struct snd_sb16 {
@@ -188,21 +163,12 @@ static unsigned int snd_sb16_pnpids[] = {
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_DEVICE(0x0027),   /* DEVICE */
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0031), /* Audio */
 	/* Sound Blaster 16 PnP */
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_DEVICE(0x0028),   /* DEVICE */
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0031), /* Audio */
-	/* Sound Blaster 16 PnP */
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_DEVICE(0x0029),   /* DEVICE */
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0031), /* Audio */
 	/* Sound Blaster 16 PnP */
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_DEVICE(0x002a),   /* DEVICE */
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0031), /* Audio */
-	/* Sound Blaster 16 PnP */
 	/* Note: This card also have a CTL0051:StereoEnhance device!!! */
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_DEVICE(0x002b),   /* DEVICE */
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0031), /* Audio */
-	/* Sound Blaster 16 PnP */
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_DEVICE(0x002c),   /* DEVICE */
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0031), /* Audio */
 	/* Sound Blaster Vibra16S */
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_DEVICE(0x0051),   /* DEVICE */
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0001), /* Audio */
@@ -215,12 +181,6 @@ static unsigned int snd_sb16_pnpids[] = {
 	/* Sound Blaster Vibra16X */
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_DEVICE(0x00f0),   /* DEVICE */
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0043), /* Audio */
-#endif
-	/* Sound Blaster AWE 32 PnP */
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_DEVICE(0x0035),   /* DEVICE */
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0031), /* Audio */
-#ifdef SND_SBAWE_EMU8000
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0021), /* WaveTable */
 #endif
 	/* Sound Blaster AWE 32 PnP */
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_DEVICE(0x0039),   /* DEVICE */
@@ -246,19 +206,6 @@ static unsigned int snd_sb16_pnpids[] = {
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0031), /* Audio */
 #ifdef SND_SBAWE_EMU8000
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0021), /* WaveTable */
-#endif
-	/* Sound Blaster AWE 32 PnP */
-	/* Note: This card have also CTL0051:StereoEnhance device!!! */
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_DEVICE(0x0045),   /* DEVICE */
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0031), /* Audio */
-#ifdef SND_SBAWE_EMU8000
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0021), /* WaveTable */
-#endif
-	/* Sound Blaster AWE 32 PnP */
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_DEVICE(0x0047),   /* DEVICE */
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0031), /* Audio */
-#ifdef SND_SBAWE_EMU8000
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0021), /* WaveTable */
 #endif
 	/* Sound Blaster AWE 32 PnP */
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_DEVICE(0x0048),   /* DEVICE */
@@ -271,12 +218,6 @@ static unsigned int snd_sb16_pnpids[] = {
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0031), /* Audio */
 #ifdef SND_SBAWE_EMU8000
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0021), /* WaveTable */
-#endif
-	/* Sound Blaster AWE 32 PnP */
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_DEVICE(0x009a),   /* DEVICE */
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0041), /* Audio */
-#ifdef SND_SBAWE_EMU8000
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0021), /* WaveTable */
 #endif
 	/* Sound Blaster AWE 32 PnP */
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_DEVICE(0x009c),   /* DEVICE */
@@ -295,18 +236,6 @@ static unsigned int snd_sb16_pnpids[] = {
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0044), /* Audio */
 #ifdef SND_SBAWE_EMU8000
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0023), /* WaveTable */
-#endif
-	/* Sound Blaster 32 PnP */
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_DEVICE(0x009f),   /* DEVICE */
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0041), /* Audio */
-#ifdef SND_SBAWE_EMU8000
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0021), /* WaveTable */
-#endif
-	/* Sound Blaster AWE 64 PnP Gold */
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_DEVICE(0x00b2),   /* DEVICE */
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0044), /* Audio */
-#ifdef SND_SBAWE_EMU8000
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0023), /* WaveTable */
 #endif
 	/* Sound Blaster AWE 64 PnP */
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_DEVICE(0x00c1),   /* DEVICE */
@@ -337,18 +266,6 @@ static unsigned int snd_sb16_pnpids[] = {
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0045), /* Audio */
 #ifdef SND_SBAWE_EMU8000
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0022), /* WaveTable */
-#endif
-	/* Sound Blaster AWE 64 PnP */
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_DEVICE(0x00e9),   /* DEVICE */
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0045), /* Audio */
-#ifdef SND_SBAWE_EMU8000
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0022), /* WaveTable */
-#endif
-	/* Sound Blaster 16 PnP (AWE) */
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_DEVICE(0x00ed),   /* DEVICE */
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0041), /* Audio */
-#ifdef SND_SBAWE_EMU8000
-	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0070), /* WaveTable */
 #endif
 	/* --- */
 	0       /* end */
@@ -377,26 +294,26 @@ static snd_pcm_t * __init snd_sb16_detect(int dev,
 	sbdsp_t *codec;
 	int mpu_flag = 0, err;
 
-	if (snd_register_ioport(card, port, 16, CARD_NAME, NULL) < 0)
+	if (snd_register_ioport(card, port, 16, "Sound Blaster 16", NULL) < 0)
 		return NULL;
 	if (snd_mpu_port[dev] == 0x300 || snd_mpu_port[dev] == 0x330) {
 		if (snd_register_ioport(card, snd_mpu_port[dev], 2,
-					CARD_NAME " - MPU-401", NULL) < 0) {
+					"Sound Blaster 16 - MPU-401", NULL) < 0) {
 			snd_printk("sb16: port 0x%x isn't free for MPU-401, midi port is disabled\n", snd_mpu_port[dev]);
 		} else {
 			mpu_flag = 1;
 		}
 	}
 	if (snd_fm_port[dev] >= 0x300 && snd_fm_port[dev] < 0x400)
-		snd_register_ioport(card, snd_fm_port[dev], 4, CARD_NAME " - FM", NULL);
+		snd_register_ioport(card, snd_fm_port[dev], 4, "Sound Blaster 16 - FM", NULL);
 #ifdef SND_SBAWE_EMU8000
 	if (snd_awe_port[dev] >= 0x620 && snd_awe_port[dev] < 0x6a0) {
 		if (snd_register_ioport(card, snd_awe_port[dev],
-					4, CARD_NAME " - WaveTable", NULL) < 0 ||
+					4, "Sound Blaster AWE", NULL) < 0 ||
 		    snd_register_ioport(card, snd_awe_port[dev] + (0xa40 - 0x640),
-		    			4, CARD_NAME " - WaveTable", NULL) < 0 ||
+		    			4, "Sound Blaster AWE", NULL) < 0 ||
 		    snd_register_ioport(card, snd_awe_port[dev] + (0xe40 - 0x640),
-		    			4, CARD_NAME " - WaveTable", NULL) < 0) {
+		    			4, "Sound Blaster AWE", NULL) < 0) {
 			snd_printk("sb16: EMU8000 ports aren't available\n");
 			snd_unregister_ioports(card);
 			return NULL;
@@ -463,20 +380,20 @@ static void snd_sb16_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 static int __init snd_sb16_resources(int dev, struct snd_sb16 *acard,
 				     snd_card_t * card)
 {
-	static long possible_irqs[] = {5, 9, 10, 7, -1};
-	static long possible_dmas8[] = {1, 3, 0, -1};
-	static long possible_dmas16[] = {5, 6, 7, -1};
+	static int possible_irqs[] = {5, 9, 10, 7, -1};
+	static int possible_dmas8[] = {1, 3, 0, -1};
+	static int possible_dmas16[] = {5, 6, 7, -1};
 	int err, ok = 0;
 
 	if ((err = snd_register_interrupt(card,
-			CARD_NAME, snd_irq[dev],
+			"Sound Blaster 16", snd_irq[dev],
 			SND_IRQ_TYPE_ISA, snd_sb16_interrupt,
 			acard, possible_irqs, &acard->irqptr)) < 0)
 		return err;
 	if ((snd_dma8[dev] >= 0 && snd_dma8[dev] <= 3) ||
 	    snd_dma8[dev] == SND_AUTO_DMA) {
 		if ((err = snd_register_dma_channel(card,
-				CARD_NAME, snd_dma8[dev],
+				"Sound Blaster 16", snd_dma8[dev],
 				SND_DMA_TYPE_ISA, snd_dma8_size[dev],
 				possible_dmas8, &acard->dma8ptr)) < 0)
 			return err;
@@ -487,7 +404,7 @@ static int __init snd_sb16_resources(int dev, struct snd_sb16 *acard,
 	if ((snd_dma16[dev] >= 4 && snd_dma16[dev] <= 7) ||
 	    snd_dma16[dev] == SND_AUTO_DMA) {
 		if ((err = snd_register_dma_channel(card,
-				CARD_NAME, snd_dma16[dev],
+				"Sound Blaster 16", snd_dma16[dev],
 				SND_DMA_TYPE_ISA, snd_dma16_size[dev],
 				possible_dmas16, &acard->dma16ptr)) < 0)
 			return err;
@@ -558,7 +475,7 @@ static int __init snd_sb16_isapnp(int dev, struct snd_sb16 *acard)
 	if (snd_mpu_port[dev] != SND_AUTO_PORT)
 		isapnp_resource_change(&pdev->resource[1], snd_mpu_port[dev], 2);
 	if (snd_fm_port[dev] != SND_AUTO_PORT)
-		isapnp_resource_change(&pdev->resource[2], snd_fm_port[dev], 4);
+		isapnp_resource_change(&pdev->resource[2], snd_mpu_port[dev], 4);
 	if (snd_dma8[dev] != SND_AUTO_DMA)
 		isapnp_resource_change(&pdev->dma_resource[0], snd_dma8[dev], 1);
 	if (snd_dma16[dev] != SND_AUTO_DMA)
@@ -575,7 +492,7 @@ static int __init snd_sb16_isapnp(int dev, struct snd_sb16 *acard)
 	snd_dma8[dev] = pdev->dma_resource[0].start;
 	snd_dma16[dev] = pdev->dma_resource[1].start;
 	snd_irq[dev] = pdev->irq_resource[0].start;
-	snd_printdd("isapnp SB16: port=0x%x, mpu port=0x%x, fm port=0x%x\n",
+	snd_printdd("isapnp SB16: port=0x%lx, mpu port=0x%lx, fm port=0x%lx\n",
 			snd_port[dev], snd_mpu_port[dev], snd_fm_port[dev]);
 	snd_printdd("isapnp SB16: dma1=%i, dma2=%i, irq=%i\n",
 			snd_dma8[dev], snd_dma16[dev], snd_irq[dev]);
@@ -594,14 +511,12 @@ static int __init snd_sb16_isapnp(int dev, struct snd_sb16 *acard)
 		return -EBUSY;
 	}
 	snd_awe_port[dev] = pdev->resource[0].start;
-#if 0
 	/* set other two ports which aren't listed in PnP EEPROM */
 	/* FIXME: remove for 2.4!!! */
 	isapnp_cfg_begin(pdev->bus->number, pdev->devfn);
 	isapnp_write_word(ISAPNP_CFG_PORT + (1 << 1), snd_awe_port[dev] + (0xa40 - 0x640));
 	isapnp_write_word(ISAPNP_CFG_PORT + (2 << 1), snd_awe_port[dev] + (0xe40 - 0x640));
 	isapnp_cfg_end();
-#endif
 	snd_printdd("isapnp SB16: wavetable port=0x%lx\n", pdev->resource[0].start);
 #endif
 	return 0;
@@ -620,8 +535,10 @@ static void snd_sb16_deactivate(struct snd_sb16 *acard)
 
 static int __init snd_sb16_probe(int dev, struct snd_sb16 *acard)
 {
+#ifndef __ISAPNP__
 	static int possible_ports[] = {0x220, 0x240, 0x260, -1};
 	int *ports = possible_ports;
+#endif
 	sbdsp_t *codec;
 	snd_card_t *card;
 	snd_pcm_t *pcm = NULL;
@@ -641,21 +558,17 @@ static int __init snd_sb16_probe(int dev, struct snd_sb16 *acard)
 			    snd_sb16_use_inc, snd_sb16_use_dec);
 	if (card == NULL)
 		return -ENOMEM;
+	card->static_data = acard;
 	card->type = SND_CARD_TYPE_SB_16;	/* overriden in probe function */
-#ifdef __ISAPNP__
-	if (snd_isapnp[dev] && snd_sb16_isapnp(dev, acard) < 0) {
-		snd_card_free(card);
-		return -EBUSY;
-	}
-#endif
+#ifndef __ISAPNP__
 	if (snd_sb16_resources(dev, acard, card) < 0) {
 		snd_card_free(card);
 		return -EBUSY;
 	}
+#endif
+	pcm = NULL;
+#ifndef __ISAPNP__
 	if (snd_port[dev] == SND_AUTO_PORT) {
-		/* Note: if snd_isapnp[dev]=1, snd_port[dev] is redefined
-		   always, so that this legacy auto-probe is not applied.
-		   */
 		for (ports = possible_ports; *ports >= 0; ports++) {
 			pcm = snd_sb16_detect(dev, card, *ports,
 					      acard->irqptr,
@@ -665,11 +578,23 @@ static int __init snd_sb16_probe(int dev, struct snd_sb16 *acard)
 				break;
 		}
 	} else {
+#else
+		if (snd_isapnp[dev] && snd_sb16_isapnp(dev, acard) < 0) {
+			snd_card_free(card);
+			return -EBUSY;
+		}
+		if (snd_sb16_resources(dev, acard, card) < 0) {
+			snd_card_free(card);
+			return -EBUSY;
+		}
+#endif
 		pcm = snd_sb16_detect(dev, card, snd_port[dev],
 				      acard->irqptr,
 				      acard->dma8ptr,
 				      acard->dma16ptr);
+#ifndef __ISAPNP__
 	}
+#endif
 	if (pcm == NULL) {
 		snd_card_free(card);
 		return -ENODEV;
@@ -727,7 +652,6 @@ static int __init snd_sb16_probe(int dev, struct snd_sb16 *acard)
 		awe_arg->mixer_index = 0;
 		if (mixer)
 			awe_arg->mixer_dest = codec->mixer.me_vol_synth;
-		awe_arg->seq_ports = snd_seq_ports[dev];
 	}
 #endif
 
@@ -738,17 +662,17 @@ static int __init snd_sb16_probe(int dev, struct snd_sb16 *acard)
 		(snd_mic_agc[dev] ? 0x00 : 0x01));
 	spin_unlock_irqrestore(&codec->mixer.lock, flags);
 
-	strcpy(card->abbreviation, DRIVER_NAME);
-	strcpy(card->shortname, CARD_NAME);
-	sprintf(card->longname, "%s at 0x%lx, irq %li, dma ",
+	strcpy(card->abbreviation, "SB16");
+	strcpy(card->shortname, "Sound Blaster 16");
+	sprintf(card->longname, "%s at 0x%x, irq %i, dma ",
 		codec->name,
 		codec->port,
 		acard->irqptr->irq);
 	if (acard->dma8ptr)
-		sprintf(card->longname + strlen(card->longname), "%li",
+		sprintf(card->longname + strlen(card->longname), "%i",
 			acard->dma8ptr->dma);
 	if (acard->dma16ptr)
-		sprintf(card->longname + strlen(card->longname), "%s%li",
+		sprintf(card->longname + strlen(card->longname), "%s%i",
 			acard->dma8ptr ? "&" : "",
 			acard->dma16ptr->dma);
 	if (!snd_card_register(card)) {
@@ -760,7 +684,7 @@ static int __init snd_sb16_probe(int dev, struct snd_sb16 *acard)
 #endif
 		acard->rmidi = rmidi;
 #ifdef CONFIG_SND_SEQUENCER
-		acard->synth = synth;
+		/*acard->synth = synth;*/
 #endif
 #ifdef SND_SBAWE_EMU8000
 		acard->awe = awe;
@@ -793,7 +717,15 @@ static int __exit snd_sb16_free(int dev)
 
 #endif
 
-static int __init alsa_card_sb16_init(void)
+#ifdef MODULE
+int __init init_module(void)
+#else
+#ifdef SND_SBAWE_EMU8000
+int __init alsa_card_sbawe_init(void)
+#else
+int __init alsa_card_sb16_init(void)
+#endif
+#endif
 {
 	int dev, cards;
 	struct snd_sb16 *acard;
@@ -810,7 +742,7 @@ static int __init alsa_card_sb16_init(void)
 			if (snd_port[dev] == SND_AUTO_PORT)
 				break;
 #ifdef MODULE
-			snd_printk(CARD_NAME " soundcard #%i not found at 0x%x or device busy\n", dev + 1, snd_port[dev]);
+			snd_printk("Sound Blaster 16 soundcard #%i not found at 0x%x or device busy\n", dev + 1, snd_port[dev]);
 #endif
 			continue;
 		}
@@ -819,14 +751,16 @@ static int __init alsa_card_sb16_init(void)
 	}
 	if (!cards) {
 #ifdef MODULE
-		snd_printk(CARD_NAME " soundcard #%i not found or device busy\n", dev + 1);
+		snd_printk("Sound Blaster 16 soundcard #%i not found or device busy\n", dev + 1);
 #endif
 		return -ENODEV;
 	}
 	return 0;
 }
 
-static void __exit alsa_card_sb16_exit(void)
+#ifdef MODULE
+
+void __exit cleanup_module(void)
 {
 	int dev;
 
@@ -834,5 +768,4 @@ static void __exit alsa_card_sb16_exit(void)
 		snd_sb16_free(dev);
 }
 
-module_init(alsa_card_sb16_init)
-module_exit(alsa_card_sb16_exit)
+#endif
