@@ -52,7 +52,7 @@
 #define SND_CARD_TYPE_ESS_ES1938	0x00000014	/* ESS Solo-1 ES1938 */
 #define SND_CARD_TYPE_ESS_ES18XX	0x00000015	/* ESS AudioDrive ES18XX */
 #define SND_CARD_TYPE_CS4231		0x00000016      /* CS4231 */
-#define SND_CARD_TYPE_OPTI9XX		0x00000017	/* Opti 9xx chipset */
+#define SND_CARD_TYPE_OPTI92X		0x00000017	/* OPTi 92x chipset */
 #define SND_CARD_TYPE_SERIAL		0x00000018	/* Serial MIDI driver */
 #define SND_CARD_TYPE_AD1848		0x00000019	/* Generic AD1848 driver */
 #define SND_CARD_TYPE_TRID4DWAVEDX	0x0000001A	/* Trident 4DWave DX */
@@ -65,9 +65,18 @@
 #define SND_CARD_TYPE_TROPEZPLUS        0x00000020      /* TB Tropez+ */
 #define SND_CARD_TYPE_MAUI              0x00000021      /* TB Maui */
 #define SND_CARD_TYPE_CMI8330           0x00000022      /* C-Media CMI8330 */
-/* --- */
+/* Various */
 #define SND_CARD_TYPE_DUMMY		0x00000023	/* dummy soundcard */
+#define SND_CARD_TYPE_ALS100		0x00000024	/* Avance Logic ALS100 */
+#define SND_CARD_TYPE_SHARE		0x00000025	/* share soundcard */
+#define SND_CARD_TYPE_SI_7018		0x00000026	/* SiS 7018 */
+#define SND_CARD_TYPE_OPTI93X		0x00000027	/* OPTi 93x chipset */
+#define SND_CARD_TYPE_MTPAV		0x00000028	/* MOTU MidiTimePiece AV multiport MIDI */
+#define SND_CARD_TYPE_VIRMIDI		0x00000029	/* Virtual MIDI */
+#define SND_CARD_TYPE_EMU10K1		0x0000002a	/* EMU10K1 */
+#define SND_CARD_TYPE_HAMMERFALL	0x0000002b	/* RME Digi9652  */
+#define SND_CARD_TYPE_HAMMERFALL_LIGHT	0x0000002c	/* RME Digi9652, but no expansion card */
 /* --- */
-#define SND_CARD_TYPE_LAST		0x00000023
+#define SND_CARD_TYPE_LAST		0x0000002c
 
 #endif				/* __ASOUNDID_H */
