@@ -22,9 +22,9 @@
  *
  */
 
-#include "pcm1.h"
+#include "pcm.h"
 #include "mixer.h"
-#include "midi.h"
+#include "rawmidi.h"
 #include "ak4531_codec.h"
 #include "ac97_codec.h"
 
@@ -37,6 +37,29 @@
 #ifndef PCI_DEVICE_ID_ENSONIQ_ES1371
 #define PCI_DEVICE_ID_ENSONIQ_ES1371    0x1371
 #endif
+#ifndef PCI_DEVICE_ID_ENSONIQ_CT5880
+#define PCI_DEVICE_ID_ENSONIQ_CT5880	0x5880
+#endif
+
+#ifndef PCI_VENDOR_ID_ECTIVA
+#define PCI_VENDOR_ID_ECTIVA		0x1102
+#endif
+#ifndef PCI_DEVICE_ID_ECTIVA_EV1938
+#define PCI_DEVICE_ID_ECTIVA_EV1938	0x8938
+#endif
+
+/* ES1371 chip ID */
+/* This is a little confusing because all ES1371 compatible chips have the
+   same DEVICE_ID, the only thing differentiating them is the REV_ID field.
+   This is only significant if you want to enable features on the later parts.
+   Yes, I know it's stupid and why didn't we use the sub IDs?
+*/
+#define ES1371REV_ES1373_A  0x04
+#define ES1371REV_ES1373_B  0x06
+#define ES1371REV_CT5880_A  0x07
+#define CT5880REV_CT5880_C  0x02
+#define ES1371REV_ES1371_B  0x09
+#define EV1938REV_EV1938_A  0x00
 
 /*
  * Direct registers
@@ -47,6 +70,7 @@
 #define ES_REG_CONTROL	0x00	/* R/W: Interrupt/Chip select control register */
 #define   ES_1370_ADC_STOP	(1<<31)		/* disable capture buffer transfers */
 #define   ES_1370_XCTL1 	(1<<30)		/* general purpose output bit */
+#define   ES_1371_SPDIF_EN	(1<<26)		/* SPDIF enable */
 #define   ES_1371_JOY_ASEL(o)	(((o)&0x03)<<24)	/* joystick port mapping */
 #define   ES_1371_JOY_ASELM	(0x03<<24)	/* mask for above */
 #define   ES_1371_JOY_ASELI(i)  (((i)>>24)&0x03)
@@ -81,8 +105,12 @@
 #define   ES_1370_SERR_DISABLE	(1<<0)	/* PCI serr signal disable */
 #define   ES_1371_PCICLKDIS     (1<<0)	/* PCI clock disable */
 #define ES_REG_STATUS	0x04	/* R/O: Interrupt/Chip select status register */
-#define   ES_INTR               (1<<31)		/* Interrupt is pending */
-#define   ES_1370_CSTAT		(1<<10)		/* CODEC is busy or register write in progress */
+#define   ES_INTR               (1<<31)	/* Interrupt is pending */
+#define   ES_1371_ST_AC97_RST	(1<<29) /* CT5880 AC'97 Reset bit */
+#define   ES_1371_ST_SPDIF_EN	(1<<18) /* SPDIF enable */
+#define   ES_1371_ST_SPDIF_TEST (1<<17) /* SPDIF test */
+#define   ES_1371_TEST          (1<<16) /* test ASIC */
+#define   ES_1370_CSTAT		(1<<10)	/* CODEC is busy or register write in progress */
 #define   ES_1370_CBUSY         (1<<9)	/* CODEC is busy */
 #define   ES_1370_CWRIP		(1<<8)	/* CODEC register write in progress */
 #define   ES_1371_SYNC_ERR	(1<<8)	/* CODEC synchronization error occured */
@@ -185,10 +213,10 @@
 #define   ES_R1_MODEI(i)	(((i)>>4)&0x03)
 #define   ES_P2_MODEO(o)	(((o)&0x03)<<2)		/* DAC2 mode; -- '' -- */
 #define   ES_P2_MODEM		(0x03<<2)	/* mask for above */
-#define   ES_P2_MODEI(i)	(((i)>>4)&0x03)
+#define   ES_P2_MODEI(i)	(((i)>>2)&0x03)
 #define   ES_P1_MODEO(o)	(((o)&0x03)<<0)		/* DAC1 mode; -- '' -- */
 #define   ES_P1_MODEM		(0x03<<0)	/* mask for above */
-#define   ES_P1_MODEI(i)	(((i)>>4)&0x03)
+#define   ES_P1_MODEI(i)	(((i)>>0)&0x03)
 
 #define ES_REG_DAC1_COUNT 0x24	/* R/W: DAC1 sample count register */
 #define ES_REG_DAC2_COUNT 0x28	/* R/W: DAC2 sample count register */
@@ -206,7 +234,7 @@
 #define ES_REG_ADC_SIZE	  0x34	/* R/W: PAGE 0x0d; ADC frame size */
 #define   ES_REG_FCURR_COUNTO(o) (((o)&0xffff)<<16)
 #define   ES_REG_FCURR_COUNTM    (0xffff<<16)
-#define   ES_REG_FCURR_COUNTI(i) (((i)>>16)&0xffff)
+#define   ES_REG_FCURR_COUNTI(i) (((i)>>14)&0x3fffc)
 #define   ES_REG_FSIZEO(o)	 (((o)&0xffff)<<0)
 #define   ES_REG_FSIZEM		 (0xffff<<0)
 #define   ES_REG_FSIZEI(i)	 (((i)>>0)&0xffff)
@@ -280,7 +308,9 @@ struct snd_stru_ensoniq {
 
 	unsigned int ctrl;	/* control register */
 	unsigned int sctrl;	/* serial control register */
+	unsigned int cssr;	/* control status register */
 	unsigned int uartc;	/* uart control register */
+	unsigned int rev;	/* chip revision */
 
 	union {
 		struct {
@@ -297,11 +327,14 @@ struct snd_stru_ensoniq {
 	snd_pcm_t *pcm;		/* DAC1/ADC PCM */
 	snd_pcm_t *pcm2;	/* DAC2 PCM */
 	snd_pcm_subchn_t *playback_subchn;
-	snd_pcm1_subchn_t *playback_subchn1;
 	snd_pcm_subchn_t *playback2_subchn;
-	snd_pcm1_subchn_t *playback2_subchn1;
 	snd_pcm_subchn_t *capture_subchn;
-	snd_pcm1_subchn_t *capture_subchn1;
+	unsigned int p1_dma_size;
+	unsigned int p2_dma_size;
+	unsigned int c_dma_size;
+	unsigned int p1_frag_size;
+	unsigned int p2_frag_size;
+	unsigned int c_frag_size;
 	snd_kmixer_t *mixer;
 	snd_rawmidi_t *rmidi;
 
@@ -309,30 +342,34 @@ struct snd_stru_ensoniq {
 	snd_info_entry_t *proc_entry;
 };
 
-ensoniq_t *snd_ensoniq_1370_create(snd_card_t * card, struct pci_dev *pci,
-			      	   snd_dma_t * dma1ptr,
-				   snd_dma_t * dma2ptr,
-			           snd_dma_t * dma3ptr,
-			           snd_irq_t * irqptr);
-void snd_ensoniq_1370_free(ensoniq_t * ensoniq);
+int snd_ensoniq_1370_create(snd_card_t * card,
+			    struct pci_dev *pci,
+			    snd_dma_t * dma1ptr,
+			    snd_dma_t * dma2ptr,
+			    snd_dma_t * dma3ptr,
+			    snd_irq_t * irqptr,
+			    ensoniq_t ** rensoniq);
+int snd_ensoniq_1370_free(ensoniq_t * ensoniq);
 void snd_ensoniq_1370_interrupt(ensoniq_t * ensoniq);
 
-snd_pcm_t *snd_ensoniq_1370_pcm(ensoniq_t * ensoniq);
-snd_pcm_t *snd_ensoniq_1370_pcm2(ensoniq_t * ensoniq);
-snd_kmixer_t *snd_ensoniq_1370_mixer(ensoniq_t * ensoniq, int pcm_count, int *pcm_devs);
-snd_rawmidi_t *snd_ensoniq_1370_midi(ensoniq_t * ensoniq);
+int snd_ensoniq_1370_pcm(ensoniq_t * ensoniq, int device, snd_pcm_t ** rpcm);
+int snd_ensoniq_1370_pcm2(ensoniq_t * ensoniq, int device, snd_pcm_t ** rpcm);
+int snd_ensoniq_1370_mixer(ensoniq_t * ensoniq, int device, int pcm_count, int *pcm_devs, snd_kmixer_t ** rmixer);
+int snd_ensoniq_1370_midi(ensoniq_t * ensoniq, int device, snd_rawmidi_t ** rrawmidi);
 
-ensoniq_t *snd_ensoniq_1371_create(snd_card_t * card, struct pci_dev *pci,
-			      	   snd_dma_t * dma1ptr,
-				   snd_dma_t * dma2ptr,
-			           snd_dma_t * dma3ptr,
-			           snd_irq_t * irqptr);
-void snd_ensoniq_1371_free(ensoniq_t * ensoniq);
+int snd_ensoniq_1371_create(snd_card_t * card,
+			    struct pci_dev *pci,
+			    snd_dma_t * dma1ptr,
+			    snd_dma_t * dma2ptr,
+			    snd_dma_t * dma3ptr,
+			    snd_irq_t * irqptr,
+			    ensoniq_t ** rensoniq);
+int snd_ensoniq_1371_free(ensoniq_t * ensoniq);
 void snd_ensoniq_1371_interrupt(ensoniq_t * ensoniq);
 
-snd_pcm_t *snd_ensoniq_1371_pcm(ensoniq_t * ensoniq);
-snd_pcm_t *snd_ensoniq_1371_pcm2(ensoniq_t * ensoniq);
-snd_kmixer_t *snd_ensoniq_1371_mixer(ensoniq_t * ensoniq, int pcm_count, int *pcm_devs);
-snd_rawmidi_t *snd_ensoniq_1371_midi(ensoniq_t * ensoniq);
+int snd_ensoniq_1371_pcm(ensoniq_t * ensoniq, int device, snd_pcm_t ** rpcm);
+int snd_ensoniq_1371_pcm2(ensoniq_t * ensoniq, int device, snd_pcm_t ** rpcm);
+int snd_ensoniq_1371_mixer(ensoniq_t * ensoniq, int device, int pcm_count, int *pcm_devs, snd_kmixer_t ** rmixer);
+int snd_ensoniq_1371_midi(ensoniq_t * ensoniq, int device, snd_rawmidi_t ** rrawmidi);
 
 #endif				/* __AUDIOPCI_H */
