@@ -22,7 +22,7 @@
  *
  */
 
-#include "pcm1.h"
+#include "pcm.h"
 #include "mixer.h"
 
 #define ES1688_HW_AUTO		0x0000
@@ -42,17 +42,37 @@ struct snd_stru_es1688 {
 	unsigned short hardware;	/* see to ES1688_HW_XXXX */
 
 	unsigned short trigger_value;
+	unsigned char rec_src;
+	unsigned char pad;
+	unsigned int dma_size;
+
+	snd_card_t *card;
+	snd_pcm_t *pcm;
+	snd_pcm_subchn_t *playback_subchn;
+	snd_pcm_subchn_t *capture_subchn;
+	snd_kmixer_t *mixer;
+
+	spinlock_t reg_lock;
+	spinlock_t mixer_lock;
 
 	snd_kmixer_element_t *me_mux_mic;
 	snd_kmixer_element_t *me_mux_line;
 	snd_kmixer_element_t *me_mux_cd;
 	snd_kmixer_element_t *me_mux_mix;
 
-	snd_card_t *card;
-	snd_pcm_t *pcm;
-
-	snd_spin_define(reg);
-	snd_spin_define(mixer);
+	snd_kmixer_element_t *me_mux;
+	snd_kmixer_element_t *me_accu;
+	snd_kmixer_element_t *me_vol_igain;
+	snd_kmixer_element_t *me_capture;
+	snd_kmixer_element_t *me_playback;
+	snd_kmixer_element_t *me_vol_master;
+	snd_kmixer_element_t *me_vol_line;
+	snd_kmixer_element_t *me_vol_pcm;
+	snd_kmixer_element_t *me_vol_cd;
+	snd_kmixer_element_t *me_vol_fm;
+	snd_kmixer_element_t *me_vol_mic;
+	snd_kmixer_element_t *me_vol_aux;
+	snd_kmixer_element_t *me_vol_speaker;
 };
 
 typedef struct snd_stru_es1688 es1688_t;
@@ -109,19 +129,20 @@ typedef struct snd_stru_es1688 es1688_t;
 
  */
 
-extern void snd_es1688_mixer_write(es1688_t * codec, unsigned char reg, unsigned char data);
-extern unsigned char snd_es1688_mixer_read(es1688_t * codec, unsigned char reg);
+void snd_es1688_mixer_write(es1688_t * codec, unsigned char reg, unsigned char data);
+unsigned char snd_es1688_mixer_read(es1688_t * codec, unsigned char reg);
 
-extern void snd_es1688_interrupt(snd_pcm_t * pcm);
+void snd_es1688_interrupt(snd_pcm_t * pcm);
 
-extern snd_pcm_t *snd_es1688_new_device(snd_card_t * card,
-					unsigned short port,
-					unsigned short mpu_port,
-					snd_irq_t * irqptr,
-					snd_irq_t * mpu_irqptr,
-					snd_dma_t * dma8ptr,
-					unsigned short hardware);
-extern int snd_es1688_init(snd_pcm_t * pcm, int enable);
-extern snd_kmixer_t *snd_es1688_new_mixer(snd_pcm_t * pcm, int pcm_dev);
+int snd_es1688_new_pcm(snd_card_t * card, int device,
+		       unsigned short port,
+		       unsigned short mpu_port,
+		       snd_irq_t * irqptr,
+		       snd_irq_t * mpu_irqptr,
+		       snd_dma_t * dma8ptr,
+		       unsigned short hardware,
+		       snd_pcm_t ** rpcm);
+int snd_es1688_init(snd_pcm_t * pcm, int enable);
+int snd_es1688_new_mixer(snd_pcm_t * pcm, int device, snd_kmixer_t ** rmixer);
 
 #endif				/* __ES1688_H */
