@@ -100,7 +100,7 @@ struct snd_stru_fm801 {
 	snd_dma_t * dma2ptr;	/* ADC */
 	snd_irq_t * irqptr;
 
-	unsigned long port;	/* I/O port number */
+	unsigned short port;	/* I/O port number */
 
 	unsigned short ply_ctrl; /* playback control */
 	unsigned short cap_ctrl; /* capture control */
