@@ -61,13 +61,8 @@ struct snd_info_entry_data {
 		       struct file * file, const char *buf, long count);
 	long long (*lseek) (void *private_data, void *file_private_data,
 			    struct file * file, long long offset, int orig);
-#ifdef SND_POLL
 	unsigned int (*poll) (void *private_data, void *file_private_data,
 			      struct file * file, poll_table * wait);
-#else
-	int (*select) (void *private_data, void *file_private_data,
-		       struct file * file, int sel_type, select_table * wait);
-#endif
 	int (*ioctl) (void *private_data, void *file_private_data,
 		      struct file * file, unsigned int cmd, unsigned long arg);
 	int (*mmap) (void *private_data, void *file_private_data,
@@ -98,7 +93,7 @@ struct snd_info_entry {
 	snd_card_t *card;
 	struct proc_dir_entry *p;
 	struct snd_info_entry *next;
-	snd_mutex_define(access);
+	struct semaphore access;
 };
 
 extern int snd_info_check_reserved_words(const char *str);
@@ -145,8 +140,6 @@ static inline snd_info_entry_t *snd_info_create_device(const char *name,
 						       unsigned int number,
 						       unsigned int mode) { return NULL; }
 static inline void snd_info_free_device(snd_info_entry_t * entry) { ; }
-static inline int snd_info_store_text(snd_info_entry_t * entry) { return 0; }
-static inline int snd_info_restore_text(snd_info_entry_t * entry) { return 0; }
 
 static inline int snd_info_card_register(snd_card_t * card) { return 0; }
 static inline int snd_info_card_unregister(snd_card_t * card) { return 0; }
@@ -170,11 +163,7 @@ static inline int snd_info_unregister(snd_info_entry_t * entry) { return 0; }
 
 #define SND_OSS_INFO_DEV_COUNT	6
 
-#ifndef LINUX_2_2
 extern int snd_oss_info_register(int dev, int num, char *string);
-#else
-static inline int snd_oss_info_register(int dev, int num, char *string) { return 0; }
-#endif
 #define snd_oss_info_unregister(dev, num) snd_oss_info_register(dev, num, NULL)
 
 #endif				/* CONFIG_SND_OSSEMUL */
