@@ -22,13 +22,19 @@
 #include "../include/driver.h"
 #include "../include/minors.h"
 #include "../include/info.h"
-#include "../include/sndpersist.h"
 #include "../include/version.h"
 #include <stdarg.h>
 #ifdef CONFIG_DEVFS_FS
 #include <linux/devfs_fs_kernel.h>
 #endif
 
+#if LinuxVersionCode(2, 3, 27) <= LINUX_VERSION_CODE
+#if LinuxVersionCode(2, 3, 29) > LINUX_VERSION_CODE
+#error "Please, upgrade to 2.3.29..."
+#endif
+#define NO_FILL_ROUTINE
+#endif
+
 /*
 
  */
@@ -70,12 +76,12 @@ extern struct file_operations snd_fops;
 
 static DECLARE_MUTEX(info_mutex);
 
-struct snd_info_private_data {
+typedef struct snd_stru_info_private_data {
 	snd_info_buffer_t *rbuffer;
 	snd_info_buffer_t *wbuffer;
 	snd_info_entry_t *entry;
 	void *file_private_data;
-};
+} snd_info_private_data_t;
 
 static snd_info_entry_t *snd_info_entries;
 static snd_info_entry_t *snd_info_devices;
@@ -118,6 +124,7 @@ struct proc_dir_entry *snd_proc_dev = NULL;
 #endif
 struct proc_dir_entry *snd_proc_seq = NULL;
 
+#ifndef NO_FILL_ROUTINE
 static void snd_info_fill_inode(struct inode *inode, int fill)
 {
 	if (fill)
@@ -126,6 +133,31 @@ static void snd_info_fill_inode(struct inode *inode, int fill)
 		MOD_DEC_USE_COUNT;
 }
 
+static inline void snd_info_entry_prepare(struct proc_dir_entry *de)
+{
+	de->fill_inode = snd_info_fill_inode;
+}
+
+static inline void snd_info_entry_remove(struct proc_dir_entry *parent,
+					 struct proc_dir_entry *de)
+{
+	if (parent && de)
+		proc_unregister(parent, de->low_ino);
+}
+#else
+static inline void snd_info_entry_prepare(struct proc_dir_entry *de)
+{
+	de->owner = THIS_MODULE;
+}
+
+static inline void snd_info_entry_remove(struct proc_dir_entry *parent,
+					 struct proc_dir_entry *de)
+{
+	if (de)
+		remove_proc_entry(de->name, parent);
+}
+#endif
+
 static snd_info_entry_t *snd_info_find_entry(unsigned short ino)
 {
 	int tmp;
@@ -133,19 +165,19 @@ static snd_info_entry_t *snd_info_find_entry(unsigned short ino)
 	snd_info_entry_t *entry;
 
 	for (entry = snd_info_entries; entry; entry = entry->next) {
-		if (!entry->p)
+		if (entry->p == NULL)
 			continue;
 		if (entry->p->low_ino == ino)
 			break;
 	}
-	if (!entry) {
-		for (tmp = 0; tmp < SND_CARDS && !entry; tmp++) {
+	if (entry == NULL) {
+		for (tmp = 0; tmp < SND_CARDS && entry == NULL; tmp++) {
 			card = snd_cards[tmp];
-			if (!card)
+			if (card == NULL)
 				continue;
 			for (entry = card->info_entries; entry;
 							entry = entry->next) {
-				if (!entry->p)
+				if (entry->p == NULL)
 					continue;
 				if (entry->p->low_ino == ino)
 					break;
@@ -162,7 +194,7 @@ static snd_card_t *snd_info_find_card(unsigned short ino)
 
 	for (tmp = 0; tmp < SND_CARDS; tmp++) {
 		card = snd_cards[tmp];
-		if (!card || !card->proc_dir_link)
+		if (card == NULL || card->proc_dir_link == NULL)
 			continue;
 		if (card->proc_dir_link->low_ino == ino)
 			return card;
@@ -172,12 +204,10 @@ static snd_card_t *snd_info_find_card(unsigned short ino)
 
 static loff_t snd_info_entry_lseek(struct file *file, loff_t offset, int orig)
 {
-	struct snd_info_private_data *data;
+	snd_info_private_data_t *data;
 	struct snd_info_entry *entry;
 
-	data = (struct snd_info_private_data *) file->private_data;
-	if (!data)
-		return -EIO;
+	data = snd_magic_cast(snd_info_private_data_t, file->private_data, -ENXIO);
 	entry = data->entry;
 	switch (entry->type) {
 	case SND_INFO_ENTRY_TEXT:
@@ -206,19 +236,18 @@ static loff_t snd_info_entry_lseek(struct file *file, loff_t offset, int orig)
 static ssize_t snd_info_entry_read(struct file *file, char *buffer,
 				   size_t count, loff_t * offset)
 {
-	struct snd_info_private_data *data;
+	snd_info_private_data_t *data;
 	struct snd_info_entry *entry;
 	snd_info_buffer_t *buf;
 	long size = 0, size1;
 
-	data = (struct snd_info_private_data *) file->private_data;
-	if (!data)
-		return -EIO;
+	data = snd_magic_cast(snd_info_private_data_t, file->private_data, -ENXIO);
+	snd_debug_check(data == NULL, -ENXIO);
 	entry = data->entry;
 	switch (entry->type) {
 	case SND_INFO_ENTRY_TEXT:
 		buf = data->rbuffer;
-		if (!buf)
+		if (buf == NULL)
 			return -EIO;
 		if (file->f_pos >= buf->size)
 			return 0;
@@ -245,19 +274,18 @@ static ssize_t snd_info_entry_read(struct file *file, char *buffer,
 static ssize_t snd_info_entry_write(struct file *file, const char *buffer,
 				    size_t count, loff_t * offset)
 {
-	struct snd_info_private_data *data;
+	snd_info_private_data_t *data;
 	struct snd_info_entry *entry;
 	snd_info_buffer_t *buf;
 	long size = 0, size1;
 
-	data = (struct snd_info_private_data *) file->private_data;
-	if (!data)
-		return -EIO;
+	data = snd_magic_cast(snd_info_private_data_t, file->private_data, -ENXIO);
+	snd_debug_check(data == NULL, -ENXIO);
 	entry = data->entry;
 	switch (entry->type) {
 	case SND_INFO_ENTRY_TEXT:
 		buf = data->wbuffer;
-		if (!buf)
+		if (buf == NULL)
 			return -EIO;
 		if (file->f_pos < 0)
 			return -EINVAL;
@@ -288,13 +316,13 @@ static ssize_t snd_info_entry_write(struct file *file, const char *buffer,
 static int snd_info_entry_open(struct inode *inode, struct file *file)
 {
 	snd_info_entry_t *entry;
-	struct snd_info_private_data *data;
+	snd_info_private_data_t *data;
 	snd_info_buffer_t *buffer;
 	int mode, err;
 
 	down(&info_mutex);
 	entry = snd_info_find_entry(inode->i_ino);
-	if (!entry) {
+	if (entry == NULL) {
 		up(&info_mutex);
 		return -ENODEV;
 	}
@@ -303,7 +331,7 @@ static int snd_info_entry_open(struct inode *inode, struct file *file)
 		if ((entry->type == SND_INFO_ENTRY_TEXT &&
 					!entry->t.text.read_size) ||
 		    (entry->type == SND_INFO_ENTRY_DATA &&
-		    			!entry->t.data.read) ||
+		    			entry->t.data.read == NULL) ||
 		    entry->type == SND_INFO_ENTRY_DEVICE) {
 			up(&info_mutex);
 			return -ENODEV;
@@ -313,15 +341,14 @@ static int snd_info_entry_open(struct inode *inode, struct file *file)
 		if ((entry->type == SND_INFO_ENTRY_TEXT &&
 					!entry->t.text.write_size) ||
 		    (entry->type == SND_INFO_ENTRY_DATA &&
-		    			!entry->t.data.write) ||
+		    			entry->t.data.write == NULL) ||
 		    entry->type == SND_INFO_ENTRY_DEVICE) {
 			up(&info_mutex);
 			return -ENODEV;
 		}
 	}
-	data = (struct snd_info_private_data *)
-			snd_kcalloc(sizeof(struct snd_info_private_data), GFP_KERNEL);
-	if (!data) {
+	data = snd_magic_kcalloc(snd_info_private_data_t, 0, GFP_KERNEL);
+	if (data == NULL) {
 		up(&info_mutex);
 		return -ENOMEM;
 	}
@@ -331,17 +358,17 @@ static int snd_info_entry_open(struct inode *inode, struct file *file)
 		if (mode == O_RDONLY || mode == O_RDWR) {
 			buffer = (snd_info_buffer_t *)
 				 	snd_kcalloc(sizeof(snd_info_buffer_t), GFP_KERNEL);
-			if (!buffer) {
-				snd_kfree(data);
+			if (buffer == NULL) {
+				snd_magic_kfree(data);
 				up(&info_mutex);
 				return -ENOMEM;
 			}
 			buffer->len = (entry->t.text.read_size +
 				      (PAGE_SIZE - 1)) & ~(PAGE_SIZE - 1);
 			buffer->buffer = snd_vmalloc(buffer->len);
-			if (!buffer->buffer) {
+			if (buffer->buffer == NULL) {
 				snd_kfree(buffer);
-				snd_kfree(data);
+				snd_magic_kfree(data);
 				up(&info_mutex);
 				return -ENOMEM;
 			}
@@ -351,25 +378,25 @@ static int snd_info_entry_open(struct inode *inode, struct file *file)
 		if (mode == O_WRONLY || mode == O_RDWR) {
 			buffer = (snd_info_buffer_t *)
 					snd_kcalloc(sizeof(snd_info_buffer_t), GFP_KERNEL);
-			if (!buffer) {
+			if (buffer == NULL) {
 				if (mode == O_RDWR) {
 					snd_vfree(data->rbuffer->buffer);
 					snd_kfree(data->rbuffer);
 				}
-				snd_kfree(data);
+				snd_magic_kfree(data);
 				up(&info_mutex);
 				return -ENOMEM;
 			}
 			buffer->len = (entry->t.text.write_size +
 				      (PAGE_SIZE - 1)) & ~(PAGE_SIZE - 1);
 			buffer->buffer = snd_vmalloc(buffer->len);
-			if (!buffer->buffer) {
+			if (buffer->buffer == NULL) {
 				if (mode == O_RDWR) {
 					snd_vfree(data->rbuffer->buffer);
 					snd_kfree(data->rbuffer);
 				}
 				snd_kfree(buffer);
-				snd_kfree(data);
+				snd_magic_kfree(data);
 				up(&info_mutex);
 				return -ENOMEM;
 			}
@@ -382,7 +409,7 @@ static int snd_info_entry_open(struct inode *inode, struct file *file)
 			if ((err = entry->t.data.open(entry->private_data,
 						      entry, mode,
 						      &data->file_private_data)) < 0) {
-				snd_kfree(data);
+				snd_magic_kfree(data);
 				up(&info_mutex);
 				return err;
 			}
@@ -408,14 +435,11 @@ static int snd_info_entry_open(struct inode *inode, struct file *file)
 static int snd_info_entry_release(struct inode *inode, struct file *file)
 {
 	snd_info_entry_t *entry;
-	struct snd_info_private_data *data;
+	snd_info_private_data_t *data;
 	int mode;
 
 	mode = file->f_flags & O_ACCMODE;
-	if ((data = (struct snd_info_private_data *) file->private_data) == NULL) {
-		MOD_DEC_USE_COUNT;
-		return -EINVAL;
-	}
+	data = snd_magic_cast(snd_info_private_data_t, file->private_data, -ENXIO);
 	entry = data->entry;
 	switch (entry->type) {
 	case SND_INFO_ENTRY_TEXT:
@@ -448,18 +472,18 @@ static int snd_info_entry_release(struct inode *inode, struct file *file)
 	if (entry->use_dec)
 		entry->use_dec(entry->card);
 	MOD_DEC_USE_COUNT;
-	snd_kfree(data);
+	snd_magic_kfree(data);
 	return 0;
 }
 
 static unsigned int snd_info_entry_poll(struct file *file, poll_table * wait)
 {
-	struct snd_info_private_data *data;
+	snd_info_private_data_t *data;
 	struct snd_info_entry *entry;
 	unsigned int mask;
 
-	data = (struct snd_info_private_data *) file->private_data;
-	if (!data)
+	data = snd_magic_cast(snd_info_private_data_t, file->private_data, -ENXIO);
+	if (data == NULL)
 		return 0;
 	entry = data->entry;
 	mask = 0;
@@ -481,11 +505,11 @@ static unsigned int snd_info_entry_poll(struct file *file, poll_table * wait)
 static int snd_info_entry_ioctl(struct inode *inode, struct file *file,
 				unsigned int cmd, unsigned long arg)
 {
-	struct snd_info_private_data *data;
+	snd_info_private_data_t *data;
 	struct snd_info_entry *entry;
 
-	data = (struct snd_info_private_data *) file->private_data;
-	if (!data)
+	data = snd_magic_cast(snd_info_private_data_t, file->private_data, -ENXIO);
+	if (data == NULL)
 		return 0;
 	entry = data->entry;
 	switch (entry->type) {
@@ -502,11 +526,11 @@ static int snd_info_entry_ioctl(struct inode *inode, struct file *file,
 static int snd_info_entry_mmap(struct file *file, struct vm_area_struct *vma)
 {
 	struct inode *inode = file->f_dentry->d_inode;
-	struct snd_info_private_data *data;
+	snd_info_private_data_t *data;
 	struct snd_info_entry *entry;
 
-	data = (struct snd_info_private_data *) file->private_data;
-	if (!data)
+	data = snd_magic_cast(snd_info_private_data_t, file->private_data, -ENXIO);
+	if (data == NULL)
 		return 0;
 	entry = data->entry;
 	switch (entry->type) {
@@ -591,9 +615,9 @@ static int snd_info_card_readlink(struct dentry *dentry,
 
 	down(&info_mutex);
 	card = snd_info_find_card(dentry->d_inode->i_ino);
-	if (!card) {
+	if (card == NULL) {
 		up(&info_mutex);
-		return -ENOENT;
+		return -ENXIO;
 	}
 	len = strlen(card->id) + 1;
 	if (buflen < len)
@@ -613,7 +637,7 @@ static struct dentry *snd_info_card_followlink(struct dentry *dentry,
 
 	down(&info_mutex);
 	card = snd_info_find_card(dentry->d_inode->i_ino);
-	if (!card || !card->proc_dir) {
+	if (card == NULL || card->proc_dir == NULL) {
 		up(&info_mutex);
 		return NULL;
 	}
@@ -659,31 +683,31 @@ struct inode_operations snd_info_card_link_inode_operations =
 
 
 
-int snd_info_init(void)
+int __init snd_info_init(void)
 {
 	struct proc_dir_entry *p;
 
 	snd_info_entries = NULL;
 	snd_info_devices = NULL;
 	p = create_proc_entry("asound", S_IFDIR | S_IRUGO | S_IXUGO, &proc_root);
-	if (!p)
+	if (p == NULL)
 		return -ENOMEM;
-	p->fill_inode = snd_info_fill_inode;
+	snd_info_entry_prepare(p);
 	snd_proc_root = p;
 #ifndef CONFIG_DEVFS_FS
 	p = create_proc_entry("dev", S_IFDIR | S_IRUGO | S_IXUGO, snd_proc_root);
-	if (!p)
+	if (p == NULL)
 		return -ENOMEM;
-	p->fill_inode = snd_info_fill_inode;
+	snd_info_entry_prepare(p);
 	snd_proc_dev = p;
 #endif
 	p = create_proc_entry("seq", S_IFDIR | S_IRUGO | S_IXUGO, snd_proc_root);
-	if (!p)
+	if (p == NULL)
 		return -ENOMEM;
-	p->fill_inode = snd_info_fill_inode;
+	snd_info_entry_prepare(p);
 	snd_proc_seq = p;
 	snd_info_version_init();
-#ifdef CONFIG_SND_MEMORY_DEBUG
+#ifdef CONFIG_SND_DEBUG_MEMORY
 	snd_memory_info_init();
 #endif
 	snd_minor_info_init();
@@ -694,14 +718,14 @@ int snd_info_init(void)
 	return 0;
 }
 
-int snd_info_done(void)
+int __exit snd_info_done(void)
 {
 	snd_card_info_done();
 #ifdef CONFIG_SND_OSSEMUL
 	snd_minor_info_oss_done();
 #endif
 	snd_minor_info_done();
-#ifdef CONFIG_SND_MEMORY_DEBUG
+#ifdef CONFIG_SND_DEBUG_MEMORY
 	snd_memory_info_done();
 #endif
 	snd_info_version_done();
@@ -714,13 +738,11 @@ int snd_info_done(void)
 	}
 #endif
 	if (snd_proc_root) {
-		if (snd_proc_seq)
-			proc_unregister(snd_proc_seq, snd_proc_seq->low_ino);
+		snd_info_entry_remove(snd_proc_root, snd_proc_seq);
 #ifndef CONFIG_DEVFS_FS
-		if (snd_proc_dev)
-			proc_unregister(snd_proc_dev, snd_proc_dev->low_ino);
+		snd_info_entry_remove(snd_proc_root, snd_proc_seq);
 #endif
-		proc_unregister(&proc_root, snd_proc_root->low_ino);
+		snd_info_entry_remove(&proc_root, snd_proc_root);
 	}
 	return 0;
 }
@@ -735,21 +757,20 @@ int snd_info_card_register(snd_card_t * card)
 	char str[4];
 	struct proc_dir_entry *p;
 
-	if (!card)
-		return -EINVAL;
+	snd_debug_check(card == NULL, -ENXIO);
 	p = create_proc_entry(card->id, S_IFDIR | S_IRUGO | S_IXUGO, snd_proc_root);
-	if (!p)
+	if (p == NULL)
 		return -ENOMEM;
-	p->fill_inode = snd_info_fill_inode;
+	snd_info_entry_prepare(p);
 	card->proc_dir = p;
 	sprintf(str, "%i", card->number);
 	p = create_proc_entry(str, S_IFLNK | S_IRUGO | S_IWUGO | S_IXUGO, snd_proc_root);
-	if (!p) {
-		proc_unregister(snd_proc_root, card->proc_dir->low_ino);
+	if (p == NULL) {
+		snd_info_entry_remove(snd_proc_root, card->proc_dir);
 		card->proc_dir = NULL;
 		return -ENOMEM;
 	}
-	p->fill_inode = snd_info_fill_inode;
+	snd_info_entry_prepare(p);
 	p->ops = &snd_info_card_link_inode_operations;
 	card->proc_dir_link = p;
 	return 0;
@@ -757,12 +778,11 @@ int snd_info_card_register(snd_card_t * card)
 
 int snd_info_card_unregister(snd_card_t * card)
 {
-	if (!card)
-		return -EINVAL;
+	snd_debug_check(card == NULL, -ENXIO);
 	if (card->proc_dir_link)
-		proc_unregister(snd_proc_root, card->proc_dir_link->low_ino);
+		snd_info_entry_remove(snd_proc_root, card->proc_dir_link);
 	if (card->proc_dir)
-		proc_unregister(snd_proc_root, card->proc_dir->low_ino);
+		snd_info_entry_remove(snd_proc_root, card->proc_dir);
 	card->proc_dir = NULL;
 	return 0;
 }
@@ -830,10 +850,10 @@ snd_info_entry_t *snd_info_create_entry(snd_card_t * card, const char *name)
 	snd_info_entry_t *entry;
 
 	entry = (snd_info_entry_t *) snd_kcalloc(sizeof(snd_info_entry_t), GFP_KERNEL);
-	if (!entry)
+	if (entry == NULL)
 		return NULL;
 	entry->name = snd_kmalloc_strdup(name, GFP_KERNEL);
-	if (!entry->name) {
+	if (entry->name == NULL) {
 		snd_kfree(entry);
 		return NULL;
 	}
@@ -850,7 +870,7 @@ snd_info_entry_t *snd_info_create_entry(snd_card_t * card, const char *name)
 
 void snd_info_free_entry(snd_info_entry_t * entry)
 {
-	if (!entry)
+	if (entry == NULL)
 		return;
 	if (entry->name)
 		snd_kfree((char *)entry->name);
@@ -860,6 +880,7 @@ void snd_info_free_entry(snd_info_entry_t * entry)
 }
 
 #ifndef CONFIG_DEVFS_FS
+#ifndef NO_FILL_ROUTINE
 static void snd_info_device_fill_inode(struct inode *inode, int fill)
 {
 	struct proc_dir_entry *de;
@@ -871,14 +892,26 @@ static void snd_info_device_fill_inode(struct inode *inode, int fill)
 	}
 	MOD_INC_USE_COUNT;
 	de = (struct proc_dir_entry *) inode->u.generic_ip;
-	if (!de)
+	if (de == NULL)
 		return;
 	entry = (snd_info_entry_t *) de->data;
-	if (!entry)
+	if (entry == NULL)
 		return;
 	inode->i_rdev = MKDEV(entry->t.device.major, entry->t.device.minor);
 }
-#endif
+
+static inline void snd_info_device_entry_prepare(struct proc_dir_entry *de, snd_info_entry_t *entry)
+{
+	de->fill_inode = snd_info_device_fill_inode;
+}
+#else
+static inline void snd_info_device_entry_prepare(struct proc_dir_entry *de, snd_info_entry_t *entry)
+{
+	de->rdev = MKDEV(entry->t.device.major, entry->t.device.minor);
+	de->owner = THIS_MODULE;
+}
+#endif /* NO_FILL_ROUTINE */
+#endif /* CONFIG_DEVFS_FS */
 
 snd_info_entry_t *snd_info_create_device(const char *name, unsigned int number, unsigned int mode)
 {
@@ -898,7 +931,7 @@ snd_info_entry_t *snd_info_create_device(const char *name, unsigned int number,
 		mode = S_IFCHR | S_IRUGO | S_IWUGO;
 	mode &= (snd_device_mode & (S_IRUGO | S_IWUGO)) | S_IFCHR | S_IFBLK;
 	entry = snd_info_create_entry(NULL, name);
-	if (!entry)
+	if (entry == NULL)
 		return NULL;
 	entry->type = SND_INFO_ENTRY_DEVICE;
 	entry->mode = mode;
@@ -907,8 +940,8 @@ snd_info_entry_t *snd_info_create_device(const char *name, unsigned int number,
 	down(&info_mutex);
 #ifndef CONFIG_DEVFS_FS
 	p = create_proc_entry(entry->name, entry->mode, snd_proc_dev);
-	p->fill_inode = snd_info_device_fill_inode;
 	if (p) {
+		snd_info_device_entry_prepare(p, entry);
 		p->ops = &snd_info_device_inode_operations;
 	} else {
 		up(&info_mutex);
@@ -932,7 +965,7 @@ snd_info_entry_t *snd_info_create_device(const char *name, unsigned int number,
 	up(&info_mutex);
 #ifdef CONFIG_DEVFS_FS
 	sprintf(dname, "snd/%s", name);
-	devfs_register(dname, 0, DEVFS_FL_TOLERANT,
+	devfs_register(NULL, dname, 0, DEVFS_FL_TOLERANT,
 			major, minor, mode, 0, 0,
 			&snd_fops, NULL);
 #endif
@@ -948,14 +981,14 @@ void snd_info_free_device(snd_info_entry_t * entry)
 	snd_info_entry_t *e;
 
 #ifndef CONFIG_DEVFS_FS
-	if (!entry || !entry->p)
+	if (entry == NULL || entry->p == NULL)
 #else
-	if (!entry)
+	if (entry == NULL)
 #endif
 		return;
 	down(&info_mutex);
 #ifndef CONFIG_DEVFS_FS
-	proc_unregister(snd_proc_dev, entry->p->low_ino);
+	snd_info_entry_remove(snd_proc_dev, entry->p);
 #endif
 	if (entry == (e = snd_info_devices)) {
 		snd_info_devices = snd_info_devices->next;
@@ -968,7 +1001,7 @@ void snd_info_free_device(snd_info_entry_t * entry)
 	up(&info_mutex);
 #ifdef CONFIG_DEVFS_FS
 	sprintf(dname, "snd/%s", entry->name);
-	master = devfs_find_handle(dname, 0, 0, 0, DEVFS_SPECIAL_CHR, 0);
+	master = devfs_find_handle(NULL, dname, 0, 0, 0, DEVFS_SPECIAL_CHR, 0);
 	devfs_unregister(master);
 #endif
 	snd_info_free_entry(entry);
@@ -998,8 +1031,7 @@ int snd_info_register(snd_info_entry_t * entry)
 	snd_card_t *card;
 	int len;
 
-	if (!entry)
-		return -EINVAL;
+	snd_debug_check(entry == NULL, -ENXIO);
 	entry->next = NULL;
 #if 0
 	snd_printk("snd_info_register: %s, 0x%lx\n",
@@ -1011,7 +1043,7 @@ int snd_info_register(snd_info_entry_t * entry)
 	down(&info_mutex);
 	p = create_proc_entry(entry->name, entry->mode, root);
 	if (p) {
-		p->fill_inode = snd_info_fill_inode;
+		snd_info_entry_prepare(p);
 		p->ops = &snd_info_entry_inode_operations;
 		p->size = entry->size;
 	} else {
@@ -1019,7 +1051,7 @@ int snd_info_register(snd_info_entry_t * entry)
 		return -ENOMEM;
 	}
 	entry->p = p;
-	if (!card) {
+	if (card == NULL) {
 		if (!(e = snd_info_entries)) {
 			snd_info_entries = entry;
 		} else {
@@ -1045,12 +1077,11 @@ int snd_info_unregister(snd_info_entry_t * entry)
 	struct proc_dir_entry *root;
 	snd_info_entry_t *e;
 
-	if (!entry || !entry->p)
-		return -EINVAL;
+	snd_debug_check(entry == NULL || entry->p == NULL, -ENXIO);
 	root = snd_info_get_root(entry);
 	down(&info_mutex);
-	proc_unregister(root, entry->p->low_ino);
-	if (!entry->card) {
+	snd_info_entry_remove(root, entry->p);
+	if (entry->card == NULL) {
 		if (entry == (e = snd_info_entries)) {
 			snd_info_entries = snd_info_entries->next;
 		} else {
@@ -1096,12 +1127,12 @@ static void snd_info_version_read(snd_info_buffer_t * buffer, void *private_data
 		    ".\n", kernel_version);
 }
 
-static int snd_info_version_init(void)
+static int __init snd_info_version_init(void)
 {
 	snd_info_entry_t *entry;
 
 	entry = snd_info_create_entry(NULL, "version");
-	if (!entry)
+	if (entry == NULL)
 		return -ENOMEM;
 	entry->t.text.read_size = 256;
 	entry->t.text.read = snd_info_version_read;
@@ -1113,86 +1144,11 @@ static int snd_info_version_init(void)
 	return 0;
 }
 
-static int snd_info_version_done(void)
+static int __exit snd_info_version_done(void)
 {
 	if (snd_info_version_entry)
 		snd_info_unregister(snd_info_version_entry);
 	return 0;
 }
 
-/*
- *  store/restore settings via persistant module
- */
-
-int snd_info_store_text(snd_info_entry_t * entry)
-{
-	int result = 0;
-	snd_info_buffer_t ibuffer;
-	char key[128];
-
-	if (!entry)
-		return -EINVAL;
-	if (entry->type != SND_INFO_ENTRY_TEXT)
-		return -EINVAL;
-	memset(&ibuffer, 0, sizeof(ibuffer));
-	ibuffer.len = (entry->t.text.read_size + (PAGE_SIZE - 1)) &
-							 ~(PAGE_SIZE - 1);
-	ibuffer.buffer = ibuffer.curr = snd_vmalloc(ibuffer.len);
-	if (!ibuffer.buffer)
-		return -ENOMEM;
-	down(&entry->access);
-	entry->t.text.read(&ibuffer, entry->private_data);
-	result = ibuffer.error;
-	if (!result) {
-		if (entry->card)
-			sprintf(key, "snd:%i/%s", entry->card->number,
-						  entry->name);
-		else
-			sprintf(key, "snd:%s", entry->name);
-		if (ibuffer.size > 0) {
-			result = snd_persist_store(key,
-						   ibuffer.buffer,
-						   ibuffer.size);
-		} else {
-			result = snd_persist_remove(key);
-		}
-	}
-	up(&entry->access);
-	snd_vfree(ibuffer.buffer);
-	return result;
-}
-
-int snd_info_restore_text(snd_info_entry_t * entry)
-{
-	int result = 0;
-	snd_info_buffer_t ibuffer;
-	char key[128];
-
-	if (!entry)
-		return -EINVAL;
-	if (entry->type != SND_INFO_ENTRY_TEXT)
-		return -EINVAL;
-	memset(&ibuffer, 0, sizeof(ibuffer));
-	ibuffer.len = (entry->t.text.write_size + (PAGE_SIZE - 1)) &
-							~(PAGE_SIZE - 1);
-	ibuffer.buffer = ibuffer.curr = snd_vmalloc(ibuffer.len);
-	if (!ibuffer.buffer)
-		return -ENOMEM;
-	down(&entry->access);
-	if (entry->card)
-		sprintf(key, "snd:%i/%s", entry->card->number, entry->name);
-	else
-		sprintf(key, "snd:%s", entry->name);
-	result = snd_persist_restore(key, ibuffer.buffer, ibuffer.len);
-	if (result > 0) {
-		ibuffer.size = result;
-		result = 0;
-		entry->t.text.write(&ibuffer, entry->private_data);
-		result = ibuffer.error;
-	}
-	up(&entry->access);
-	snd_vfree(ibuffer.buffer);
-	return result;
-}
-
 #endif /* CONFIG_PROC_FS */
