@@ -107,7 +107,7 @@ static int snd_mixer_oss_set_recsrc(snd_kmixer_t *mixer, int recsrc)
 			continue;
 		if (group->control == NULL)
 			continue;
-		if (group->control(group->private_data, NULL, 0, &g) < 0)
+		if (group->control(group, NULL, 0, &g) < 0)
 			continue;
 		if (!(g.caps & SND_MIXER_GRPCAP_CAPTURE))
 			continue;
@@ -116,8 +116,8 @@ static int snd_mixer_oss_set_recsrc(snd_kmixer_t *mixer, int recsrc)
 		} else {
 			g.capture = 0;
 		}
-		if (group->control(group->private_data, &mfile, 1, &g) > 0)
-			snd_mixer_group_value_change(NULL, group, 0);
+		if (group->control(group, &mfile, 1, &g) > 0)
+			snd_mixer_group_value_change_all(mixer, group, 0);
 	}
 	snd_mixer_lock(mixer, 1);
 	return 0;
@@ -152,7 +152,7 @@ static int snd_mixer_oss_stereodevs(snd_kmixer_t *mixer)
 			continue;
 		if (group->control == NULL)
 			continue;
-		if (group->control(group->private_data, NULL, 0, &g) < 0)
+		if (group->control(group, NULL, 0, &g) < 0)
 			continue;
 		if (((g.channels & SND_MIXER_CHN_MASK_STEREO) == SND_MIXER_CHN_MASK_STEREO) &&
 		    !(g.caps & SND_MIXER_GRPCAP_JOINTLY_VOLUME))
@@ -174,7 +174,7 @@ static int snd_mixer_oss_recmask(snd_kmixer_t *mixer)
 			continue;
 		if (group->control == NULL)
 			continue;
-		if (group->control(group->private_data, NULL, 0, &g) < 0)
+		if (group->control(group, NULL, 0, &g) < 0)
 			continue;
 		if (g.caps & SND_MIXER_GRPCAP_CAPTURE)
 			result |= 1 << group->oss_dev;
@@ -195,7 +195,7 @@ static int snd_mixer_oss_caps(snd_kmixer_t *mixer)
 			continue;
 		if (group->control == NULL)
 			continue;
-		if (group->control(group->private_data, NULL, 0, &g) < 0)
+		if (group->control(group, NULL, 0, &g) < 0)
 			continue;
 		if (g.caps & SND_MIXER_GRPCAP_EXCL_CAPTURE)
 			result |= SND_MIXER_OSS_CAP_EXCL_INPUT;
@@ -216,7 +216,7 @@ static int snd_mixer_oss_recsrc(snd_kmixer_t *mixer)
 			continue;
 		if (group->control == NULL)
 			continue;
-		if (group->control(group->private_data, NULL, 0, &g) < 0)
+		if (group->control(group, NULL, 0, &g) < 0)
 			continue;
 		if (g.caps & SND_MIXER_GRPCAP_CAPTURE) {
 			if ((g.capture & SND_MIXER_CHN_MASK_STEREO) == SND_MIXER_CHN_MASK_STEREO)
@@ -279,7 +279,7 @@ static int snd_mixer_oss_get_volume(snd_kmixer_t *mixer, int channel)
 		snd_mixer_lock(mixer, 1);
 		return -ENXIO;
 	}
-	if (group->control(group->private_data, NULL, 0, &g) < 0) {
+	if (group->control(group, NULL, 0, &g) < 0) {
 		snd_mixer_lock(mixer, 1);
 		return -ENXIO;
 	}
@@ -334,7 +334,7 @@ static int snd_mixer_oss_set_volume(snd_kmixer_t *mixer, int channel, int volume
 		snd_mixer_lock(mixer, 1);
 		return -ENXIO;
 	}
-	if (group->control(group->private_data, NULL, 0, &g) < 0) {
+	if (group->control(group, NULL, 0, &g) < 0) {
 		snd_mixer_lock(mixer, 1);
 		return -ENXIO;
 	}
@@ -373,20 +373,17 @@ static int snd_mixer_oss_set_volume(snd_kmixer_t *mixer, int channel, int volume
 	}
 	mixer->oss_volume_levels[channel][0] = left;
 	mixer->oss_volume_levels[channel][1] = right;
-	if (group->control(group->private_data, &mfile, 1, &g) > 0)
-		snd_mixer_group_value_change(NULL, group, 0);
+	if (group->control(group, &mfile, 1, &g) > 0)
+		snd_mixer_group_value_change_all(mixer, group, 0);
 	snd_mixer_lock(mixer, 1);
  	return (left & 0xff) | ((right & 0xff) << 8);
 }
 
-int snd_mixer_oss_ioctl(struct file *file, unsigned int cmd, unsigned long arg)
+static int snd_mixer_oss_ioctl1(snd_kmixer_t *mixer, unsigned int cmd, unsigned long arg)
 {
-	snd_kmixer_t *mixer;
 	int tmp;
 
-	mixer = (snd_kmixer_t *) file->private_data;
-	if (!mixer)
-		return -EIO;
+	snd_debug_check(mixer == NULL, -ENXIO);
 	if (((cmd >> 8) & 0xff) == 'M') {
 		switch (cmd) {
 		case SND_MIXER_OSS_INFO:
@@ -394,37 +391,67 @@ int snd_mixer_oss_ioctl(struct file *file, unsigned int cmd, unsigned long arg)
 		case SND_MIXER_OSS_OLD_INFO:
  			return snd_mixer_oss_info_obsolete(mixer, (struct snd_oss_mixer_info_obsolete *)arg);
 		case SND_MIXER_OSS_SET_RECSRC:
-			tmp = snd_mixer_oss_set_recsrc(mixer, snd_ioctl_in((long *) arg));
-			return tmp >= 0 ? snd_ioctl_out((long *) arg, tmp) : tmp;
+			if (get_user(tmp, (int *)arg))
+				return -EFAULT;
+			tmp = snd_mixer_oss_set_recsrc(mixer, tmp);
+			if (tmp < 0)
+				return tmp;
+			return put_user(tmp, (int *)arg) ? -EFAULT : 0;
 		case SND_OSS_GETVERSION:
-			return snd_ioctl_out((long *) arg, SND_OSS_VERSION);
+			return put_user(SND_OSS_VERSION, (int *) arg);
 		case SND_MIXER_OSS_DEVMASK:
 			tmp = snd_mixer_oss_devmask(mixer);
-			return tmp >= 0 ? snd_ioctl_out((long *) arg, tmp) : tmp;
+			if (tmp < 0)
+				return tmp;
+			return put_user(tmp, (int *)arg) ? -EFAULT : 0;
 		case SND_MIXER_OSS_STEREODEVS:
 			tmp = snd_mixer_oss_stereodevs(mixer);
-			return tmp >= 0 ? snd_ioctl_out((long *) arg, tmp) : tmp;
+			if (tmp < 0)
+				return tmp;
+			return put_user(tmp, (int *)arg) ? -EFAULT : 0;
 		case SND_MIXER_OSS_RECMASK:
 			tmp = snd_mixer_oss_recmask(mixer);
-			return tmp >= 0 ? snd_ioctl_out((long *) arg, tmp) : tmp;
+			if (tmp < 0)
+				return tmp;
+			return put_user(tmp, (int *)arg) ? -EFAULT : 0;
 		case SND_MIXER_OSS_CAPS:
 			tmp = snd_mixer_oss_caps(mixer);
-			return tmp >= 0 ? snd_ioctl_out((long *) arg, tmp) : tmp;
+			if (tmp < 0)
+				return tmp;
+			return put_user(tmp, (int *)arg) ? -EFAULT : 0;
 		case SND_MIXER_OSS_RECSRC:
 			tmp = snd_mixer_oss_recsrc(mixer);
-			return tmp >= 0 ? snd_ioctl_out((long *) arg, tmp) : tmp;
+			if (tmp < 0)
+				return tmp;
+			return put_user(tmp, (int *)arg) ? -EFAULT : 0;
 		}
 	}
 	if (cmd & IOC_IN) {
-		tmp = snd_mixer_oss_set_volume(mixer, cmd & 0xff, snd_ioctl_in((long *) arg));
-		return tmp >= 0 ? snd_ioctl_out((long *) arg, tmp) : tmp;
+		if (get_user(tmp, (int *)arg))
+			return -EFAULT;
+		tmp = snd_mixer_oss_set_volume(mixer, cmd & 0xff, tmp);
+		if (tmp < 0)
+			return tmp;
+		return put_user(tmp, (int *)arg) ? -EFAULT : 0;
 	} else if (cmd & IOC_OUT) {
 		tmp = snd_mixer_oss_get_volume(mixer, cmd & 0xff);
-		return tmp >= 0 ? snd_ioctl_out((long *) arg, tmp) : tmp;
+		if (tmp < 0)
+			return tmp;
+		return put_user(tmp, (int *)arg) ? -EFAULT : 0;
 	}
 	return -ENXIO;
 }
 
+int snd_mixer_oss_ioctl(struct file *file, unsigned int cmd, unsigned long arg)
+{
+	return snd_mixer_oss_ioctl1((snd_kmixer_t *) file->private_data, cmd, arg);
+}
+
+int snd_mixer_oss_ioctl_card(snd_card_t *card, unsigned int cmd, unsigned long arg)
+{
+	return snd_mixer_oss_ioctl1(snd_mixers[(card->number * SND_MINOR_MIXERS) + 0], cmd, arg);
+}
+
 /*
  *  REGISTRATION PART
  */
@@ -434,8 +461,6 @@ static snd_minor_t snd_mixer_oss_reg =
 	"mixer",		/* comment */
 	NULL,			/* reserved */
 
-	NULL,			/* unregister */
-
 	NULL,			/* lseek */
 	NULL,			/* read */
 	NULL,			/* write */
@@ -456,16 +481,17 @@ static int snd_mixer_oss_register_minor(unsigned short native_minor,
 	char name[32];
 	
 	mixer->ossreg = 0;
-	if (mixer->device < 1) {
+	if (mixer->device <= 1) {
 		sprintf(name, "mixer%i%i", mixer->card->number, mixer->device);
 		if (snd_register_oss_device(SND_OSS_DEVICE_TYPE_MIXER,
 				mixer->card, mixer->device, &snd_mixer_oss_reg,
 				name) < 0) {
 			snd_printk("unable to register OSS mixer device %i:%i\n", mixer->card->number, mixer->device);
 		} else {
-			snd_oss_info_register(SND_OSS_INFO_DEV_MIXERS,
-					      mixer->card->number,
-					      mixer->name);
+			if (mixer->device == 0)
+				snd_oss_info_register(SND_OSS_INFO_DEV_MIXERS,
+						      mixer->card->number,
+						      mixer->name);
 			mixer->ossreg = 1;
 		}
 	}
@@ -476,9 +502,10 @@ static int snd_mixer_oss_unregister_minor(unsigned short native_minor,
 					  snd_kmixer_t * mixer)
 {
 	if (mixer->ossreg) {
+		if (mixer->device == 0)
+			snd_oss_info_unregister(SND_OSS_INFO_DEV_MIXERS, mixer->card->number);
 		snd_unregister_oss_device(SND_OSS_DEVICE_TYPE_MIXER,
-				mixer->card, mixer->device);
-		snd_oss_info_unregister(SND_OSS_INFO_DEV_MIXERS, mixer->device);
+					  mixer->card, mixer->device);
 		mixer->ossreg = 0;
 	}
 	return 0;
@@ -491,7 +518,11 @@ static struct snd_stru_mixer_notify snd_mixer_oss_notify =
 	NULL
 };
 
-int init_module(void)
+#ifdef MODULE
+int __init init_module(void)
+#else
+int __init alsa_mixer_oss_init(void)
+#endif
 {
 	int err;
 
@@ -500,7 +531,13 @@ int init_module(void)
 	return 0;
 }
 
-void cleanup_module(void)
+#ifdef MODULE
+
+void __exit cleanup_module(void)
 {
 	snd_mixer_notify(&snd_mixer_oss_notify, 1);
 }
+
+#endif
+
+EXPORT_SYMBOL(snd_mixer_oss_ioctl_card);
