@@ -347,7 +347,7 @@ int snd_pcm_open_subchn(snd_pcm_t *pcm, int channel,
 	runtime->sleep_lock = SPIN_LOCK_UNLOCKED;
 
 	runtime->mode = SND_PCM_MODE_BLOCK;
-	*runtime->status = SND_PCM_STATUS_READY;
+	*runtime->status = SND_PCM_STATUS_NOTREADY;
 
 	subchn->runtime = runtime;
 	*rsubchn = subchn;
