@@ -191,6 +191,8 @@ int snd_pcm_lib_fragment(long size, snd_pcm_hardware_t *hw)
 	size += hw->fragment_align;
 	size /= (hw->fragment_align + 1);
 	size *= (hw->fragment_align + 1);
+	if (size == 0)
+		size = hw->fragment_align + 1;
 	return size;
 }
 
@@ -201,7 +203,7 @@ int snd_pcm_lib_set_buffer_size(snd_pcm_subchn_t *subchn, long size)
 
 	if (runtime->mode == SND_PCM_MODE_BLOCK) {
 		if (size < runtime->buf.block.frag_size) {
-			runtime->buf.block.frag_size = size;
+			runtime->buf.block.frag_size = snd_pcm_lib_fragment(size, runtime->hw);
 			runtime->frags = 1;
 		}
 		else
@@ -220,7 +222,7 @@ int snd_pcm_lib_set_buffer_size(snd_pcm_subchn_t *subchn, long size)
 		if (runtime->frags > tmp)
 			runtime->frags = tmp;
 		if (runtime->frags == 1) {
-			runtime->buf.block.frag_size /= 2;
+			runtime->buf.block.frag_size = snd_pcm_lib_fragment(runtime->buf.block.frag_size / 2, runtime->hw);
 			runtime->frags++;
 		}
 		if (runtime->buf.block.frags_min < 0)
@@ -356,7 +358,6 @@ static int snd_pcm_lib_ioctl_status(void *private_data,
 	int stop = 0;
 
 	spin_lock_irqsave(&runtime->lock, flags);
-	status->status = *runtime->status;
 	if (runtime->flags & SND_PCM_FLG_TIME) {
 		status->stime = runtime->stime;
 	} else {
@@ -1403,6 +1404,7 @@ EXPORT_SYMBOL(snd_pcm_playback_ok);
 EXPORT_SYMBOL(snd_pcm_capture_ok);
 EXPORT_SYMBOL(snd_pcm_playback_data);
 EXPORT_SYMBOL(snd_pcm_capture_empty);
+EXPORT_SYMBOL(snd_pcm_clear_values);
 EXPORT_SYMBOL(snd_pcm_transfer_stop);
 EXPORT_SYMBOL(snd_pcm_transfer_done);
 EXPORT_SYMBOL(snd_pcm_playback_write);
