@@ -205,7 +205,7 @@ static int snd_pcm_channel_status(snd_pcm_subchn_t * subchn, snd_pcm_channel_sta
 		return -EFAULT;
 	runtime = subchn->runtime;
 	if (*runtime->status == SND_PCM_STATUS_NOTREADY)
-		return -EIO;
+		return -EBADFD;
 	memset(&status, 0, sizeof(status));
 	status.channel = subchn->channel;
 	status.mode = runtime->mode;
@@ -213,6 +213,7 @@ static int snd_pcm_channel_status(snd_pcm_subchn_t * subchn, snd_pcm_channel_sta
 				 SND_PCM_IOCTL1_STATUS, (unsigned long *)&status);
 	if (res < 0)
 		return res;
+	status.status = *runtime->status;
 	if (copy_to_user(_status, &status, sizeof(status)))
 		return -EFAULT;
 	return 0;
@@ -229,7 +230,10 @@ static int snd_pcm_channel_prepare(snd_pcm_subchn_t * subchn)
 		return -EBUSY;
 	res = runtime->hw->prepare(subchn->pchn->private_data, subchn);
 	if (res >= 0) {
-		snd_pcm_clear_values(subchn);
+#ifdef CONFIG_SND_OSSEMUL
+		if (!subchn->oss.oss)
+#endif
+			snd_pcm_clear_values(subchn);
 		*runtime->status = SND_PCM_STATUS_PREPARED;
 	}
 	return res;
@@ -460,7 +464,8 @@ static int snd_pcm_release_file(snd_pcm_file_t * pcm_file)
 	subchn = pcm_file->subchn;
 	snd_debug_check(subchn == NULL, -ENXIO);
 	chn = subchn->pchn;
-	snd_pcm_channel_go(subchn, SND_PCM_TRIGGER_STOP);
+	if (*subchn->runtime->status == SND_PCM_STATUS_RUNNING)
+		snd_pcm_channel_go(subchn, SND_PCM_TRIGGER_STOP);
 	chn->close(chn->private_data, subchn);
 	subchn->ffile = NULL;
 	snd_pcm_remove_file(chn, pcm_file);
