@@ -30,23 +30,12 @@
 #include "../include/pcm.h"
 #include "../include/info.h"
 
-static int snd_dsp_map[SND_CARDS] = {[0 ... (SND_CARDS-1)] = 0};
-static int snd_adsp_map[SND_CARDS] = {[0 ... (SND_CARDS-1)] = 1};
-static int snd_nonblock_open = 0;
-MODULE_PARM(snd_dsp_map, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dsp_map, "PCM device number assigned to 1st OSS device.");
-MODULE_PARM(snd_adsp_map, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_adsp_map, "PCM device number assigned to 2nd OSS device.");
-MODULE_PARM(snd_nonblock_open, "i");
-MODULE_PARM_DESC(snd_nonblock_open, "Don't block opening busy PCM devices.");
-
 extern int snd_mixer_oss_ioctl_card(snd_card_t *card, unsigned int cmd, unsigned long arg);
 static int snd_pcm_oss_get_rate(snd_pcm_oss_file_t *pcm_oss_file);
 static int snd_pcm_oss_get_voices(snd_pcm_oss_file_t *pcm_oss_file);
 static int snd_pcm_oss_get_format(snd_pcm_oss_file_t *pcm_oss_file);
 
 EXPORT_NO_SYMBOLS;
-MODULE_LICENSE("GPL");
 
 int snd_pcm_oss_plugin_clear(snd_pcm_runtime_t *runtime)
 {
@@ -198,6 +187,7 @@ static int snd_pcm_oss_change_params(snd_pcm_subchn_t *subchn)
 		if (subchn->oss.setup->fragment_size > 64)
 			runtime->oss.fragment_size = subchn->oss.setup->fragment_size;
 	}
+
 	memset(&params, 0, sizeof(params));
 	params.channel = subchn->channel;
 	params.mode = SND_PCM_MODE_BLOCK;
@@ -420,7 +410,6 @@ static void snd_pcm_oss_alloc_free(snd_pcm_runtime_t * runtime)
 	for (idx = 0; idx < 2; idx++) {
 		if (runtime->oss.xbuffer[idx]) {
 			snd_vfree(runtime->oss.xbuffer[idx]);
-			runtime->oss.xbuffer[idx] = NULL;
 			runtime->oss.xbuffer_size[idx] = 0;
 			runtime->oss.xbuffer_lock[idx] = 0;
 		}
@@ -477,7 +466,7 @@ static long snd_pcm_oss_write2(snd_pcm_subchn_t * subchn, char *src_ptr, long sr
 	while (1) {
 		if (in_kernel)
 			fs = snd_enter_user();
-		tmp = snd_pcm_lib_write(subchn, src_ptr, src_size);
+		tmp = runtime->hw->transfer(subchn->pchn->private_data, subchn, src_ptr, src_size);
 		if (in_kernel)
 			snd_leave_user(fs);
 		if (tmp <= 0) {
@@ -580,7 +569,7 @@ static long snd_pcm_oss_read2(snd_pcm_subchn_t * subchn, char *final_ptr, long f
 	while (1) {
 		if (in_kernel)
 			fs = snd_enter_user();
-		tmp = snd_pcm_lib_read(subchn, src_ptr, src_size);
+		tmp = runtime->hw->transfer(subchn->pchn->private_data, subchn, src_ptr, src_size);
 		if (in_kernel)
 			snd_leave_user(fs);
 		if (tmp <= 0) {
@@ -710,7 +699,7 @@ static int snd_pcm_oss_sync(snd_pcm_oss_file_t *pcm_oss_file)
 			return err;
 		runtime = subchn->runtime;
 		while (runtime->oss.buffer && runtime->oss.buffer_used > 0) {
-			neutral = snd_pcm_format_silence(runtime->format.format);
+			neutral = snd_pcm_lib_silence(subchn);
 			memset(runtime->oss.buffer + runtime->oss.buffer_used, neutral, runtime->oss.fragment_size - runtime->oss.buffer_used);
 			fs = snd_enter_user();
 			result = snd_pcm_oss_write2(subchn, runtime->oss.buffer, runtime->oss.fragment_size);
@@ -940,7 +929,7 @@ static int snd_pcm_oss_set_fragment1(snd_pcm_subchn_t *subchn, int fragment)
 		return -EINVAL;
 	bytes = fragment & 0xffff;
 	count = (fragment >> 16) & 0xffff;
-	if (count == 0 || count > 128) {
+	if (count == 0) {
 		count = 128;
 	} else if (count < 128) {
 		count++;
@@ -1103,8 +1092,7 @@ static int snd_pcm_oss_get_odelay(snd_pcm_oss_file_t *pcm_oss_file)
 	int err;
 
 	subchn = pcm_oss_file->chn[SND_PCM_CHANNEL_PLAYBACK];
-	if (subchn == NULL)
-		return -EINVAL;
+	snd_debug_check(subchn == NULL, -EINVAL);
 	if ((err = snd_pcm_oss_make_ready(subchn)) < 0)
 		return err;
 	runtime = subchn->runtime;
@@ -1128,8 +1116,7 @@ static int snd_pcm_oss_get_ptr(snd_pcm_oss_file_t *pcm_oss_file, int channel, st
 	if (_info == NULL)
 		return -EFAULT;
 	subchn = pcm_oss_file->chn[channel];
-	if (subchn == NULL)
-		return -EINVAL;
+	snd_debug_check(subchn == NULL, -EINVAL);
 	if ((err = snd_pcm_oss_make_ready(subchn)) < 0)
 		return err;
 	runtime = subchn->runtime;
@@ -1164,13 +1151,11 @@ static int snd_pcm_oss_get_space(snd_pcm_oss_file_t *pcm_oss_file, int channel,
 	snd_pcm_channel_status_t status;
 	struct snd_pcm_buffer_info info;
 	int err;
-	unsigned int tmp;
 
 	if (_info == NULL)
 		return -EFAULT;
 	subchn = pcm_oss_file->chn[channel];
-	if (subchn == NULL)
-		return -EINVAL;
+	snd_debug_check(subchn == NULL, -EINVAL);
 	runtime = subchn->runtime;
 
 	if (runtime->oss.params &&
@@ -1197,12 +1182,6 @@ static int snd_pcm_oss_get_space(snd_pcm_oss_file_t *pcm_oss_file, int channel,
 	} else {
 		info.fragments = status.count / runtime->buf.block.frag_size;
 	}
-	/* very experimental stuff to get Quake2 working */
-	runtime->oss.fragment = (info.fragments - 1) << 16;
-	for (tmp = info.fragsize; tmp > 1; tmp >>= 1)
-		runtime->oss.fragment++;
-	runtime->oss.subdivision = 1;	/* disable SUBDIVIDE */
-	// printk("space: bytes = %i, fragments = %i, fragstotal = %i, fragsize = %i\n", info.bytes, info.fragments, info.fragstotal, info.fragsize);
 	if (copy_to_user(_info, &info, sizeof(info)))
 		return -EFAULT;
 	return 0;
@@ -1402,8 +1381,7 @@ static int snd_pcm_oss_open(unsigned short minor, int cardnum,
 	snd_pcm_oss_setup_t *psetup = NULL, *csetup = NULL;
 	int nonblock;
 
-	pcm = snd_pcm_devices[(cardnum * SND_PCM_DEVICES) +
-			      (device == 0 ? snd_dsp_map[cardnum] : snd_adsp_map[cardnum])];
+	pcm = snd_pcm_devices[(cardnum * SND_PCM_DEVICES) + device];
 	if (pcm == NULL)
 		return -ENODEV;
 	if (snd_task_name(current, task_name, sizeof(task_name)) < 0)
@@ -1419,15 +1397,11 @@ static int snd_pcm_oss_open(unsigned short minor, int cardnum,
 			nonblock = 1;
 		else if (psetup->block)
 			nonblock = 0;
-		else if (!nonblock)
-			nonblock = snd_nonblock_open;
 	} else if (csetup && !csetup->disable) {
 		if (csetup->nonblock)
 			nonblock = 1;
 		else if (csetup->block)
 			nonblock = 0;
-		else if (!nonblock)
-			nonblock = snd_nonblock_open;
 	}
 
 	while (1) {
@@ -1699,8 +1673,6 @@ static unsigned int snd_pcm_oss_poll(struct file *file, poll_table * wait)
 		if (*csubchn->runtime->status == SND_PCM_STATUS_OVERRUN)
 			csubchn->runtime->oss.prepare = 1;
 		snd_pcm_oss_make_ready(csubchn);
-		if (*csubchn->runtime->status == SND_PCM_STATUS_PREPARED)
-			snd_pcm_kernel_capture_ioctl(csubchn, SND_PCM_IOCTL_CHANNEL_GO, 0);
 	}
 
 	mask = 0;
@@ -1775,7 +1747,7 @@ static int snd_pcm_oss_mmap(struct inode *inode, struct file *file,
 	pdma = runtime->dma_area;
 	if (pdma == NULL)
 		return -ENOENT;
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 3, 25)
+#if LinuxVersionCode(2, 3, 25) <= LINUX_VERSION_CODE
 	if (area->vm_pgoff != 0)
 #else
 	if (area->vm_offset != 0)
@@ -1789,7 +1761,7 @@ static int snd_pcm_oss_mmap(struct inode *inode, struct file *file,
 	if (vma == NULL)
 		return -ENOMEM;
 	if (remap_page_range(area->vm_start,
-			     virt_to_phys(pdma->buf),
+			     virt_to_bus(pdma->buf),
 			     size,
 			     area->vm_page_prot)) {
 		snd_kfree(vma);
@@ -1981,48 +1953,42 @@ static void snd_pcm_oss_proc_done(snd_pcm_t * pcm)
 
 static snd_minor_t snd_pcm_oss_reg =
 {
-	comment:	"digital audio",
-	read:		snd_pcm_oss_read,
-	write:		snd_pcm_oss_write,
-	open:		snd_pcm_oss_open,
-	release:	snd_pcm_oss_release,
-	poll:		snd_pcm_oss_poll,
-	ioctl:		snd_pcm_oss_ioctl,
-	mmap:		snd_pcm_oss_mmap,
-};
+	"digital audio",	/* comment */
+	NULL,			/* reserved */
 
-static void register_oss_dsp(unsigned short native_minor, snd_pcm_t *pcm, int index)
-{
-	char name[128];
-	sprintf(name, "dsp%i%i", pcm->card->number, index);
-	if (snd_register_oss_device(SND_OSS_DEVICE_TYPE_PCM,
-				    pcm->card, index, &snd_pcm_oss_reg,
-				    name) < 0) {
-		snd_printk("unable to register OSS PCM device %i:%i\n", pcm->card->number, index);
-	}
-}
+	NULL,			/* lseek */
+	snd_pcm_oss_read,	/* read */
+	snd_pcm_oss_write,	/* write */
+	snd_pcm_oss_open,	/* open */
+	snd_pcm_oss_release,	/* release */
+	snd_pcm_oss_poll,	/* poll */
+	snd_pcm_oss_ioctl,	/* ioctl */
+	snd_pcm_oss_mmap,	/* mmap */
+};
 
 static int snd_pcm_oss_register_minor(unsigned short native_minor,
 				      snd_pcm_t * pcm)
 {
+	char name[128];
+
 	pcm->oss.reg = 0;
-	if (snd_dsp_map[pcm->card->number] == pcm->device) {
-		char name[128];
-		register_oss_dsp(native_minor, pcm, 0);
-		sprintf(name, "%s%s", pcm->name, pcm->info_flags & SND_PCM_INFO_DUPLEX ? " (DUPLEX)" : "");
-		snd_oss_info_register(SND_OSS_INFO_DEV_AUDIO,
-				      pcm->card->number,
-				      name);
-		pcm->oss.reg++;
+	if (pcm->device < 2) {
+		sprintf(name, "dsp%i%i", pcm->card->number, pcm->device);
+		if (snd_register_oss_device(SND_OSS_DEVICE_TYPE_PCM,
+				pcm->card, pcm->device, &snd_pcm_oss_reg,
+				name) < 0) {
+			snd_printk("unable to register OSS PCM device %i:%i\n", pcm->card->number, pcm->device);
+		} else {
+			if (pcm->device == 0) {
+				sprintf(name, "%s%s", pcm->name, pcm->info_flags & SND_PCM_INFO_DUPLEX ? " (DUPLEX)" : "");
+				snd_oss_info_register(SND_OSS_INFO_DEV_AUDIO,
+						      pcm->card->number,
+						      name);
+			}
+			pcm->oss.reg = 1;
+			snd_pcm_oss_proc_init(pcm);
+		}
 	}
-	if (snd_adsp_map[pcm->card->number] == pcm->device) {
-		register_oss_dsp(native_minor, pcm, 1);
-		pcm->oss.reg++;
-	}
-
-	if (pcm->oss.reg)
-		snd_pcm_oss_proc_init(pcm);
-
 	return 0;
 }
 
@@ -2030,14 +1996,10 @@ static int snd_pcm_oss_unregister_minor(unsigned short native_minor,
 				        snd_pcm_t * pcm)
 {
 	if (pcm->oss.reg) {
-		if (snd_dsp_map[pcm->card->number] == pcm->device) {
-			snd_unregister_oss_device(SND_OSS_DEVICE_TYPE_PCM,
-						  pcm->card, 0);
+		snd_unregister_oss_device(SND_OSS_DEVICE_TYPE_PCM,
+					  pcm->card, pcm->device);
+		if (pcm->device == 0)
 			snd_oss_info_unregister(SND_OSS_INFO_DEV_AUDIO, pcm->card->number);
-		}
-		if (snd_adsp_map[pcm->card->number] == pcm->device)
-			snd_unregister_oss_device(SND_OSS_DEVICE_TYPE_PCM,
-						  pcm->card, 1);
 		pcm->oss.reg = 0;
 		snd_pcm_oss_proc_done(pcm);
 	}
@@ -2051,30 +2013,24 @@ static struct snd_stru_pcm_notify snd_pcm_oss_notify =
 	NULL
 };
 
-static int __init alsa_pcm_oss_init(void)
+#ifdef MODULE
+int __init init_module(void)
+#else
+int __init alsa_pcm_oss_init(void)
+#endif
 {
-	int i, err;
+	int err;
 
 	if ((err = snd_pcm_notify(&snd_pcm_oss_notify, 0)) < 0)
 		return err;
-	/* check device map table */
-	for (i = 0; i < SND_CARDS; i++) {
-		if (snd_dsp_map[i] < 0 || snd_dsp_map[i] >= SND_PCM_DEVICES) {
-			snd_printk("invalid dsp_map[%d] = %d\n", i, snd_dsp_map[i]);
-			snd_dsp_map[i] = 0;
-		}
-		if (snd_adsp_map[i] < 0 || snd_adsp_map[i] >= SND_PCM_DEVICES) {
-			snd_printk("invalid adsp_map[%d] = %d\n", i, snd_adsp_map[i]);
-			snd_adsp_map[i] = 1;
-		}
-	}
 	return 0;
 }
 
-static void __exit alsa_pcm_oss_exit(void)
+#ifdef MODULE
+
+void __exit cleanup_module(void)
 {
 	snd_pcm_notify(&snd_pcm_oss_notify, 1);
 }
 
-module_init(alsa_pcm_oss_init)
-module_exit(alsa_pcm_oss_exit)
+#endif
