@@ -463,8 +463,8 @@ static int snd_rawmidi_info(snd_rawmidi_t * rmidi, snd_rawmidi_info_t * _info)
 	return 0;
 }
 
-static int snd_rawmidi_output_params(snd_rawmidi_t * rmidi,
-				     snd_rawmidi_params_t * params)
+int snd_rawmidi_output_params(snd_rawmidi_t * rmidi,
+			      snd_rawmidi_params_t * params)
 {
 	char *newbuf;
 	snd_rawmidi_channel_t *pchn;
@@ -478,19 +478,21 @@ static int snd_rawmidi_output_params(snd_rawmidi_t * rmidi,
 		return -EINVAL;
 	if (params->max < 1 || params->max >= params->size || params->max < params->room)
 		return -EINVAL;
-	if ((newbuf = (char *) snd_kmalloc(params->size, GFP_KERNEL)) == NULL)
-		return -ENOMEM;
 	pchn = &rmidi->chn[SND_RAWMIDI_CHANNEL_OUTPUT];
-	snd_kfree(pchn->buffer);
-	pchn->buffer = newbuf;
-	pchn->size = params->size;
+	if (params->size != pchn->size) {
+		if ((newbuf = (char *) snd_kmalloc(params->size, GFP_KERNEL)) == NULL)
+			return -ENOMEM;
+		snd_kfree(pchn->buffer);
+		pchn->buffer = newbuf;
+		pchn->size = params->size;
+	}
 	pchn->used_max = params->max;
 	pchn->used_room = params->room;
 	return 0;
 }
 
-static int snd_rawmidi_input_params(snd_rawmidi_t * rmidi,
-				    snd_rawmidi_params_t * params)
+int snd_rawmidi_input_params(snd_rawmidi_t * rmidi,
+			     snd_rawmidi_params_t * params)
 {
 	char *newbuf;
 	snd_rawmidi_channel_t *pchn;
@@ -501,11 +503,13 @@ static int snd_rawmidi_input_params(snd_rawmidi_t * rmidi,
 		return -EINVAL;
 	if (params->min < 1 || params->min >= params->size)
 		return -EINVAL;
-	if ((newbuf = (char *) snd_kmalloc(params->size, GFP_KERNEL)) == NULL)
-		return -ENOMEM;
-	snd_kfree(pchn->buffer);
-	pchn->buffer = newbuf;
-	pchn->size = params->size;
+	if (params->size != pchn->size) {
+		if ((newbuf = (char *) snd_kmalloc(params->size, GFP_KERNEL)) == NULL)
+			return -ENOMEM;
+		snd_kfree(pchn->buffer);
+		pchn->buffer = newbuf;
+		pchn->size = params->size;
+	}
 	pchn->used_min = params->min;
 	return 0;
 }
@@ -1299,6 +1303,8 @@ void __exit cleanup_module(void)
 
 #endif
 
+EXPORT_SYMBOL(snd_rawmidi_output_params);
+EXPORT_SYMBOL(snd_rawmidi_input_params);
 EXPORT_SYMBOL(snd_rawmidi_drain_output);
 EXPORT_SYMBOL(snd_rawmidi_flush_output);
 EXPORT_SYMBOL(snd_rawmidi_flush_input);
