@@ -22,6 +22,7 @@
 #include "seq_oss_synth.h"
 #include "seq_oss_midi.h"
 #include "seq_oss_event.h"
+#include "seq_oss_timer.h"
 #include "../../../include/seq_oss_legacy.h"
 #include "seq_oss_readq.h"
 #include "seq_oss_writeq.h"
@@ -40,7 +41,7 @@ static int note_on_event(seq_oss_devinfo_t *dp, int dev, int ch, int note, int v
 static int note_off_event(seq_oss_devinfo_t *dp, int dev, int ch, int note, int vel, snd_seq_event_t *ev);
 static int set_note_event(seq_oss_devinfo_t *dp, int dev, int type, int ch, int note, int vel, snd_seq_event_t *ev);
 static int set_control_event(seq_oss_devinfo_t *dp, int dev, int type, int ch, int param, int val, snd_seq_event_t *ev);
-static int set_echo_event(seq_oss_devinfo_t *dp, unsigned int echo, snd_seq_event_t *ev);
+static int set_echo_event(seq_oss_devinfo_t *dp, evrec_t *rec, snd_seq_event_t *ev);
 
 
 /*
@@ -72,6 +73,8 @@ snd_seq_oss_process_event(seq_oss_devinfo_t *dp, evrec_t *q, snd_seq_event_t *ev
 		return snd_seq_oss_synth_sysex(dp, q->x.dev, q->x.buf, ev);
 
 	case SND_OSS_SEQ_MIDIPUTC:
+		if (dp->seq_mode == SND_SEQ_OSS_MODE_MUSIC)
+			return -EINVAL;
 		/* put a midi byte */
 		if (! is_write_mode(dp->file_mode))
 			break;
@@ -82,18 +85,24 @@ snd_seq_oss_process_event(seq_oss_devinfo_t *dp, evrec_t *q, snd_seq_event_t *ev
 		break;
 
 	case SND_OSS_SEQ_ECHO:
-		return set_echo_event(dp, q->echo >> 8, ev);
+		if (dp->seq_mode == SND_SEQ_OSS_MODE_MUSIC)
+			return -EINVAL;
+		return set_echo_event(dp, q, ev);
 
 	case SND_OSS_SEQ_PRIVATE:
+		if (dp->seq_mode == SND_SEQ_OSS_MODE_MUSIC)
+			return -EINVAL;
 		return snd_seq_oss_synth_raw_event(dp, q->c[1], q->c, ev);
 
 	default:
+		if (dp->seq_mode == SND_SEQ_OSS_MODE_MUSIC)
+			return -EINVAL;
 		return old_event(dp, q, ev);
 	}
 	return -EINVAL;
 }
 
-/* old type events */
+/* old type events: mode1 only */
 static int
 old_event(seq_oss_devinfo_t *dp, evrec_t *q, snd_seq_event_t *ev)
 {
@@ -113,15 +122,13 @@ old_event(seq_oss_devinfo_t *dp, evrec_t *q, snd_seq_event_t *ev)
 					 q->n.chn, 0, q->n.note, ev);
 
 	case SND_OSS_SEQ_SYNCTIMER:
-		/* start timer -- already parsed */
-		break;
-
+		return snd_seq_oss_timer_reset(dp->timer);
 	}
 
 	return -EINVAL;
 }
 
-/* 8bytes extended event */
+/* 8bytes extended event: mode1 only */
 static int
 extended_event(seq_oss_devinfo_t *dp, evrec_t *q, snd_seq_event_t *ev)
 {
@@ -175,7 +182,7 @@ extended_event(seq_oss_devinfo_t *dp, evrec_t *q, snd_seq_event_t *ev)
 	return -EINVAL;
 }
 
-/* channel voice events */
+/* channel voice events: mode1 and 2 */
 static int
 chn_voice_event(seq_oss_devinfo_t *dp, evrec_t *q, snd_seq_event_t *ev)
 {
@@ -194,7 +201,7 @@ chn_voice_event(seq_oss_devinfo_t *dp, evrec_t *q, snd_seq_event_t *ev)
 	return -EINVAL;
 }
 
-/* channel common events */
+/* channel common events: mode1 and 2 */
 static int
 chn_common_event(seq_oss_devinfo_t *dp, evrec_t *q, snd_seq_event_t *ev)
 {
@@ -219,17 +226,42 @@ chn_common_event(seq_oss_devinfo_t *dp, evrec_t *q, snd_seq_event_t *ev)
 	return -EINVAL;
 }
 
-/* timer events */
+/* timer events: mode1 and mode2 */
 static int
 timing_event(seq_oss_devinfo_t *dp, evrec_t *q, snd_seq_event_t *ev)
 {
-	if (q->t.cmd == SND_OSS_TMR_ECHO)
-		return set_echo_event(dp, q->t.time, ev);
+	switch (q->t.cmd) {
+	case SND_OSS_TMR_ECHO:
+		if (dp->seq_mode == SND_SEQ_OSS_MODE_MUSIC)
+			return set_echo_event(dp, q, ev);
+		else {
+			evrec_t tmp;
+			memset(&tmp, 0, sizeof(tmp));
+			/* XXX: only for little-endian! */
+			tmp.echo = (q->t.time << 8) | SND_OSS_SEQ_ECHO;
+			return set_echo_event(dp, &tmp, ev);
+		} 
+
+	case SND_OSS_TMR_STOP:
+		if (dp->seq_mode)
+			return snd_seq_oss_timer_stop(dp->timer);
+		return 0;
+
+	case SND_OSS_TMR_CONTINUE:
+		if (dp->seq_mode)
+			return snd_seq_oss_timer_continue(dp->timer);
+		return 0;
+
+	case SND_OSS_TMR_TEMPO:
+		if (dp->seq_mode)
+			return snd_seq_oss_timer_tempo(dp->timer, q->t.time);
+		return 0;
+	}
 
 	return -EINVAL;
 }
 
-/* local events */
+/* local events: mode1 and 2 */
 static int
 local_event(seq_oss_devinfo_t *dp, evrec_t *q, snd_seq_event_t *ev)
 {
@@ -340,7 +372,7 @@ set_note_event(seq_oss_devinfo_t *dp, int dev, int type, int ch, int note, int v
 	
 	ev->type = type;
 	snd_seq_oss_synth_addr(dp, dev, ev);
-	ev->dest.channel = ch;
+	ev->data.note.channel = ch;
 	ev->data.note.note = note;
 	ev->data.note.velocity = vel;
 
@@ -358,7 +390,7 @@ set_control_event(seq_oss_devinfo_t *dp, int dev, int type, int ch, int param, i
 	
 	ev->type = type;
 	snd_seq_oss_synth_addr(dp, dev, ev);
-	ev->dest.channel = ch;
+	ev->data.control.channel = ch;
 	ev->data.control.param = param;
 	ev->data.control.value = val;
 
@@ -369,14 +401,12 @@ set_control_event(seq_oss_devinfo_t *dp, int dev, int type, int ch, int param, i
  * create an echo event
  */
 static int
-set_echo_event(seq_oss_devinfo_t *dp, unsigned int echo, snd_seq_event_t *ev)
+set_echo_event(seq_oss_devinfo_t *dp, evrec_t *rec, snd_seq_event_t *ev)
 {
 	ev->type = SND_SEQ_EVENT_ECHO;
 	/* echo back to itself */
 	snd_seq_oss_fill_addr(dp, ev, dp->addr.client, dp->addr.port);
-	ev->data.raw32.d[0] = echo;
-	ev->data.raw32.d[1] = 0; /* for user event */
-
+	memcpy(&ev->data, rec, LONG_EVENT_SIZE);
 	return 0;
 }
 
@@ -385,24 +415,28 @@ set_echo_event(seq_oss_devinfo_t *dp, unsigned int echo, snd_seq_event_t *ev)
  * the echo event is processed here.
  */
 int
-snd_seq_oss_event_input(snd_seq_event_t *ev, int direct, void *private_data)
+snd_seq_oss_event_input(snd_seq_event_t *ev, int direct, void *private_data,
+			int atomic, int hop)
 {
 	seq_oss_devinfo_t *dp = (seq_oss_devinfo_t *)private_data;
+	evrec_t *rec;
 
 	if (ev->type != SND_SEQ_EVENT_ECHO)
 		return snd_seq_oss_midi_input(ev, direct, private_data);
 
-	if (ev->data.raw32.d[1]) {
+	if (ev->source.client != dp->cseq)
+		return 0; /* ignored */
+
+	rec = (evrec_t*)&ev->data;
+	if (rec->s.code == SND_OSS_SEQ_SYNCTIMER) {
 		/* sync echo back */
-		snd_seq_oss_writeq_wakeup(dp->writeq, ev->data.raw32.d[0]);
+		snd_seq_oss_writeq_wakeup(dp->writeq, rec->t.time);
 		
 	} else {
 		/* echo back event */
-		evrec_t rec;
 		if (dp->readq == NULL)
 			return 0;
-		rec.echo = (ev->data.raw32.d[0] << 8) | SND_OSS_SEQ_ECHO;
-		snd_seq_oss_readq_put_event(dp->readq, &rec);
+		snd_seq_oss_readq_put_event(dp->readq, rec);
 	}
 	return 0;
 }
