@@ -56,7 +56,7 @@ snd_seq_oss_read(seq_oss_devinfo_t *dp, char *buf, int count)
 	cnt = count;
 	pos = 0;
 	q = snd_seq_oss_readq_pick(readq, !is_nonblock_mode(dp->file_mode), &flags);
-	if (! q)
+	if (q == NULL)
 		return 0;
 	do {
 		int ev_len;
@@ -87,7 +87,7 @@ snd_seq_oss_write(seq_oss_devinfo_t *dp, const char *buf, int count, struct file
 	int rc, c, p, ev_size;
 	evrec_t rec;
 
-	if (! is_write_mode(dp->file_mode) || ! dp->writeq)
+	if (! is_write_mode(dp->file_mode) || dp->writeq == NULL)
 		return -EIO;
 
 	c = count;
@@ -105,6 +105,9 @@ snd_seq_oss_write(seq_oss_devinfo_t *dp, const char *buf, int count, struct file
 		}
 		if (ev_is_long(&rec)) {
 			/* extended code */
+			if (rec.s.code == SND_OSS_SEQ_EXTENDED &&
+			    dp->seq_mode == SND_SEQ_OSS_MODE_MUSIC)
+				return -EINVAL;
 			ev_size = LONG_EVENT_SIZE;
 			if (c < ev_size)
 				break;
@@ -116,6 +119,8 @@ snd_seq_oss_write(seq_oss_devinfo_t *dp, const char *buf, int count, struct file
 
 		} else {
 			/* old-type code */
+			if (dp->seq_mode == SND_SEQ_OSS_MODE_MUSIC)
+				return -EINVAL;
 			ev_size = SHORT_EVENT_SIZE;
 		}
 
@@ -156,13 +161,13 @@ insert_queue(seq_oss_devinfo_t *dp, evrec_t *rec, struct file *opt)
 		return 0; /* invalid event - no need to insert queue */
 
 	event.time.tick = snd_seq_oss_timer_cur_tick(dp->timer);
-	if (snd_seq_oss_timer_is_realtime(dp->timer)) {
-		snd_seq_oss_dispatch(dp, &event, 0);
+	if (dp->timer->realtime || !dp->timer->running) {
+		snd_seq_oss_dispatch(dp, &event, 0, 0);
 	} else {
 		if (is_nonblock_mode(dp->file_mode))
-			rc = snd_seq_kernel_client_enqueue(dp->cseq, &event, 0);
+			rc = snd_seq_kernel_client_enqueue(dp->cseq, &event, 0, 0);
 		else
-			rc = snd_seq_kernel_client_enqueue_blocking(dp->cseq, &event, opt, 0);
+			rc = snd_seq_kernel_client_enqueue_blocking(dp->cseq, &event, opt, 0, 0);
 	}
 	free_sysex_data(&event);
 	return rc;
@@ -172,8 +177,7 @@ insert_queue(seq_oss_devinfo_t *dp, evrec_t *rec, struct file *opt)
 static void
 free_sysex_data(snd_seq_event_t *ev)
 {
-	if ((ev->flags & SND_SEQ_EVENT_LENGTH_MASK) == SND_SEQ_EVENT_LENGTH_VARIABLE &&
-	    ev->data.ext.ptr && ev->data.ext.len)
+	if (snd_seq_ev_is_variable(ev) && ev->data.ext.ptr)
 		snd_seq_ext_free(ev->data.ext.ptr, ev->data.ext.len);
 }
 
