@@ -33,6 +33,15 @@ Possible options for midisynth module:
 #include "../../include/seq_device.h"
 #include "../../include/seq_midi_event.h"
 
+MODULE_AUTHOR("Frank van de Pol <frank@vande-pol.demon.nl>");
+MODULE_DESCRIPTION("Advanced Linux Sound Architecture sequencer MIDI synth.");
+MODULE_SUPPORTED_DEVICE("sound");
+int output_buffer_size = PAGE_SIZE;
+MODULE_PARM(output_buffer_size, "i");
+MODULE_PARM_DESC(output_buffer_size, "Output buffer size in bytes.");
+int input_buffer_size = PAGE_SIZE;
+MODULE_PARM(input_buffer_size, "i");
+MODULE_PARM_DESC(input_buffer_size, "Input buffer size in bytes.");
 
 /* data for this midi synth driver */
 typedef struct {
@@ -129,16 +138,20 @@ static int event_process_midi(snd_seq_event_t * ev, int direct,
 			snd_printd("seq_midi: invalid sysex event flags = 0x%x\n", ev->flags);
 			return 0;
 		}
-		if (dump_midi(rmidi, (char *)ev->data.ext.ptr, ev->data.ext.len) < 0)
+		if ((res = dump_midi(rmidi, (char *)ev->data.ext.ptr, ev->data.ext.len)) < 0) {
 			snd_midi_event_reset_decode(msynth->parser);
+			return res;
+		}
 	} else {
 		if (msynth->parser == NULL)
-			return 0;	/* ignore */
+			return -EIO;
 		res = snd_midi_event_decode(msynth->parser, msg, sizeof(msg), ev);
 		if (res < 0)
-			return 0;
-		if (dump_midi(rmidi, msg, res) < 0)
+			return res;
+		if ((res = dump_midi(rmidi, msg, res)) < 0) {
 			snd_midi_event_reset_decode(msynth->parser);
+			return res;
+		}
 	}
 	return 0;
 }
@@ -170,6 +183,7 @@ static int midisynth_subscribe(void *private_data, snd_seq_port_subscribe_t *inf
 {
 	int err;
 	seq_midisynth_t *msynth = (seq_midisynth_t *)private_data;
+	snd_rawmidi_params_t params;
 
 	MOD_INC_USE_COUNT;
 
@@ -185,6 +199,14 @@ static int midisynth_subscribe(void *private_data, snd_seq_port_subscribe_t *inf
 		MOD_DEC_USE_COUNT;
 		return err;
 	}
+	memset(&params, 0, sizeof(params));
+	params.min = 1;
+	params.size = input_buffer_size;
+	if ((err = snd_rawmidi_input_params(msynth->input_rmidi, &params)) < 0) {
+		snd_rawmidi_kernel_release(msynth->input_rmidi, SND_RAWMIDI_LFLG_INPUT);
+		MOD_DEC_USE_COUNT;
+		return err;
+	}
 	msynth->input_rmidi->chn[SND_RAWMIDI_CHANNEL_INPUT].event = snd_midi_input_event;
 	msynth->input_rmidi->chn[SND_RAWMIDI_CHANNEL_INPUT].private_data = msynth;
 	snd_rawmidi_kernel_read(msynth->input_rmidi, NULL, 0);
@@ -209,6 +231,7 @@ static int midisynth_use(void *private_data, snd_seq_port_subscribe_t *info)
 {
 	int err;
 	seq_midisynth_t *msynth = (seq_midisynth_t *)private_data;
+	snd_rawmidi_params_t params;
 
 	MOD_INC_USE_COUNT;
 
@@ -224,6 +247,15 @@ static int midisynth_use(void *private_data, snd_seq_port_subscribe_t *info)
 		MOD_DEC_USE_COUNT;
 		return err;
 	}
+	memset(&params, 0, sizeof(params));
+	params.room = 1;
+	params.max = output_buffer_size - 1;
+	params.size = output_buffer_size;
+	if ((err = snd_rawmidi_output_params(msynth->output_rmidi, &params)) < 0) {
+		snd_rawmidi_kernel_release(msynth->output_rmidi, SND_RAWMIDI_LFLG_OUTPUT);
+		MOD_DEC_USE_COUNT;
+		return err;
+	}
 	return 0;
 }
 
@@ -417,8 +449,3 @@ void __exit cleanup_module(void)
 }
 
 #endif
-
-
-MODULE_AUTHOR("Frank van de Pol <frank@vande-pol.demon.nl>");
-MODULE_DESCRIPTION("Advanced Linux Sound Architecture sequencer MIDI synth.");
-MODULE_SUPPORTED_DEVICE("sound");
