@@ -1,6 +1,6 @@
 /*
  *   ALSA sequencer Priority Queue
- *   Copyright (c) 1998 by Frank van de Pol <frank@vande-pol.demon.nl>
+ *   Copyright (c) 1998-1999 by Frank van de Pol <frank@vande-pol.demon.nl>
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -105,12 +105,43 @@ void snd_seq_prioq_delete(prioq_t **fifo)
 /* return 1 if a >= b; 0 */
 static inline int compare_timestamp(snd_seq_event_t * a, snd_seq_event_t * b)
 {
-	if ((a->flags & SND_SEQ_TIME_STAMP_MASK) == SND_SEQ_TIME_STAMP_REAL) {
+	if ((a->flags & SND_SEQ_TIME_STAMP_MASK) == SND_SEQ_TIME_STAMP_TICK) {
 		/* compare ticks */
 		return (snd_seq_compare_tick_time(&a->time.tick, &b->time.tick));
 	} else {
 		/* compare real time */
-		return (snd_seq_compare_real_time(&a->time.real, &b->time.real));
+		return (snd_seq_compare_real_time(&a->time.time, &b->time.time));
+	}
+}
+
+/* compare timestamp between events */
+/* return negative if a < b;
+ *        zero     if a = b;
+ *        positive if a > b;
+ */
+static inline int compare_timestamp_rel(snd_seq_event_t *a, snd_seq_event_t *b)
+{
+	if ((a->flags & SND_SEQ_TIME_STAMP_MASK) == SND_SEQ_TIME_STAMP_TICK) {
+		/* compare ticks */
+		if (a->time.tick > b->time.tick)
+			return 1;
+		else if (a->time.tick == b->time.tick)
+			return 0;
+		else
+			return -1;
+	} else {
+		/* compare real time */
+		if (a->time.time.tv_sec > b->time.time.tv_sec)
+			return 1;
+		else if (a->time.time.tv_sec == b->time.time.tv_sec) {
+			if (a->time.time.tv_nsec > b->time.time.tv_nsec)
+				return 1;
+			else if (a->time.time.tv_nsec == b->time.time.tv_nsec)
+				return 0;
+			else
+				return -1;
+		} else
+			return -1;
 	}
 }
 
@@ -119,6 +150,7 @@ void snd_seq_prioq_cell_in(prioq_t * f, snd_seq_event_cell_t * cell)
 {
 	snd_seq_event_cell_t *cur, *prev;
 	unsigned long flags;
+	int prior;
 
 	if (f == NULL) {
 		snd_printd("oops: snd_seq_prioq_cell_in() called with NULL prioq\n");
@@ -129,12 +161,15 @@ void snd_seq_prioq_cell_in(prioq_t * f, snd_seq_event_cell_t * cell)
 		return;
 	}
 	
+	/* check flags */
+	prior = (cell->event.flags & SND_SEQ_PRIORITY_MASK);
+
 	spin_lock_irqsave(&f->lock, flags);
 
 	/* check if this element needs to inserted at the end (ie. ordered 
 	   data is inserted) This will be very likeley if a sequencer 
 	   application or midi file player is feeding us (sequential) data */
-	if (f->tail) {
+	if (f->tail && !prior) {
 		if (compare_timestamp(&cell->event, &f->tail->event)) {
 			/* add new cell to tail of the fifo */
 			f->tail->ptr_l = cell;
@@ -153,36 +188,28 @@ void snd_seq_prioq_cell_in(prioq_t * f, snd_seq_event_cell_t * cell)
 
 	while (cur != NULL) {
 		/* compare timestamps */
-		if (!compare_timestamp(&cell->event, &cur->event)) {
+		int rel = compare_timestamp_rel(&cell->event, &cur->event);
+		if (rel < 0)
 			/* new cell has earlier schedule time, */
-			/* insert it before cursor */
-
-			if (prev != NULL)
-				prev->ptr_l = cell;
-			cell->ptr_l = cur;
-
-			/* this is the first cell, set head to it */
-			if (f->head == cur)
-				f->head = cell;
-
-			f->cells++;
-			spin_unlock_irqrestore(&f->lock, flags);
-			return;
-		} else {
-			/* new cell has equal or larger schedule time, */
-			/* move cursor to next cell */
-			prev = cur;
-			cur = cur->ptr_l;
-		}
+			break;
+		else if (rel == 0 && prior)
+			/* equal schedule time and prior to others */
+			break;
+		/* new cell has equal or larger schedule time, */
+		/* move cursor to next cell */
+		prev = cur;
+		cur = cur->ptr_l;
 	}
 
-	/* reached end of the list */
+	/* insert it before cursor */
 	if (prev != NULL)
 		prev->ptr_l = cell;
-	cell->ptr_l = NULL;
-	f->tail = cell;
-	if (f->head == NULL)
+	cell->ptr_l = cur;
+
+	if (f->head == cur) /* this is the first cell, set head to it */
 		f->head = cell;
+	if (cur == NULL) /* reached end of the list */
+		f->tail = cell;
 	f->cells++;
 	spin_unlock_irqrestore(&f->lock, flags);
 }
@@ -250,8 +277,8 @@ static inline int prioq_match(snd_seq_event_cell_t *cell, int client, int timest
 			return 1;
 		break;
 	case SND_SEQ_TIME_STAMP_REAL:
-		if (cell->event.time.real.tv_sec ||
-		    cell->event.time.real.tv_nsec)
+		if (cell->event.time.time.tv_sec ||
+		    cell->event.time.time.tv_nsec)
 			return 1;
 		break;
 	}
@@ -283,7 +310,7 @@ void snd_seq_prioq_leave(prioq_t * f, int client, int timestamp)
 			f->cells--;
 			/* add cell to free list */
 			cell->ptr_l = NULL;
-			if (!freefirst) {
+			if (freefirst == NULL) {
 				freefirst = cell;
 			} else {
 				freeprev->ptr_l = cell;
@@ -310,3 +337,113 @@ void snd_seq_prioq_leave(prioq_t * f, int client, int timestamp)
 		freefirst = freenext;
 	}
 }
+
+static int prioq_remove_match(snd_seq_remove_events_t *info,
+	snd_seq_event_t *ev)
+{
+	int res;
+
+	if (info->remove_mode & SND_SEQ_REMOVE_DEST) {
+		if (ev->dest.client != info->dest.client ||
+				ev->dest.port != info->dest.port)
+			return 0;
+	}
+	if (info->remove_mode & SND_SEQ_REMOVE_DEST_CHANNEL) {
+		if (! snd_seq_ev_is_channel_type(ev))
+			return 0;
+		/* data.note.channel and data.control.channel are identical */
+		if (ev->data.note.channel != info->channel)
+			return 0;
+	}
+	if (info->remove_mode & SND_SEQ_REMOVE_TIME_AFTER) {
+		if (info->tick)
+			res = snd_seq_compare_tick_time(&ev->time.tick, &info->time.tick);
+		else
+			res = snd_seq_compare_real_time(&ev->time.time, &info->time.time);
+		if (!res)
+			return 0;
+	}
+	if (info->remove_mode & SND_SEQ_REMOVE_TIME_BEFORE) {
+		if (info->tick)
+			res = snd_seq_compare_tick_time(&ev->time.tick, &info->time.tick);
+		else
+			res = snd_seq_compare_real_time(&ev->time.time, &info->time.time);
+		if (res)
+			return 0;
+	}
+	if (info->remove_mode & SND_SEQ_REMOVE_EVENT_TYPE) {
+		if (ev->type != info->type)
+			return 0;
+	}
+	if (info->remove_mode & SND_SEQ_REMOVE_IGNORE_OFF) {
+		/* Do not remove off events */
+		switch (ev->type) {
+		case SND_SEQ_EVENT_NOTEOFF:
+		/* case SND_SEQ_EVENT_SAMPLE_STOP: */
+			return 0;
+		default:
+			break;
+		}
+	}
+	if (info->remove_mode & SND_SEQ_REMOVE_TAG_MATCH) {
+		if (info->tag != ev->tag)
+			return 0;
+	}
+
+	return 1;
+}
+
+/* remove cells matching remove criteria */
+void snd_seq_prioq_remove_events(prioq_t * f, int client,
+	snd_seq_remove_events_t *info)
+{
+	register snd_seq_event_cell_t *cell, *next;
+	unsigned long flags;
+	snd_seq_event_cell_t *prev = NULL;
+	snd_seq_event_cell_t *freefirst = NULL, *freeprev = NULL, *freenext;
+
+	/* collect all removed cells */
+	spin_lock_irqsave(&f->lock, flags);
+	cell = f->head;
+
+	while (cell) {
+		next = cell->ptr_l;
+		if (cell->event.source.client == client &&
+			prioq_remove_match(info, &cell->event)) {
+
+			/* remove cell from prioq */
+			if (cell == f->head) {
+				f->head = cell->ptr_l;
+			} else {
+				prev->ptr_l = cell->ptr_l;
+			}
+
+			if (cell == f->tail)
+				f->tail = cell->ptr_l;
+			f->cells--;
+
+			/* add cell to free list */
+			cell->ptr_l = NULL;
+			if (freefirst == NULL) {
+				freefirst = cell;
+			} else {
+				freeprev->ptr_l = cell;
+			}
+
+			freeprev = cell;
+		} else {
+			prev = cell;
+		}
+		cell = next;		
+	}
+	spin_unlock_irqrestore(&f->lock, flags);	
+
+	/* remove selected cells */
+	while (freefirst) {
+		freenext = freefirst->ptr_l;
+		snd_seq_cell_free(freefirst);
+		freefirst = freenext;
+	}
+}
+
+
