@@ -612,28 +612,30 @@ static int snd_emu10k1_playback_open(void *private_data,
 	emu10k1_pcm_t *epcm;
 	snd_pcm_runtime_t *runtime = subchn->runtime;
 	char *buf;
-	unsigned int start_addr;
 	int err;
+	snd_emux_memblk_t *memblk;
 
 	if ((err = snd_pcm_dma_alloc(subchn,
 				     emu->dma1ptr,
 				     "EMU10K1 (playback)")) < 0)
 		return err;
 	buf = runtime->dma_area->buf;
-	if ((err = snd_emu10k1_ptb_alloc(emu, buf, runtime->dma_area->size, &start_addr)) < 0) {
+	memblk = snd_emu10k1_alloc_pages(emu, buf, runtime->dma_area->size);
+	if (memblk == NULL) {
 		snd_pcm_dma_free(subchn);
-		return err;
+		return -ENOMEM;
 	}
 	epcm = snd_magic_kcalloc(emu10k1_pcm_t, 0, GFP_KERNEL);
 	if (epcm == NULL) {
-		snd_emu10k1_ptb_free(emu, buf, NULL);
+		snd_emu10k1_free_pages(emu, memblk);
 		snd_pcm_dma_free(subchn);
 		return -ENOMEM;
 	}
 	epcm->emu = emu;
 	epcm->type = PLAYBACK_EMUVOICE;
 	epcm->subchn = subchn;
-	epcm->start_addr = start_addr;
+	epcm->memblk = memblk;
+	epcm->start_addr = memblk->offset;
 	// emu10k1->pcm_mixer[subchn->number].number = emu->number;
 	// snd_emu10k1_pcm_mixer_build(emu, subchn->number);
 	/* FIXME: */
@@ -648,9 +650,14 @@ static int snd_emu10k1_playback_close(void *private_data,
 				      snd_pcm_subchn_t * subchn)
 {
 	emu10k1_t *emu = snd_magic_cast(emu10k1_t, private_data, -ENXIO);
-
+	snd_pcm_runtime_t *runtime = subchn->runtime;
+	emu10k1_pcm_t *epcm;
 	// snd_emu10k1_pcm_mixer_free(emu, subchn->number);
-	snd_emu10k1_ptb_free(emu, subchn->runtime->dma_area->buf, NULL);
+	if (runtime && runtime->private_data) {
+		epcm = snd_magic_cast(emu10k1_pcm_t, runtime->private_data, -ENXIO);
+		if (epcm->memblk)
+			snd_emu10k1_free_pages(emu, epcm->memblk);
+	}
 	snd_pcm_dma_free(subchn);
 	return 0;
 }
