@@ -314,7 +314,7 @@ static int snd_emu10k1_init(emu10k1_t * emu)
 
 	snd_emu10k1_intr_enable(emu, INTE_PCIERRORENABLE);
 
-	emu->reserved_page = snd_emu10k1_synth_malloc(emu, 4096, &emu->reserved_page_addr);
+	emu->reserved_page = snd_emu10k1_synth_alloc(emu, 4096);
 	
 	return 0;
 }
@@ -558,6 +558,8 @@ static int snd_emu10k1_free(emu10k1_t * emu)
 {
 	snd_emu10k1_proc_done(emu);
 	snd_emu10k1_done(emu);
+	if (emu->memhdr)
+		snd_emux_memhdr_free(emu->memhdr);
 	if (emu->silent_page)
 		snd_free_pages(emu->silent_page, EMUPAGESIZE);
 	if (emu->ptb_pages)
@@ -612,6 +614,12 @@ int snd_emu10k1_create(snd_card_t * card,
 		snd_emu10k1_free(emu);
 		return -ENOMEM;
 	}
+	emu->memhdr = snd_emux_memhdr_new(EMUPAGESIZE * MAXPAGES);
+	if (emu->memhdr == NULL) {
+		snd_emu10k1_free(emu);
+		return -ENOMEM;
+	}
+	emu->memhdr->block_extra_size = sizeof(snd_emu10k1_memblk_arg_t);
 
 	pci_set_master(pci);
 	pci_read_config_word(pci, PCI_COMMAND, &cmdw);
