@@ -302,9 +302,9 @@ static int snd_ak4531_mono_out_sw(snd_kmixer_element_t *element, int w_flag,
 }                                                                                                                                                                                                                                                                                                            
 
 #define AK4531_MONO_OUT_MUTE(name, reg, bit) \
-	static int snd_ak4531_out_mute_##name(snd_kmixer_element_t *element, int w_flag, int *value) \
+	static int snd_ak4531_out_mute_##name(snd_kmixer_element_t *element, int w_flag, unsigned int *bitmap) \
 	{ \
-		return snd_ak4531_mono_out_sw(element, w_flag, value, reg, bit); \
+		return snd_ak4531_mono_out_sw(element, w_flag, bitmap, reg, bit); \
 	}
 
 AK4531_MONO_OUT_MUTE(mono1, AK4531_OUT_SW2, 0)
@@ -729,8 +729,8 @@ static int snd_ak4531_mixer_group_ctrl4(snd_kmixer_group_t * group,
 					int max,
 					snd_mixer_sw2_control_t *sw2,
 					snd_kmixer_element_t *sw2_element,
-					snd_mixer_sw2_control_t *sw2_out,
-					snd_kmixer_element_t *sw2_out_element,
+					snd_mixer_sw1_control_t *sw1_out,
+					snd_kmixer_element_t *sw1_out_element,
 					snd_mixer_sw1_control_t *sw1_in,
 					snd_kmixer_element_t *sw1_in_element)
 {
@@ -751,13 +751,13 @@ static int snd_ak4531_mixer_group_ctrl4(snd_kmixer_group_t * group,
 			ugroup->min = 0;
 			ugroup->max = max;
 		}
-		if (sw2_out) {
+		if (sw1_out) {
 			ugroup->caps |= SND_MIXER_GRPCAP_MUTE;
-			sw2_out(sw2_out_element, 0, &value);
-			if (!value) {
+			sw1_out(sw1_out_element, 0, &bitmap);
+			if (!snd_mixer_get_bit(&bitmap, 0))
 				ugroup->mute |= SND_MIXER_CHN_MASK_FRONT_LEFT;
+			if (!snd_mixer_get_bit(&bitmap, 1))
 				ugroup->mute |= SND_MIXER_CHN_MASK_FRONT_RIGHT;
-			}
 		}
 		if (sw1_in) {
 			ugroup->caps |= SND_MIXER_GRPCAP_CAPTURE;
@@ -773,6 +773,8 @@ static int snd_ak4531_mixer_group_ctrl4(snd_kmixer_group_t * group,
 			if (!value) {
 				ugroup->mute |= SND_MIXER_CHN_MASK_FRONT_LEFT;
 				ugroup->capture &= ~SND_MIXER_CHN_MASK_FRONT_LEFT;
+			}
+			if (!value) {
 				ugroup->mute |= SND_MIXER_CHN_MASK_FRONT_RIGHT;
 				ugroup->capture &= ~SND_MIXER_CHN_MASK_FRONT_RIGHT;
 			}
@@ -787,31 +789,23 @@ static int snd_ak4531_mixer_group_ctrl4(snd_kmixer_group_t * group,
 		}
 		if (sw2) {
 			value = 0;
-			if (!(ugroup->mute & SND_MIXER_CHN_MASK_STEREO)) {
+			if (!(ugroup->mute & SND_MIXER_CHN_MASK_STEREO))
 				value = 1;
-			} else if ((ugroup->mute & SND_MIXER_CHN_MASK_STEREO) == SND_MIXER_CHN_MASK_STEREO) {
-				value = 0;
-			} else {
-				value ^= 1;
-			}
-			if (!(ugroup->capture & SND_MIXER_CHN_MASK_STEREO)) {
-				value = 0;
-			} else if ((ugroup->capture & SND_MIXER_CHN_MASK_STEREO) == SND_MIXER_CHN_MASK_STEREO) {
+			if (ugroup->capture & SND_MIXER_CHN_MASK_STEREO)
 				value = 1;
-			} else {
-				value ^= 1;
-			}
-			if (sw2(sw2_element, 1, &value) > 0) {
+			if (sw2 && sw2(sw2_element, 1, &value) > 0) {
 				snd_mixer_element_value_change(file, sw2_element, 0);
 				change = 1;
 			}
 		}
-		if (sw2_out) {
-			value = 0;
-			if (!(ugroup->mute & SND_MIXER_CHN_MASK_STEREO))
-				value = 1;
-			if (sw2_out(sw2_out_element, 1, &value) > 0) {
-				snd_mixer_element_value_change(file, sw2_out_element, 0);
+		if (sw1_out) {
+			bitmap = 0;
+			if (!(ugroup->mute & SND_MIXER_CHN_MASK_FRONT_LEFT))
+				snd_mixer_set_bit(&bitmap, 0, 1);
+			if (!(ugroup->mute & SND_MIXER_CHN_MASK_FRONT_RIGHT))
+				snd_mixer_set_bit(&bitmap, 1, 1);
+			if (sw1_out && sw1_out(sw1_out_element, 1, &bitmap) > 0) {
+				snd_mixer_element_value_change(file, sw1_out_element, 0);
 				change = 1;
 			}
 		}
@@ -1055,9 +1049,12 @@ static int snd_ak4531_set_mix_switch(snd_kmixer_t * mixer,
 static snd_kswitch_t snd_ak4531_switches[] =
 {
 	{
-		name:   "AK4531 Recording Source",
-		get:    (snd_get_switch_t *)snd_ak4531_get_mix_switch,
-		set:	(snd_set_switch_t *)snd_ak4531_set_mix_switch,
+		"AK4531 Recording Source",
+		(snd_get_switch_t *)snd_ak4531_get_mix_switch,
+		(snd_set_switch_t *)snd_ak4531_set_mix_switch,
+		0,
+		NULL,
+		NULL
 	},
 };
 
@@ -1109,7 +1106,7 @@ int snd_ak4531_mixer(snd_card_t * card, int device,
 	if (ak4531 == NULL)
 		return -ENOMEM;
 	memcpy(ak4531, _ak4531, sizeof(*_ak4531));
-	spin_lock_init(&ak4531->reg_lock);
+	ak4531->reg_lock = SPIN_LOCK_UNLOCKED;
 	if ((err = snd_mixer_new(card, "AK4531", device, &mixer)) < 0)
 		return err;
 	strcpy(mixer->name, "Asahi Kasei AK4531");
@@ -1363,7 +1360,7 @@ int snd_ak4531_mixer(snd_card_t * card, int device,
 		goto __error;
 	if (snd_mixer_element_route_add(mixer, ak4531->me_vol_mono, ak4531->me_sw_mono) < 0)
 		goto __error;
-	if ((ak4531->me_sw_mono_out = snd_mixer_lib_sw2(mixer, "Mono Output Switch", 0, snd_ak4531_out_mute_mono1, ak4531)) == NULL)
+	if ((ak4531->me_sw_mono_out = snd_mixer_lib_sw1(mixer, "Mono Output Switch", 0, 2, snd_ak4531_out_mute_mono1, ak4531)) == NULL)
 		goto __error;
 	if (snd_mixer_group_element_add(mixer, group, ak4531->me_sw_mono_out) < 0)
 		goto __error;
@@ -1406,7 +1403,7 @@ int snd_ak4531_mixer(snd_card_t * card, int device,
 		goto __error;
 	if (snd_mixer_element_route_add(mixer, ak4531->me_vol_mono1, ak4531->me_sw_mono1) < 0)
 		goto __error;
-	if ((ak4531->me_sw_mono1_out = snd_mixer_lib_sw2(mixer, "Mono Output Switch", 1, snd_ak4531_out_mute_mono2, ak4531)) == NULL)
+	if ((ak4531->me_sw_mono1_out = snd_mixer_lib_sw1(mixer, "Mono Output Switch", 1, 2, snd_ak4531_out_mute_mono2, ak4531)) == NULL)
 		goto __error;
 	if (snd_mixer_group_element_add(mixer, group, ak4531->me_sw_mono1_out) < 0)
 		goto __error;
@@ -1456,7 +1453,7 @@ int snd_ak4531_mixer(snd_card_t * card, int device,
 		goto __error;
 	if (snd_mixer_element_route_add(mixer, ak4531->me_vol_mic, ak4531->me_sw_mic) < 0)
 		goto __error;
-	if ((ak4531->me_sw_mic_out = snd_mixer_lib_sw2(mixer, "MIC Output Switch", 0, snd_ak4531_out_mute_mic, ak4531)) == NULL)
+	if ((ak4531->me_sw_mic_out = snd_mixer_lib_sw1(mixer, "MIC Output Switch", 0, 2, snd_ak4531_out_mute_mic, ak4531)) == NULL)
 		goto __error;
 	if (snd_mixer_group_element_add(mixer, group, ak4531->me_sw_mic_out) < 0)
 		goto __error;
@@ -1537,15 +1534,15 @@ EXPORT_SYMBOL(snd_ak4531_mixer);
  *  INIT part
  */
 
-static int __init alsa_ak4531_init(void)
+#ifdef MODULE
+
+int __init init_module(void)
 {
 	return 0;
 }
 
-static void __exit alsa_ak4531_exit(void)
+void __exit cleanup_module(void)
 {
 }
 
-module_init(alsa_ak4531_init)
-module_exit(alsa_ak4531_exit)
-MODULE_LICENSE("GPL");
+#endif
