@@ -15,7 +15,7 @@
  *      ported to sound driver version 3.5.4
  *      added support for setting divisor from configuration
  *
- * Fri Jan 8 1999 yamahata@kusm.kyoto-u.ac.jp
+ * Fri Jan 8 1999 yamahata@private.email.ne.jp
  *     ported to Advanced Linux Sound Archtecture
  *     first version 
  *     - added SERIAL* definitions.
@@ -46,6 +46,11 @@
  *       snd_uart16550_free
  *       snd_uart16550_new_device
  *
+ *   Sun Jun 7 1999 yamahata@private.email.ne.jp
+ *   fixed a bug in snd_uart16550_output_write.
+ *   (When the buffer is full, midi data will be lost)
+ *   deleted snd_uart16550_out.
+ *   
  *
  *   the serial-midi patch for Open Sound System is got from
  *   http://crystal.apana.org.au/~ghansper/midi/uart16550.3a-s354.tar.gz
@@ -75,12 +80,12 @@
 
 
 #define SND_MAIN_OBJECT_FILE
-#include "driver.h"
-#include "midi.h"
+#include "../../include/driver.h"
+#include "../../include/rawmidi.h"
 
 #include <linux/serial_reg.h>
 
-#include "uart16550.h"
+#include "../../include/uart16550.h"
 #include "uart16550_type.h"
 
 #define SERIAL_MODE_NOT_OPENED (0)
@@ -93,48 +98,6 @@
 #define SERIAL_TRUE (1)
 #define SERIAL_FALSE (0)
 
-#if 0
-#define TX_BUFF_SIZE 32		/* Must be 2^n */
-#define TX_BUFF_MASK  (TX_BUFF_SIZE - 1)
-
-typedef struct snd_stru_uart16550 {
-	snd_card_t *card;
-	snd_rawmidi_t *rmidi;
-	snd_seq_uart16550_client_t *seq_client;
-
-	int filemode;		//open status of file
-
-	int polled;
-	 snd_spin_define(open);
-	//snd_spin_define (input);
-	//snd_spin_define (output);
-
-	unsigned short irq_number;
-	snd_irq_t *irq;
-
-	unsigned short base;
-	//snd_port_t* port;
-	//unsigned short size;
-
-	unsigned char divisor;
-
-	unsigned char old_divisor_lsb;
-	unsigned char old_divisor_msb;
-	unsigned char old_line_ctrl_reg;
-
-	//parameter for using of write loop
-	short int fifo_limit;	//used in uart16550
-
-	short int fifo_count;	//used in uart16550
-
-	//write buffer and its writing/reading position
-	char tx_buff[TX_BUFF_SIZE];
-	short int buff_in;
-	short int buff_out;
-} snd_uart16550_t;
-
-#endif
-
 ////////////////////////////////////////////////////////////snd_uart16550_t
 static snd_uart16550_t *snd_uart16550_t_new(snd_card_t * card,
 					    snd_rawmidi_t * rmidi)
@@ -145,7 +108,7 @@ static snd_uart16550_t *snd_uart16550_t_new(snd_card_t * card,
 	if (card == NULL || rmidi == NULL)
 		return NULL;
 
-	uart = snd_calloc(sizeof(snd_uart16550_t));
+	uart = snd_magic_kcalloc(snd_uart16550_t, 0, GFP_KERNEL);
 	if (uart == NULL)
 		return NULL;
 
@@ -154,9 +117,7 @@ static snd_uart16550_t *snd_uart16550_t_new(snd_card_t * card,
 	uart->seq_client = NULL;
 	uart->filemode = 0;
 	uart->irq = NULL;
-	snd_spin_prepare(uart, open);
-	//snd_spin_prepare (uart, input);
-	//snd_spin_prepare (uart, output);
+	uart->open_lock = SPIN_LOCK_UNLOCKED;
 
 	return uart;
 }
@@ -166,7 +127,7 @@ static void snd_uart16550_t_free(snd_uart16550_t * uart)
 	if (uart == NULL)
 		return;
 
-	snd_free(uart, sizeof(snd_uart16550_t));
+	snd_magic_kfree(uart);
 }
 
 ///////////////////////////////////////////////////////////interrupt/polling
@@ -188,7 +149,7 @@ static void snd_uart16550_io_loop(snd_uart16550_t * uart)
 		/* while receive data ready */
 		c = inb(uart->base + UART_RX);
 		if (uart->filemode & SERIAL_MODE_INPUT_OPEN) {
-			uart->rmidi->input.data(uart->rmidi, &c, 1);
+			snd_rawmidi_receive(uart->rmidi, &c, 1);
 		}
 		if (status & UART_LSR_OE)
 			printk("%s: Overrun on device at %x\n",
@@ -243,14 +204,14 @@ static void snd_uart16550_interrupt(int irq, void *dev_id,
 	unsigned long flags;
 
 	uart = (snd_uart16550_t *) dev_id;
-	snd_spin_lock(uart, open, &flags);
+	spin_lock_irqsave(&uart->open_lock, flags);
 	if (uart->filemode == SERIAL_MODE_NOT_OPENED) {
-		snd_spin_unlock(uart, open, &flags);
+		spin_unlock_irqrestore(&uart->open_lock, flags);
 		return;
 	}
 	c = inb(uart->base + UART_IIR);		/* indicate to the UART that the interupt has been serviced */
 	snd_uart16550_io_loop(uart);
-	snd_spin_unlock(uart, open, &flags);
+	spin_unlock_irqrestore(&uart->open_lock, flags);
 }
 
 
@@ -263,12 +224,11 @@ static int snd_uart16550_do_set_param(snd_uart16550_t * uart,
 	unsigned long flags;
 
 	//snd_printd ("snd_uart16550_set_param called\n");
-	if (uart == NULL)
-		return -EINVAL;
+	snd_debug_check(uart == NULL, -EINVAL);
 
-	snd_spin_lock(uart, open, &flags);
+	spin_lock_irqsave(&uart->open_lock, flags);
 	if (uart->filemode != SERIAL_MODE_NOT_OPENED) {
-		snd_spin_unlock(uart, open, &flags);
+		spin_unlock_irqrestore(&uart->open_lock, flags);
 		return -EBUSY;
 	}
 	uart->irq_number = irq_number;
@@ -276,7 +236,7 @@ static int snd_uart16550_do_set_param(snd_uart16550_t * uart,
 	uart->divisor = divisor;
 	uart->polled = polled;
 
-	snd_spin_unlock(uart, open, &flags);
+	spin_unlock_irqrestore(&uart->open_lock, flags);
 
 	return 0;
 }
@@ -288,7 +248,7 @@ int snd_uart16550_set_param(snd_rawmidi_t * rmidi,
 {
 	snd_uart16550_t *uart;
 
-	uart = (snd_uart16550_t *) rmidi->private_data;
+	uart = snd_magic_cast(snd_uart16550_t, rmidi->private_data, -ENXIO);
 	return snd_uart16550_do_set_param(uart,
 				    irq_number, iobase, divisor, polled);
 }
@@ -351,12 +311,10 @@ static int snd_uart16550_do_open(snd_uart16550_t * uart)
 	char byte;
 
 	//snd_printd ("snd_uart16550_do_open called\n");
-	if (uart == NULL)
-		return -EINVAL;
+	snd_debug_check(uart == NULL, -EINVAL);
 
 	if (uart->filemode != SERIAL_MODE_NOT_OPENED)	//no need?
-
-		return -EBUSY;
+		return -EAGAIN;
 
 	//snd_printd ("snd_uart16550_register_resources now setting up serial port\n");
 
@@ -473,19 +431,19 @@ static int snd_uart16550_do_close(snd_uart16550_t * uart)
 static int snd_uart16550_input_open(snd_rawmidi_t * rmidi)
 {
 	unsigned long flags;
-	snd_uart16550_t *uart = (snd_uart16550_t *) rmidi->private_data;
-	snd_spin_lock(uart, open, &flags);
+	snd_uart16550_t *uart = snd_magic_cast(snd_uart16550_t, rmidi->private_data, -ENXIO);
+	spin_lock_irqsave(&uart->open_lock, flags);
 	if (uart->filemode == SERIAL_MODE_NOT_OPENED) {
 		int ret;
 		ret = snd_uart16550_do_open(uart);
 		if (ret < 0) {
-			snd_spin_unlock(uart, open, &flags);
+			spin_unlock_irqrestore(&uart->open_lock, flags);
 			return ret;
 		}
 	}
 	uart->filemode |= SERIAL_MODE_INPUT_OPEN;
 	//snd_printd ("input_open uart->filemode=%d\n", uart->filemode);
-	snd_spin_unlock(uart, open, &flags);
+	spin_unlock_irqrestore(&uart->open_lock, flags);
 
 	return 0;
 }
@@ -493,12 +451,12 @@ static int snd_uart16550_input_open(snd_rawmidi_t * rmidi)
 static int snd_uart16550_input_close(snd_rawmidi_t * rmidi)
 {
 	unsigned long flags;
-	snd_uart16550_t *uart = (snd_uart16550_t *) rmidi->private_data;
-	snd_spin_lock(uart, open, &flags);
+	snd_uart16550_t *uart = snd_magic_cast(snd_uart16550_t, rmidi->private_data, -ENXIO);
+	spin_lock_irqsave(&uart->open_lock, flags);
 	uart->filemode &= ~SERIAL_MODE_INPUT_OPEN;
 	if (uart->filemode == SERIAL_MODE_NOT_OPENED)
 		snd_uart16550_do_close(uart);
-	snd_spin_unlock(uart, open, &flags);
+	spin_unlock_irqrestore(&uart->open_lock, flags);
 	return 0;
 }
 
@@ -508,15 +466,15 @@ static void snd_uart16550_input_trigger(snd_rawmidi_t * rmidi, int up)
 	snd_uart16550_t *uart;
 
 	snd_printd("snd_uart16550_input_trigger called\n");
-	uart = (snd_uart16550_t *) rmidi->private_data;
+	uart = snd_magic_cast(snd_uart16550_t, rmidi->private_data, );
 
-	snd_spin_lock(uart, open, &flags);
+	spin_lock_irqsave(&uart->open_lock, flags);
 	if (up) {
 		uart->filemode |= SERIAL_MODE_INPUT_TRIGGERED;
 	} else {
 		uart->filemode &= ~SERIAL_MODE_INPUT_TRIGGERED;
 	}
-	snd_spin_unlock(uart, open, &flags);
+	spin_unlock_irqrestore(&uart->open_lock, flags);
 }
 
 
@@ -524,121 +482,50 @@ static void snd_uart16550_input_trigger(snd_rawmidi_t * rmidi, int up)
 static int snd_uart16550_output_open(snd_rawmidi_t * rmidi)
 {
 	unsigned long flags;
-	snd_uart16550_t *uart = (snd_uart16550_t *) rmidi->private_data;
+	snd_uart16550_t *uart = snd_magic_cast(snd_uart16550_t, rmidi->private_data, -ENXIO);
 
 	//snd_printd ("snd_uart16550_output_open called\n");
 
-	snd_spin_lock(uart, open, &flags);
+	spin_lock_irqsave(&uart->open_lock, flags);
 	if (uart->filemode == SERIAL_MODE_NOT_OPENED) {
 		int ret;
 		ret = snd_uart16550_do_open(uart);
 		if (ret < 0) {
-			snd_spin_unlock(uart, open, &flags);
+			spin_unlock_irqrestore(&uart->open_lock, flags);
 			return ret;
 		}
 	}
 	uart->filemode |= SERIAL_MODE_OUTPUT_OPEN;
 	//snd_printd ("write_open uart->filemode=%d\n", uart->filemode);
-	snd_spin_unlock(uart, open, &flags);
+	spin_unlock_irqrestore(&uart->open_lock, flags);
 	return 0;
 };
 
 static int snd_uart16550_output_close(snd_rawmidi_t * rmidi)
 {
 	unsigned long flags;
-	snd_uart16550_t *uart = (snd_uart16550_t *) rmidi->private_data;
+	snd_uart16550_t *uart = snd_magic_cast(snd_uart16550_t, rmidi->private_data, -ENXIO);
 
 	//snd_printd ("snd_uart16550_output_close called\n");
-	snd_spin_lock(uart, open, &flags);
+	spin_lock_irqsave(&uart->open_lock, flags);
 	uart->filemode &= ~SERIAL_MODE_OUTPUT_OPEN;
 	if (uart->filemode == SERIAL_MODE_NOT_OPENED)
 		snd_uart16550_do_close(uart);
-	snd_spin_unlock(uart, open, &flags);
+	spin_unlock_irqrestore(&uart->open_lock, flags);
 	return 0;
 };
 
 
-/* This loop should be called with interrupts disabled
- * We don't want to interrupt this, 
- * as we're already handling an interupt 
- */
-//output 1byte
-//static int snd_uart16550_out (int dev, unsigned char midi_byte)
-static int snd_uart16550_out(snd_uart16550_t * uart, char midi_byte)
-{
-	unsigned char status;
-	short int buff_in;
-
-	//snd_printd ("snd_uart16550_out called\n");
-
-	if (uart->buff_in == uart->buff_out) {	/* Buffer empty? */
-		/* Tx Buffer Empty - try to write immediately */
-		status = inb(uart->base + UART_LSR);
-		if ((status & UART_LSR_THRE) != 0) {
-			/* Transmitter holding register (and Tx FIFO) empty */
-			uart->fifo_count = 1;
-			outb(midi_byte, uart->base + UART_TX);
-			return 1;
-		} else {
-			if (uart->fifo_count < uart->fifo_limit) {
-				uart->fifo_count++;
-				outb(midi_byte, uart->base + UART_TX);
-				return 1;
-			} else {
-				/* Cannot write (buffer empty) - put char in buffer */
-				buff_in = uart->buff_in;
-				uart->tx_buff[buff_in] = midi_byte;
-				buff_in++;
-				buff_in &= TX_BUFF_MASK;
-				uart->buff_in = buff_in;
-				return 1;
-			}
-		}
-	} else {
-		/* Tx Buffer NOT Empty - try to put char in buffer */
-		/* buffer is full is indicated by (buff_in+1 == buff_out)
-		 * This means that there is always at least 1 'dead' byte
-		 * in tx_buff.
-		 * It also means that it is always 'safe' to write into tx_buff[buff_in]
-		 * even if we can't use it.
-		 */
-		buff_in = uart->buff_in;
-		uart->tx_buff[buff_in] = midi_byte;
-		buff_in++;
-		buff_in &= TX_BUFF_MASK;
-		if (buff_in == uart->buff_out) {
-			/* printk("%s: Write Busy on device %d at %x\n",uart->my_name,uart->my_dev,uart->base); */
-#if 1
-			if (uart->polled)
-				snd_uart16550_io_loop(uart);	/* Feeble attempt to improve performance - but it helps */
-#endif
-			return 0;	/* Tx busy - try again later */
-		} else {
-			uart->buff_in = buff_in;
-#if 1
-			if (uart->polled)
-				snd_uart16550_io_loop(uart);	/* Feeble attempt to improve performance - but it helps */
-#endif
-			return 1;
-		}
-	}
-	/* Not reached */
-}
-
-
-//write 1byte.
 //uart = rmidi->private;
 //handling is differ by uart->polled.
 static void snd_uart16550_output_write(snd_rawmidi_t * rmidi)
 {
 	unsigned long flags;
-	unsigned char byte;
-	int max = 32;		//what is value is best???
-
-	snd_uart16550_t *uart;
-
-	uart = (snd_uart16550_t *) rmidi->private_data;
-
+	unsigned char midi_byte;
+	unsigned char status;
+	short int buff_in;
+	snd_uart16550_t *uart = snd_magic_cast(snd_uart16550_t, rmidi->private_data, );
+	
 	//snd_printd ("snd_uart16550_output_write called\n");
 
 	/* Interupts are disabled during the updating of the tx_buff,
@@ -646,17 +533,50 @@ static void snd_uart16550_output_write(snd_rawmidi_t * rmidi)
 	 * variables (ie buff_in & buff_out)
 	 */
 
-	snd_spin_lock(uart, open, &flags);
+	spin_lock_irqsave(&uart->open_lock, flags);
 	if (uart->polled) {	//polling
-
 		snd_uart16550_io_loop(uart);
 	}
-	while (max-- > 0) {
-		if (rmidi->output.data(rmidi, &byte, 1) != 1)
-			break;
-		snd_uart16550_out(uart, byte);
+
+	while (1) {
+	         /* buffer is full is indicated by (buff_in+1 == buff_out) */
+	        if (((uart->buff_in+1)&TX_BUFF_MASK) == uart->buff_out)
+		  break;
+		if (snd_rawmidi_transmit(rmidi, &midi_byte, 1) != 1)
+		  break;
+
+		if (uart->buff_in == uart->buff_out) {	/* Buffer empty? */
+		        /* Tx Buffer Empty - try to write immediately */
+		        status = inb(uart->base + UART_LSR);
+			if ((status & UART_LSR_THRE) != 0) {
+			        /* Transmitter holding register (and Tx FIFO) empty */
+			        uart->fifo_count = 1;
+				outb(midi_byte, uart->base + UART_TX);
+			} else {
+			        if (uart->fifo_count < uart->fifo_limit) {
+				        uart->fifo_count++;
+					outb(midi_byte, uart->base + UART_TX);
+				} else {
+				        /* Cannot write (buffer empty) - put char in buffer */
+				        buff_in = uart->buff_in;
+					uart->tx_buff[buff_in] = midi_byte;
+					buff_in++;
+					buff_in &= TX_BUFF_MASK;
+					uart->buff_in = buff_in;
+				}
+			}
+		} else {
+		        buff_in = uart->buff_in;
+			uart->tx_buff[buff_in] = midi_byte;
+			buff_in++;
+			buff_in &= TX_BUFF_MASK;
+			uart->buff_in = buff_in;
+			if (uart->polled)
+			        snd_uart16550_io_loop(uart);	/* Feeble attempt to improve performance - but it helps */
+		}
 	}
-	snd_spin_unlock(uart, open, &flags);
+	spin_unlock_irqrestore(&uart->open_lock, flags);
+	//snd_printd ("snd_uart16550_output_write done\n");
 }
 
 
@@ -675,11 +595,11 @@ static int snd_uart16550_register_resources(snd_uart16550_t * uart)
 		//snd_printd ("snd_uart16550_register_resources uart == NULL bug?\n");
 		return -EINVAL;
 	}
-	snd_spin_lock(uart, open, &flags);
+	spin_lock_irqsave(&uart->open_lock, flags);
 	if (snd_register_ioport(uart->card, uart->base, 8,
 			      "serial MIDI(uart16550A)", &rioport) < 0) {
 		snd_printd("snd_uart16550_register_resources snd_register_ioport failed, port\n");
-		snd_spin_unlock(uart, open, &flags);
+		spin_unlock_irqrestore(&uart->open_lock, flags);
 		return -EBUSY;
 	}
 	if (!uart->polled) {
@@ -701,12 +621,12 @@ static int snd_uart16550_register_resources(snd_uart16550_t * uart)
 			       uart->rmidi->name, uart->irq_number);
 		}
 	}
-	snd_spin_unlock(uart, open, &flags);
+	spin_unlock_irqrestore(&uart->open_lock, flags);
 	return 0;
 }
 
 
-static struct snd_stru_rawmidi_direction_hw snd_uart16550_output =
+static struct snd_stru_rawmidi_channel_hw snd_uart16550_output =
 {
 	SND_RAWMIDI_HW_POLL,	/* flags */
 	NULL,			/* private_data */
@@ -719,7 +639,7 @@ static struct snd_stru_rawmidi_direction_hw snd_uart16550_output =
 
 };
 
-static struct snd_stru_rawmidi_direction_hw snd_uart16550_input =
+static struct snd_stru_rawmidi_channel_hw snd_uart16550_input =
 {
 	0,			/* flags */
 	NULL,			/* private_data */
@@ -735,45 +655,51 @@ static struct snd_stru_rawmidi_direction_hw snd_uart16550_input =
 
 static void snd_uart16550_free(void *private_data)
 {
-	snd_uart16550_t_free((snd_uart16550_t *) private_data);
+	snd_uart16550_t *uart = snd_magic_cast(snd_uart16550_t, private_data, );
+	snd_uart16550_t_free(uart);
 };
 
-snd_rawmidi_t *snd_uart16550_new_device(snd_card_t * card,
-					unsigned short irq_number,
-					unsigned short iobase,
-					unsigned char divisor,
-					int polled)
+int snd_uart16550_new_device(snd_card_t * card, int device,
+			     unsigned short irq_number,
+			     unsigned short iobase,
+			     unsigned char divisor,
+			     int polled,
+			     snd_rawmidi_t ** rrawmidi)
 {
 	snd_rawmidi_t *rmidi;
 	snd_uart16550_t *uart;
+	int err;
 
 	//snd_printd ("snd_uart16550_new_device called\n");
 
+	*rrawmidi = NULL;
 	//XXX should check device type
-	rmidi = snd_rawmidi_new_device(card, "UART Serial MIDI");
-	if (rmidi == NULL)
-		return NULL;
+	if ((err = snd_rawmidi_new(card, "UART Serial MIDI", device, &rmidi)) < 0)
+		return err;
 
 	uart = snd_uart16550_t_new(card, rmidi);
 	if (uart == NULL) {
-		snd_rawmidi_free(rmidi);
-		return NULL;
+		snd_device_free(card, rmidi);
+		return -ENOMEM;
 	}
 	if (snd_uart16550_do_set_param(uart,
-			      irq_number, iobase, divisor, polled) < 0) {	//error!!!
+				       irq_number,
+				       iobase,
+				       divisor,
+				       polled) < 0) {	//error!!!
 
 		snd_uart16550_t_free(uart);
-		snd_rawmidi_free(rmidi);
-		return NULL;
+		snd_device_free(card, rmidi);
+		return -EIO;
 	}
 	if (snd_uart16550_register_resources(uart) < 0) {
 		snd_uart16550_t_free(uart);
-		snd_rawmidi_free(rmidi);
-		return NULL;
+		snd_device_free(card, rmidi);
+		return -EBUSY;
 	}
-	memcpy(&rmidi->input.hw, &snd_uart16550_input,
+	memcpy(&rmidi->chn[SND_RAWMIDI_CHANNEL_INPUT].hw, &snd_uart16550_input,
 	       sizeof(snd_uart16550_input));
-	memcpy(&rmidi->output.hw, &snd_uart16550_output,
+	memcpy(&rmidi->chn[SND_RAWMIDI_CHANNEL_OUTPUT].hw, &snd_uart16550_output,
 	       sizeof(snd_uart16550_output));
 	strcpy(rmidi->name, "uart16550 MIDI");
 	rmidi->info_flags |= SND_RAWMIDI_INFO_OUTPUT |
@@ -781,26 +707,27 @@ snd_rawmidi_t *snd_uart16550_new_device(snd_card_t * card,
 	    SND_RAWMIDI_INFO_DUPLEX;
 	rmidi->private_data = uart;
 	rmidi->private_free = snd_uart16550_free;
-	return rmidi;
+	*rrawmidi = rmidi;
+	return 0;
 }
 
 
 
 ////////////////////////////////////////////////////////////////////module
 
-#ifndef LINUX_2_1
-extern struct symbol_table snd_symbol_table_uart16550_export;
-#endif
+EXPORT_SYMBOL(snd_uart16550_new_device);
+EXPORT_SYMBOL(snd_uart16550_set_param);
+EXPORT_SYMBOL(snd_uart16550_detect);
 
-int init_module(void)
+#ifdef MODULE
+
+int __init init_module(void)
 {
-#ifndef LINUX_2_1
-	if (register_symtab(&snd_symbol_table_uart16550_export) < 0)
-		return -ENOMEM;
-#endif
 	return 0;
 }
 
-void cleanup_module(void)
+void __exit cleanup_module(void)
 {
 }
+
+#endif
