@@ -21,8 +21,8 @@
  *
  */
 
-#include "driver.h"
-#include "gus.h"
+#include "../../include/driver.h"
+#include "../../include/gus.h"
 
 /*
  *  Timer 1 - 80us
@@ -35,14 +35,14 @@ static void snd_gf1_timer1_start(snd_timer_t * timer)
 	unsigned int ticks;
 	snd_gus_card_t *gus;
 
-	gus = (snd_gus_card_t *) timer->private_data;
-	snd_spin_lock(gus, reg, &flags);
+	gus = snd_magic_cast(snd_gus_card_t, timer->private_data, );
+	spin_lock_irqsave(&gus->reg_lock, flags);
 	ticks = timer->sticks;
 	tmp = (gus->gf1.timer_enabled |= 4);
 	snd_gf1_write8(gus, SND_GF1_GB_ADLIB_TIMER_1, 256 - ticks);	/* timer 1 count */
 	snd_gf1_write8(gus, SND_GF1_GB_SOUND_BLASTER_CONTROL, tmp);	/* enable timer 1 IRQ */
 	snd_gf1_adlib_write(gus, 0x04, tmp >> 2);	/* timer 2 start */
-	snd_spin_unlock(gus, reg, &flags);
+	spin_unlock_irqrestore(&gus->reg_lock, flags);
 }
 
 static void snd_gf1_timer1_stop(snd_timer_t * timer)
@@ -51,11 +51,11 @@ static void snd_gf1_timer1_stop(snd_timer_t * timer)
 	unsigned char tmp;
 	snd_gus_card_t *gus;
 
-	gus = (snd_gus_card_t *) timer->private_data;
-	snd_spin_lock(gus, reg, &flags);
+	gus = snd_magic_cast(snd_gus_card_t, timer->private_data, );
+	spin_lock_irqsave(&gus->reg_lock, flags);
 	tmp = (gus->gf1.timer_enabled &= ~4);
 	snd_gf1_write8(gus, SND_GF1_GB_SOUND_BLASTER_CONTROL, tmp);	/* disable timer #1 */
-	snd_spin_unlock(gus, reg, &flags);
+	spin_unlock_irqrestore(&gus->reg_lock, flags);
 }
 
 /*
@@ -69,14 +69,14 @@ static void snd_gf1_timer2_start(snd_timer_t * timer)
 	unsigned int ticks;
 	snd_gus_card_t *gus;
 
-	gus = (snd_gus_card_t *) timer->private_data;
-	snd_spin_lock(gus, reg, &flags);
+	gus = snd_magic_cast(snd_gus_card_t, timer->private_data, );
+	spin_lock_irqsave(&gus->reg_lock, flags);
 	ticks = timer->sticks;
 	tmp = (gus->gf1.timer_enabled |= 8);
 	snd_gf1_write8(gus, SND_GF1_GB_ADLIB_TIMER_2, 256 - ticks);	/* timer 2 count */
 	snd_gf1_write8(gus, SND_GF1_GB_SOUND_BLASTER_CONTROL, tmp);	/* enable timer 2 IRQ */
 	snd_gf1_adlib_write(gus, 0x04, tmp >> 2);	/* timer 2 start */
-	snd_spin_unlock(gus, reg, &flags);
+	spin_unlock_irqrestore(&gus->reg_lock, flags);
 }
 
 static void snd_gf1_timer2_stop(snd_timer_t * timer)
@@ -85,11 +85,11 @@ static void snd_gf1_timer2_stop(snd_timer_t * timer)
 	unsigned char tmp;
 	snd_gus_card_t *gus;
 
-	gus = (snd_gus_card_t *) timer->private_data;
-	snd_spin_lock(gus, reg, &flags);
+	gus = snd_magic_cast(snd_gus_card_t, timer->private_data, );
+	spin_lock_irqsave(&gus->reg_lock, flags);
 	tmp = (gus->gf1.timer_enabled &= ~8);
 	snd_gf1_write8(gus, SND_GF1_GB_SOUND_BLASTER_CONTROL, tmp);	/* disable timer #1 */
-	snd_spin_unlock(gus, reg, &flags);
+	spin_unlock_irqrestore(&gus->reg_lock, flags);
 }
 
 /*
@@ -149,27 +149,17 @@ void snd_gf1_timers_init(snd_gus_card_t * gus)
 	gus->gf1.interrupt_handler_timer1 = snd_gf1_interrupt_timer1;
 	gus->gf1.interrupt_handler_timer2 = snd_gf1_interrupt_timer2;
 
-	timer = snd_timer_new_device(gus->card, "GF1 timer");
-	if (timer) {
+	if (snd_timer_new(gus->card, "GF1 timer", gus->timer_dev, &timer) >= 0) {
 		strcpy(timer->name, "GF1 timer #1");
 		timer->private_data = gus;
 		memcpy(&timer->hw, &snd_gf1_timer1, sizeof(snd_gf1_timer1));
-		if (snd_timer_register(timer, gus->timer_dev) < 0) {
-			snd_timer_free(timer);
-			timer = NULL;
-		}
 	}
 	gus->gf1.timer1 = timer;
 
-	timer = snd_timer_new_device(gus->card, "GF1 timer");
-	if (timer) {
+	if (snd_timer_new(gus->card, "GF1 timer", gus->timer_dev+1, &timer) >= 0) {
 		strcpy(timer->name, "GF1 timer #2");
 		timer->private_data = gus;
 		memcpy(&timer->hw, &snd_gf1_timer2, sizeof(snd_gf1_timer2));
-		if (snd_timer_register(timer, gus->timer_dev + 1) < 0) {
-			snd_timer_free(timer);
-			timer = NULL;
-		}
 	}
 	gus->gf1.timer2 = timer;
 }
@@ -178,11 +168,11 @@ void snd_gf1_timers_done(snd_gus_card_t * gus)
 {
 	snd_gf1_set_default_handlers(gus, SND_GF1_HANDLER_TIMER1 | SND_GF1_HANDLER_TIMER2);
 	if (gus->gf1.timer1) {
-		snd_timer_unregister(gus->gf1.timer1);
+		snd_device_free(gus->card, gus->gf1.timer1);
 		gus->gf1.timer1 = NULL;
 	}
 	if (gus->gf1.timer2) {
-		snd_timer_unregister(gus->gf1.timer2);
+		snd_device_free(gus->card, gus->gf1.timer2);
 		gus->gf1.timer2 = NULL;
 	}
 }
