@@ -142,7 +142,7 @@ static int snd_es18xx_read(es18xx_t *codec, unsigned char reg)
                 return -1;
 	data = snd_es18xx_dsp_get_byte(codec);
 #ifdef REG_DEBUG
-	snd_printk("Reg %02x == %02x\n", reg, data);
+	snd_printk("Reg %02x set to %02x\n", reg, data);
 #endif
 	return data;
 }
@@ -229,6 +229,9 @@ static void snd_es18xx_config_write(es18xx_t *codec,
 {
 	outb(reg, codec->ctrl_port);
 	outb(data, codec->ctrl_port+1);
+#ifdef REG_DEBUG
+	snd_printk("Config reg %02x set to %02x\n", reg, data);
+#endif
 }
 
 static int snd_es18xx_reset(es18xx_t * codec)
@@ -487,19 +490,19 @@ static int snd_es18xx_probe(es18xx_t *codec)
 		codec->caps = ES18XX_DUPLEX_MONO | ES18XX_DUPLEX_SAME | ES18XX_CONTROL | ES18XX_HWV;
 		break;
 	case 0x1869:
-		codec->caps = ES18XX_PCM2 | ES18XX_3D | ES18XX_RECMIX | ES18XX_NEW_RATE | ES18XX_AUXB | ES18XX_SPEAKER | ES18XX_MONO | ES18XX_MUTEREC | ES18XX_CONTROL | ES18XX_HWV;
+		codec->caps = ES18XX_PCM2 | ES18XX_3D | ES18XX_RECMIX | ES18XX_NEW_RATE | ES18XX_AUXB | ES18XX_MONO | ES18XX_MUTEREC | ES18XX_CONTROL | ES18XX_HWV;
 		break;
 	case 0x1878:
 		codec->caps = ES18XX_DUPLEX_MONO | ES18XX_DUPLEX_SAME | ES18XX_I2S | ES18XX_CONTROL | ES18XX_HWV;
 		break;
 	case 0x1879 | ES18XX_HWV:
-		codec->caps = ES18XX_PCM2 | ES18XX_3D | ES18XX_RECMIX | ES18XX_NEW_RATE | ES18XX_AUXB | ES18XX_SPEAKER | ES18XX_I2S | ES18XX_CONTROL | ES18XX_HWV;
+		codec->caps = ES18XX_PCM2 | ES18XX_3D | ES18XX_RECMIX | ES18XX_NEW_RATE | ES18XX_AUXB | ES18XX_I2S | ES18XX_CONTROL | ES18XX_HWV;
 		break;
 	case 0x1887:
-		codec->caps = ES18XX_PCM2 | ES18XX_RECMIX | ES18XX_AUXB | ES18XX_SPEAKER | ES18XX_DUPLEX_SAME | ES18XX_HWV;
+		codec->caps = ES18XX_PCM2 | ES18XX_RECMIX | ES18XX_AUXB | ES18XX_DUPLEX_SAME | ES18XX_HWV;
 		break;
 	case 0x1888:
-		codec->caps = ES18XX_PCM2 | ES18XX_RECMIX | ES18XX_AUXB | ES18XX_SPEAKER | ES18XX_DUPLEX_SAME | ES18XX_HWV;
+		codec->caps = ES18XX_PCM2 | ES18XX_RECMIX | ES18XX_AUXB | ES18XX_DUPLEX_SAME | ES18XX_HWV;
 		break;
 	default:
                 snd_printk("[0x%x] unsupported chip ES%x\n",
@@ -680,7 +683,7 @@ static int snd_es18xx_playback1_trigger(es18xx_t *codec,
 	spin_lock_irqsave(&codec->reg_lock, flags);
 	if (cmd == SND_PCM_TRIGGER_GO) {	
                 /* Start DMA */
-		if (codec->dma2 == 5)
+		if (codec->dma2 >= 4)
 			snd_es18xx_mixer_write(codec, 0x78, 0xb3);
 		else
 			snd_es18xx_mixer_write(codec, 0x78, 0x93);
@@ -1516,6 +1519,52 @@ static int snd_es18xx_mixer_micpre_v(snd_kmixer_element_t *element, int w_flag,
         return change;
 }
 
+static int snd_es18xx_mixer_speaker_v(snd_kmixer_element_t *element, int w_flag, int *voices)
+{
+	es18xx_t * codec = snd_magic_cast(es18xx_t, element->private_data, -ENXIO);
+        unsigned long flags;
+        int change = 0;
+        unsigned char oval, nval;
+
+        spin_lock_irqsave(&codec->mixer_lock, flags);
+	oval = snd_es18xx_mixer_read(codec, 0x3c);
+        if (!w_flag) {
+                voices[0] = oval & 0x07;
+        } else {
+                nval = voices[0];
+                if ((change = (oval != nval))) {
+			snd_es18xx_mixer_write(codec, 0x3c, nval);
+		}
+        }
+        spin_unlock_irqrestore(&codec->mixer_lock, flags);
+        return change;
+}
+
+static int snd_es18xx_mixer_speaker_g(snd_kmixer_group_t * group,
+				      snd_kmixer_file_t *file,
+				      int w_flag,
+				      snd_mixer_group_t *ugroup)
+{
+	es18xx_t * codec = snd_magic_cast(es18xx_t, group->private_data, -ENXIO);
+	int voice;
+	int change = 0;
+	if (!w_flag) {
+		ugroup->caps = SND_MIXER_GRPCAP_VOLUME;
+		ugroup->channels = SND_MIXER_CHN_MASK_MONO;
+		snd_es18xx_mixer_speaker_v(codec->mix_speaker_v, 0, &voice);
+		ugroup->volume.names.front_left = voice;
+		ugroup->min = 0;
+		ugroup->max = 7;
+	} else {
+		voice = ugroup->volume.names.front_left & 7;
+		if (snd_es18xx_mixer_speaker_v(codec->mix_speaker_v, 1, &voice)) {
+			snd_mixer_element_value_change(file, codec->mix_speaker_v, 0);
+			change = 1;
+		}
+	}
+	return 1;
+}
+
 static int snd_es18xx_mixer_group_ctrl(snd_kmixer_group_t * group,
 				       snd_kmixer_file_t * file,
 				       int w_flag,
@@ -1770,6 +1819,7 @@ int snd_es18xx_mixer(es18xx_t *codec, int device, snd_pcm_t *pcm, snd_kmixer_t *
 	snd_kmixer_group_t *output_g;
 	snd_kmixer_group_t *igain_g;
 	snd_kmixer_group_t *e3d_g;
+	snd_kmixer_group_t *speaker_g;
 	
 	snd_kmixer_element_t *pcm1, *ipcm1_v, *opcm1_v;
 	snd_kmixer_element_t *pcm2, *opcm2_v;
@@ -1785,6 +1835,7 @@ int snd_es18xx_mixer(es18xx_t *codec, int device, snd_pcm_t *pcm, snd_kmixer_t *
 	snd_kmixer_element_t *e3d;
 	snd_kmixer_element_t *recmon; 
 	snd_kmixer_element_t *input, *output, *adc;
+	snd_kmixer_element_t *speaker, *speaker_v;
 	int err;
 	/* FIXME */
 	static struct snd_mixer_element_volume1_range opcm1_db_range[2] = {
@@ -1815,6 +1866,10 @@ int snd_es18xx_mixer(es18xx_t *codec, int device, snd_pcm_t *pcm, snd_kmixer_t *
 		{0, 1, 0, 2600}
 	};
 
+	static struct snd_mixer_element_volume1_range speaker_db_range[1] = {
+		{0, 7, -2100, 0}
+	};
+
 	char str[16];
 
 	snd_debug_check(rmixer == NULL, -EINVAL);
@@ -2025,6 +2080,21 @@ int snd_es18xx_mixer(es18xx_t *codec, int device, snd_pcm_t *pcm, snd_kmixer_t *
 		goto __error;
 	codec->mix_oaux_v = oaux_v;
 
+	/* Speaker */
+	if ((speaker_g = snd_mixer_lib_group_ctrl(mixer, SND_MIXER_IN_SPEAKER, 0, SND_MIXER_OSS_SPEAKER, snd_es18xx_mixer_speaker_g, codec)) == NULL)
+		goto __error;
+ 	if ((speaker = snd_mixer_lib_io_mono(mixer, SND_MIXER_IN_SPEAKER, 0, SND_MIXER_ETYPE_INPUT, 0)) == NULL)
+		goto __error;
+	if ((speaker_v = snd_mixer_lib_volume1(mixer, "PC Speaker Volume", 0, 1, speaker_db_range, snd_es18xx_mixer_speaker_v, codec)) == NULL)
+		goto __error;
+	if (snd_mixer_group_element_add(mixer, speaker_g, speaker_v) < 0)
+		goto __error;
+	if (snd_mixer_element_route_add(mixer, speaker, speaker_v))
+		goto __error;
+	if (snd_mixer_element_route_add(mixer, speaker, oaccu))
+		goto __error;
+	codec->mix_speaker_v = speaker_v;
+
 	/* Input mixer */
 	if (codec->caps & ES18XX_RECMIX) {
 		/* Accumulator */
