@@ -775,7 +775,7 @@ static snd_pcm_hardware_t snd_ensoniq_playback2 =
 	SND_PCM_CHNINFO_MMAP_VALID |
 	SND_PCM_CHNINFO_PAUSE,	/* flags */
 	SND_PCM_FMT_U8 | SND_PCM_FMT_S16_LE,	/* hardware formats */
-#ifndef ENS1370
+#ifndef CHIP1370
 	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_48000,
 #else
 	SND_PCM_RATE_KNOT | 	/* 5512Hz rate */
@@ -834,8 +834,8 @@ static int snd_ensoniq_playback1_open(void *private_data,
 	ensoniq->playback_subchn = subchn;
 	subchn->runtime->hw = &snd_ensoniq_playback1;
 	snd_pcm_set_sync(subchn);
-#ifdef ENS1370
-	snd_pcm_set_mixer(subchn, ensoniq->mixer->device, ensoniq->u.es1370.ak4531->me_playback1);
+#ifdef CHIP1370
+	snd_pcm_set_mixer(subchn, ensoniq->mixer->device, ensoniq->u.es1370.ak4531->me_playback);
 #else
 	snd_pcm_set_mixer(subchn, ensoniq->mixer->device, ensoniq->u.es1371.ac97->me_playback);
 #endif
@@ -855,8 +855,8 @@ static int snd_ensoniq_playback2_open(void *private_data,
 	ensoniq->playback2_subchn = subchn;
 	subchn->runtime->hw = &snd_ensoniq_playback2;
 	snd_pcm_set_sync(subchn);
-#ifdef ENS1370
-	snd_pcm_set_mixer(subchn, ensoniq->mixer->device, ensoniq->u.es1370.ak4531->me_playback2);
+#ifdef CHIP1370
+	snd_pcm_set_mixer(subchn, ensoniq->mixer->device, ensoniq->u.es1370.ak4531->me_playback1);
 #else
 	snd_pcm_set_mixer(subchn, ensoniq->mixer->device, ensoniq->u.es1371.ac97->me_playback);
 #endif
@@ -876,7 +876,7 @@ static int snd_ensoniq_capture_open(void *private_data,
 	ensoniq->capture_subchn = subchn;
 	subchn->runtime->hw = &snd_ensoniq_capture;	
 	snd_pcm_set_sync(subchn);
-#ifdef ENS1370
+#ifdef CHIP1370
 	snd_pcm_set_mixer(subchn, ensoniq->mixer->device, ensoniq->u.es1370.ak4531->me_capture);
 #else
 	snd_pcm_set_mixer(subchn, ensoniq->mixer->device, ensoniq->u.es1371.ac97->me_capture);
