@@ -1599,7 +1599,7 @@ int snd_solo_new_mixer(es1938_t * codec, int device, snd_pcm_t * _pcm, snd_kmixe
 		goto __error;
 	codec->mix_capture = input;
 
-	*rmixer = mixer;
+	*rmixer = codec->mixer = mixer;
 	return 0;
 
       __error:
