@@ -2259,9 +2259,9 @@ snd_rme9652_proc_read (snd_info_buffer_t * buffer,
 	}
 
 	if (rme9652->control_register & RME9652_opt_out) {
-		snd_iprintf (buffer, "S/PDIF output: ADAT1\n");
+		snd_iprintf (buffer, "S/PDIF output: Coaxial & ADAT1\n");
 	} else {
-		snd_iprintf (buffer, "S/PDIF output: Coaxial (Phono)\n");
+		snd_iprintf (buffer, "S/PDIF output: Coaxial only\n");
 	}
 
 	if (rme9652->control_register & RME9652_PRO) {
