@@ -23,8 +23,8 @@
  *   working.
  */
 
-#include "driver.h"
-#include "sb.h"
+#include "../../include/driver.h"
+#include "../../include/sb.h"
 
 /*
 
@@ -37,19 +37,19 @@ void snd_sb8dsp_midi_interrupt(snd_rawmidi_t * rmidi)
 	int max = 64;
 	char byte;
 
-	codec = (sbdsp_t *) rmidi->private_data;
-	if (!codec) {
+	codec = snd_magic_cast(sbdsp_t, rmidi->private_data, );
+	if (codec == NULL) {
 		inb(SBP(codec, READ));	/* ack interrupt */
 		return;
 	}
 	while (max-- > 0) {
-		snd_spin_lock(codec, midi_input, &flags);
+		spin_lock_irqsave(&codec->midi_input_lock, flags);
 		if (SBP(codec, DATA_AVAIL) & 0x80) {
 			byte = inb(SBP(codec, READ));
-			snd_spin_unlock(codec, midi_input, &flags);
-			rmidi->input.data(rmidi, &byte, 1);
+			spin_unlock_irqrestore(&codec->midi_input_lock, flags);
+			snd_rawmidi_receive(rmidi, &byte, 1);
 		} else {
-			snd_spin_unlock(codec, midi_input, &flags);
+			spin_unlock_irqrestore(&codec->midi_input_lock, flags);
 		}
 	}
 }
@@ -63,18 +63,18 @@ static int snd_sb8dsp_midi_input_open(snd_rawmidi_t * rmidi)
 	unsigned long flags;
 	sbdsp_t *codec;
 
-	codec = (sbdsp_t *) rmidi->private_data;
-	snd_spin_lock(codec, open8, &flags);
+	codec = snd_magic_cast(sbdsp_t, rmidi->private_data, -ENXIO);
+	spin_lock_irqsave(&codec->open8_lock, flags);
 	if (codec->open8) {
-		snd_spin_unlock(codec, open8, &flags);
-		return -EBUSY;
+		spin_unlock_irqrestore(&codec->open8_lock, flags);
+		return -EAGAIN;
 	}
 	codec->open8 |= SB_OPEN_MIDI_INPUT;
 	if (!(codec->open8 & SB_OPEN_MIDI_OUTPUT)) {
-		snd_spin_unlock(codec, open8, &flags);
+		spin_unlock_irqrestore(&codec->open8_lock, flags);
 		snd_sb8dsp_reset(codec);		/* reset DSP */
 	} else {
-		snd_spin_unlock(codec, open8, &flags);
+		spin_unlock_irqrestore(&codec->open8_lock, flags);
 	}
 	return 0;
 }
@@ -84,18 +84,18 @@ static int snd_sb8dsp_midi_output_open(snd_rawmidi_t * rmidi)
 	unsigned long flags;
 	sbdsp_t *codec;
 
-	codec = (sbdsp_t *) rmidi->private_data;
-	snd_spin_lock(codec, open8, &flags);
+	codec = snd_magic_cast(sbdsp_t, rmidi->private_data, -ENXIO);
+	spin_lock_irqsave(&codec->open8_lock, flags);
 	if (codec->open8) {
-		snd_spin_unlock(codec, open8, &flags);
-		return -EBUSY;
+		spin_unlock_irqrestore(&codec->open8_lock, flags);
+		return -EAGAIN;
 	}
 	codec->open8 |= SB_OPEN_MIDI_OUTPUT;
 	if (!(codec->open8 & SB_OPEN_MIDI_INPUT)) {
-		snd_spin_unlock(codec, open8, &flags);
+		spin_unlock_irqrestore(&codec->open8_lock, flags);
 		snd_sb8dsp_reset(codec);		/* reset DSP */
 	} else {
-		snd_spin_unlock(codec, open8, &flags);
+		spin_unlock_irqrestore(&codec->open8_lock, flags);
 	}
 	return 0;
 }
@@ -105,14 +105,14 @@ static int snd_sb8dsp_midi_input_close(snd_rawmidi_t * rmidi)
 	unsigned long flags;
 	sbdsp_t *codec;
 
-	codec = (sbdsp_t *) rmidi->private_data;
-	snd_spin_lock(codec, open8, &flags);
+	codec = snd_magic_cast(sbdsp_t, rmidi->private_data, -ENXIO);
+	spin_lock_irqsave(&codec->open8_lock, flags);
 	codec->open8 &= ~(SB_OPEN_MIDI_INPUT | SB_OPEN_MIDI_TRIGGER);
 	if (!(codec->open8 & SB_OPEN_MIDI_OUTPUT)) {
-		snd_spin_unlock(codec, open8, &flags);
+		spin_unlock_irqrestore(&codec->open8_lock, flags);
 		snd_sb8dsp_reset(codec);		/* reset DSP */
 	} else {
-		snd_spin_unlock(codec, open8, &flags);
+		spin_unlock_irqrestore(&codec->open8_lock, flags);
 	}
 	return 0;
 }
@@ -122,14 +122,14 @@ static int snd_sb8dsp_midi_output_close(snd_rawmidi_t * rmidi)
 	unsigned long flags;
 	sbdsp_t *codec;
 
-	codec = (sbdsp_t *) rmidi->private_data;
-	snd_spin_lock(codec, open8, &flags);
+	codec = snd_magic_cast(sbdsp_t, rmidi->private_data, -ENXIO);
+	spin_lock_irqsave(&codec->open8_lock, flags);
 	codec->open8 &= ~SB_OPEN_MIDI_OUTPUT;
 	if (!(codec->open8 & SB_OPEN_MIDI_INPUT)) {
-		snd_spin_unlock(codec, open8, &flags);
+		spin_unlock_irqrestore(&codec->open8_lock, flags);
 		snd_sb8dsp_reset(codec);		/* reset DSP */
 	} else {
-		snd_spin_unlock(codec, open8, &flags);
+		spin_unlock_irqrestore(&codec->open8_lock, flags);
 	}
 	return 0;
 }
@@ -139,8 +139,8 @@ static void snd_sb8dsp_midi_input_trigger(snd_rawmidi_t * rmidi, int up)
 	unsigned long flags;
 	sbdsp_t *codec;
 
-	codec = (sbdsp_t *) rmidi->private_data;
-	snd_spin_lock(codec, open8, &flags);
+	codec = snd_magic_cast(sbdsp_t, rmidi->private_data, );
+	spin_lock_irqsave(&codec->open8_lock, flags);
 	if (up) {
 		if (!(codec->open8 & SB_OPEN_MIDI_TRIGGER)) {
 			snd_sb8dsp_command(codec, SB_DSP_MIDI_INPUT_IRQ);
@@ -152,7 +152,7 @@ static void snd_sb8dsp_midi_input_trigger(snd_rawmidi_t * rmidi, int up)
 			codec->open8 &= ~SB_OPEN_MIDI_TRIGGER;
 		}
 	}
-	snd_spin_unlock(codec, open8, &flags);
+	spin_unlock_irqrestore(&codec->open8_lock, flags);
 }
 
 static void snd_sb8dsp_midi_output_write(snd_rawmidi_t * rmidi)
@@ -163,16 +163,16 @@ static void snd_sb8dsp_midi_output_write(snd_rawmidi_t * rmidi)
 	int max = 32;
 
 	/* how big is Tx FIFO? */
-	codec = (sbdsp_t *) rmidi->private_data;
+	codec = snd_magic_cast(sbdsp_t, rmidi->private_data, );
 	while (max-- > 0) {
-		snd_spin_lock(codec, open8, &flags);
-		if (rmidi->output.data(rmidi, &byte, 1) != 1) {
-			snd_spin_unlock(codec, open8, &flags);
+		spin_lock_irqsave(&codec->open8_lock, flags);
+		if (snd_rawmidi_transmit(rmidi, &byte, 1) != 1) {
+			spin_unlock_irqrestore(&codec->open8_lock, flags);
 			return;
 		}
 		snd_sb8dsp_command(codec, SB_DSP_MIDI_OUTPUT);
 		snd_sb8dsp_command(codec, byte);
-		snd_spin_unlock(codec, open8, &flags);
+		spin_unlock_irqrestore(&codec->open8_lock, flags);
 	}
 }
 
@@ -180,7 +180,7 @@ static void snd_sb8dsp_midi_output_write(snd_rawmidi_t * rmidi)
 
  */
 
-static struct snd_stru_rawmidi_direction_hw snd_sb8dsp_midi_output =
+static struct snd_stru_rawmidi_channel_hw snd_sb8dsp_midi_output =
 {
 	SND_RAWMIDI_HW_POLL,	/* flags */
 	NULL,			/* private_data */
@@ -192,7 +192,7 @@ static struct snd_stru_rawmidi_direction_hw snd_sb8dsp_midi_output =
 	NULL,			/* abort */
 };
 
-static struct snd_stru_rawmidi_direction_hw snd_sb8dsp_midi_input =
+static struct snd_stru_rawmidi_channel_hw snd_sb8dsp_midi_input =
 {
 	0,			/* flags */
 	NULL,			/* private_data */
@@ -204,20 +204,19 @@ static struct snd_stru_rawmidi_direction_hw snd_sb8dsp_midi_input =
 	NULL,			/* abort */
 };
 
-snd_rawmidi_t *snd_sb8dsp_midi_new_device(snd_card_t * card, snd_pcm_t * pcm)
+int snd_sb8dsp_midi_new(sbdsp_t * codec, int device, snd_rawmidi_t ** rrawmidi)
 {
-	snd_pcm1_t *pcm1;
-	sbdsp_t *codec;
 	snd_rawmidi_t *rmidi;
+	int err;
 
-	pcm1 = (snd_pcm1_t *) pcm->private_data;
-	codec = (sbdsp_t *) pcm1->private_data;
-	if ((rmidi = snd_rawmidi_new_device(card, "SB8 MIDI")) == NULL)
-		return NULL;
+	*rrawmidi = NULL;
+	if ((err = snd_rawmidi_new(codec->card, "SB8 MIDI", device, &rmidi)) < 0)
+		return err;
 	strcpy(rmidi->name, "SB8 MIDI");
-	memcpy(&rmidi->output.hw, &snd_sb8dsp_midi_output, sizeof(snd_sb8dsp_midi_output));
-	memcpy(&rmidi->input.hw, &snd_sb8dsp_midi_input, sizeof(snd_sb8dsp_midi_input));
+	memcpy(&rmidi->chn[SND_RAWMIDI_CHANNEL_OUTPUT].hw, &snd_sb8dsp_midi_output, sizeof(snd_sb8dsp_midi_output));
+	memcpy(&rmidi->chn[SND_RAWMIDI_CHANNEL_INPUT].hw, &snd_sb8dsp_midi_input, sizeof(snd_sb8dsp_midi_input));
 	rmidi->info_flags |= SND_RAWMIDI_INFO_OUTPUT | SND_RAWMIDI_INFO_INPUT | SND_RAWMIDI_INFO_DUPLEX;
 	rmidi->private_data = codec;
-	return rmidi;
+	*rrawmidi = rmidi;
+	return 0;
 }
