@@ -20,7 +20,6 @@
  */
 
 #ifdef ALSA_BUILD
-#define __KERNEL__
 #define MODULE
 #define CONFIG_ISAPNP
 #endif
@@ -51,6 +50,7 @@
 #include <linux/ioport.h>
 #include <linux/string.h>
 #include <linux/malloc.h>
+#include <linux/delay.h>
 #include <linux/proc_fs.h>
 #include <asm/io.h>
 #include <asm/dma.h>
@@ -66,10 +66,6 @@
 #include "isapnp.h"
 #endif
 
-#ifdef CONFIG_PROC_FS
-#include "isapnp_proc.c"
-#endif
-
 #if 0
 #define ISAPNP_REGION_OK
 #endif
@@ -113,6 +109,10 @@ MODULE_PARM_DESC(isapnp_reserve_io, "ISA Plug & Play - reserve I/O region(s) - p
 MODULE_PARM(isapnp_reserve_mem, "1-16i");
 MODULE_PARM_DESC(isapnp_reserve_mem, "ISA Plug & Play - reserve memory region(s) - address,size");
 
+int isapnp_proc_init(void);
+int isapnp_proc_done(void);
+void isapnp_fixup_device(struct isapnp_dev *dev);
+
 #define _PIDXR		0x279
 #define _PNPWRP		0xa79
 
@@ -161,7 +161,7 @@ static inline void write_data(unsigned char x)
 static inline void write_address(unsigned char x)
 {
 	outb(x, _PIDXR);
-	udelay(10);
+	udelay(20);
 }
 
 static inline unsigned char read_data(void)
@@ -216,12 +216,12 @@ void isapnp_write_dword(unsigned char idx, unsigned int val)
 	isapnp_write_byte(idx+3, val);
 }
 
-static void *isapnp_alloc(long size)
+void *isapnp_alloc(long size)
 {
 	void *result;
 
 	result = kmalloc(size, GFP_KERNEL);
-	if (!result)
+	if (result == NULL)
 		return NULL;
 	memset(result, 0, size);
 	return result;
@@ -265,7 +265,7 @@ void isapnp_activate(unsigned char logdev)
 {
 	isapnp_device(logdev);
 	isapnp_write_byte(ISAPNP_CFG_ACTIVATE, 1);
-	udelay(250);
+	udelay(500);
 }
 
 void isapnp_deactivate(unsigned char logdev)
@@ -284,7 +284,7 @@ static void __init isapnp_peek(unsigned char *data, int bytes)
 			d = isapnp_read_byte(0x05);
 			if (d & 1)
 				break;
-			udelay(10);
+			udelay(100);
 		}
 		if (!(d & 1)) {
 			*data++ = 0xff;
@@ -389,7 +389,10 @@ static int __init isapnp_isolate(void)
 			udelay(250);
 			iteration++;
 			isapnp_wake(0x00);
+			isapnp_set_rdp();
+			udelay(1000);
 			write_address(0x01);
+			udelay(1000);
 			goto __next;
 		}
 		if (iteration == 1) {
@@ -457,7 +460,7 @@ static struct isapnp_dev * __init isapnp_parse_device(struct isapnp_card *card,
 
 	isapnp_peek(tmp, size);
 	dev = isapnp_alloc(sizeof(struct isapnp_dev));
-	if (!dev)
+	if (dev == NULL)
 		return NULL;
 	dev->devfn = number;
 	dev->vendor = (tmp[1] << 8) | tmp[0];
@@ -481,7 +484,7 @@ static struct isapnp_resources * __init isapnp_build_resources(struct isapnp_dev
 	struct isapnp_resources *res, *ptr, *ptra;
 	
 	res = isapnp_alloc(sizeof(struct isapnp_resources));
-	if (!res)
+	if (res == NULL)
 		return NULL;
 	res->dev = dev;
 	ptr = (struct isapnp_resources *)dev->sysdata;
@@ -491,12 +494,12 @@ static struct isapnp_resources * __init isapnp_build_resources(struct isapnp_dev
 		ptra = ptr->alt;
 		while (ptra && ptra->alt)
 			ptra = ptra->alt;
-		if (!ptra)
+		if (ptra == NULL)
 			ptr->alt = res;
 		else
 			ptra->alt = res;
 	} else {
-		if (!ptr)
+		if (ptr == NULL)
 			dev->sysdata = res;
 		else
 			ptr->next = res;
@@ -526,7 +529,7 @@ static void __init isapnp_add_irq_resource(struct isapnp_dev *dev,
 
 	isapnp_peek(tmp, size);
 	irq = isapnp_alloc(sizeof(struct isapnp_irq));
-	if (!irq)
+	if (irq == NULL)
 		return;
 	if (*res == NULL) {
 		*res = isapnp_build_resources(dev, dependent);
@@ -563,7 +566,7 @@ static void __init isapnp_add_dma_resource(struct isapnp_dev *dev,
 
 	isapnp_peek(tmp, size);
 	dma = isapnp_alloc(sizeof(struct isapnp_dma));
-	if (!dma)
+	if (dma == NULL)
 		return;
 	if (*res == NULL) {
 		*res = isapnp_build_resources(dev, dependent);
@@ -597,7 +600,7 @@ static void __init isapnp_add_port_resource(struct isapnp_dev *dev,
 
 	isapnp_peek(tmp, size);
 	port = isapnp_alloc(sizeof(struct isapnp_port));
-	if (!port)
+	if (port == NULL)
 		return;
 	if (*res == NULL) {
 		*res = isapnp_build_resources(dev, dependent);
@@ -634,7 +637,7 @@ static void __init isapnp_add_fixed_port_resource(struct isapnp_dev *dev,
 
 	isapnp_peek(tmp, size);
 	port = isapnp_alloc(sizeof(struct isapnp_port));
-	if (!port)
+	if (port == NULL)
 		return;
 	if (*res == NULL) {
 		*res = isapnp_build_resources(dev, dependent);
@@ -670,7 +673,7 @@ static void __init isapnp_add_mem_resource(struct isapnp_dev *dev,
 
 	isapnp_peek(tmp, size);
 	mem = isapnp_alloc(sizeof(struct isapnp_mem));
-	if (!mem)
+	if (mem == NULL)
 		return;
 	if (*res == NULL) {
 		*res = isapnp_build_resources(dev, dependent);
@@ -707,7 +710,7 @@ static void __init isapnp_add_mem32_resource(struct isapnp_dev *dev,
 
 	isapnp_peek(tmp, size);
 	mem32 = isapnp_alloc(sizeof(struct isapnp_mem32));
-	if (!mem32)
+	if (mem32 == NULL)
 		return;
 	if (*res == NULL) {
 		*res = isapnp_build_resources(dev, dependent);
@@ -740,7 +743,7 @@ static void __init isapnp_add_fixed_mem32_resource(struct isapnp_dev *dev,
 
 	isapnp_peek(tmp, size);
 	mem32 = isapnp_alloc(sizeof(struct isapnp_mem32));
-	if (!mem32)
+	if (mem32 == NULL)
 		return;
 	if (*res == NULL) {
 		*res = isapnp_build_resources(dev, dependent);
@@ -991,6 +994,7 @@ static int __init isapnp_build_device_list(void)
 	int csn;
 	unsigned char header[9], checksum;
 	struct isapnp_card *card, *prev = NULL;
+	struct isapnp_dev *dev;
 
 	isapnp_wait();
 	isapnp_key();
@@ -1017,12 +1021,14 @@ static int __init isapnp_build_device_list(void)
 		if (isapnp_checksum_value != 0x00)
 			printk("isapnp: checksum for device %i is not valid (0x%x)\n", csn, isapnp_checksum_value);
 		card->checksum = isapnp_checksum_value;
-		if (!isapnp_cards)
+		if (isapnp_cards == NULL)
 			isapnp_cards = card;
 		else
 			prev->next = card;
 		prev = card;
 	}
+	for (dev = isapnp_devices; dev != NULL; dev = dev->next)
+		isapnp_fixup_device(dev);
 	return 0;
 }
 
@@ -1074,7 +1080,7 @@ static struct isapnp_port *isapnp_find_port(struct isapnp_dev *dev, int index)
 	struct isapnp_resources *res;
 	struct isapnp_port *port;
 	
-	if (!dev || index < 0 || index > 7)
+	if (dev == NULL || index < 0 || index > 7)
 		return NULL;
 	for (res = (struct isapnp_resources *)dev->sysdata; res; res = res->next) {
 		for (port = res->port; port; port = port->next) {
@@ -1092,7 +1098,7 @@ struct isapnp_irq *isapnp_find_irq(struct isapnp_dev *dev, int index)
 	struct isapnp_irq *irq;
 	int index1, index2, index3;
 	
-	if (!dev || index < 0 || index > 7)
+	if (dev == NULL || index < 0 || index > 7)
 		return NULL;
 	for (res = (struct isapnp_resources *)dev->sysdata; res; res = res->next) {
 		index3 = 0;
@@ -1118,7 +1124,7 @@ struct isapnp_dma *isapnp_find_dma(struct isapnp_dev *dev, int index)
 	struct isapnp_resources *res;
 	struct isapnp_dma *dma;
 	
-	if (!dev || index < 0 || index > 7)
+	if (dev == NULL || index < 0 || index > 7)
 		return NULL;
 	for (res = (struct isapnp_resources *)dev->sysdata; res; res = res->next) {
 		for (dma = res->dma; dma; dma = dma->next) {
@@ -1135,7 +1141,7 @@ struct isapnp_mem *isapnp_find_mem(struct isapnp_dev *dev, int index)
 	struct isapnp_resources *res;
 	struct isapnp_mem *mem;
 	
-	if (!dev || index < 0 || index > 7)
+	if (dev == NULL || index < 0 || index > 7)
 		return NULL;
 	for (res = (struct isapnp_resources *)dev->sysdata; res; res = res->next) {
 		for (mem = res->mem; mem; mem = mem->next) {
@@ -1152,7 +1158,7 @@ struct isapnp_mem32 *isapnp_find_mem32(struct isapnp_dev *dev, int index)
 	struct isapnp_resources *res;
 	struct isapnp_mem32 *mem32;
 	
-	if (!dev || index < 0 || index > 7)
+	if (dev == NULL || index < 0 || index > 7)
 		return NULL;
 	for (res = (struct isapnp_resources *)dev->sysdata; res; res = res->next) {
 		for (mem32 = res->mem32; mem32; mem32 = mem32->next) {
@@ -1214,7 +1220,7 @@ struct isapnp_dev *isapnp_find_dev(struct isapnp_card *card,
 		} else {
 			from = from->next;
 		}
-		if (from->bus != card)	/* something is wrong */
+		if (from == NULL || from->bus != card)	/* something is wrong */
 			return NULL;
 		for (dev = from; dev; dev = dev->sibling) {
 			if (dev->vendor == vendor && dev->device == function)
@@ -1357,27 +1363,27 @@ static int isapnp_alternative_switch(struct isapnp_cfgtmp *cfg,
 	struct isapnp_dma *dma;
 	struct isapnp_mem *mem;
 
-	if (!cfg)
+	if (cfg == NULL)
 		return -EINVAL;
 	/* process port settings */
 	for (tmp = 0; tmp < 8; tmp++) {
 		if (!(cfg->request->resource[tmp].flags & IORESOURCE_AUTO))
 			continue;		/* don't touch */
 		port = cfg->port[tmp];
-		if (!port) {
+		if (port == NULL) {
 			cfg->port[tmp] = port = isapnp_find_port(cfg->request, tmp);
-			if (!port)
+			if (port == NULL)
 				return -EINVAL;
 		}
 		if (from && port->res == from) {
 			while (port->res != to) {
-				if (!port->res->alt)
+				if (port->res->alt == NULL)
 					return -EINVAL;
 				port = port->res->alt->port;
 				for (tmp1 = tmp; tmp1 > 0 && port; tmp1--)
 					port = port->next;
 				cfg->port[tmp] = port;
-				if (!port)
+				if (port == NULL)
 					return -ENOENT;
 				cfg->result.resource[tmp].flags = isapnp_port_resource_flags(port);
 			}
@@ -1388,20 +1394,20 @@ static int isapnp_alternative_switch(struct isapnp_cfgtmp *cfg,
 		if (!(cfg->request->irq_resource[tmp].flags & IORESOURCE_AUTO))
 			continue;		/* don't touch */
 		irq = cfg->irq[tmp];
-		if (!irq) {
+		if (irq == NULL) {
 			cfg->irq[tmp] = irq = isapnp_find_irq(cfg->request, tmp);
-			if (!irq)
+			if (irq == NULL)
 				return -EINVAL;
 		}
 		if (from && irq->res == from) {
 			while (irq->res != to) {
-				if (!irq->res->alt)
+				if (irq->res->alt == NULL)
 					return -EINVAL;
 				irq = irq->res->alt->irq;
 				for (tmp1 = tmp; tmp1 > 0 && irq; tmp1--)
 					irq = irq->next;
 				cfg->irq[tmp] = irq;
-				if (!irq)
+				if (irq == NULL)
 					return -ENOENT;
 				cfg->result.irq_resource[tmp].flags = isapnp_irq_resource_flags(irq);
 			}
@@ -1412,20 +1418,20 @@ static int isapnp_alternative_switch(struct isapnp_cfgtmp *cfg,
 		if (!(cfg->request->dma_resource[tmp].flags & IORESOURCE_AUTO))
 			continue;		/* don't touch */
 		dma = cfg->dma[tmp];
-		if (!dma) {
+		if (dma == NULL) {
 			cfg->dma[tmp] = dma = isapnp_find_dma(cfg->request, tmp);
-			if (!dma)
+			if (dma == NULL)
 				return -EINVAL;
 		}
 		if (from && dma->res == from) {
 			while (dma->res != to) {
-				if (!dma->res->alt)
+				if (dma->res->alt == NULL)
 					return -EINVAL;
 				dma = dma->res->alt->dma;
 				for (tmp1 = tmp; tmp1 > 0 && dma; tmp1--)
 					dma = dma->next;
 				cfg->dma[tmp] = dma;
-				if (!dma)
+				if (dma == NULL)
 					return -ENOENT;
 				cfg->result.dma_resource[tmp].flags = isapnp_dma_resource_flags(dma);
 			}
@@ -1436,20 +1442,20 @@ static int isapnp_alternative_switch(struct isapnp_cfgtmp *cfg,
 		if (!(cfg->request->resource[tmp + 8].flags & IORESOURCE_AUTO))
 			continue;		/* don't touch */
 		mem = cfg->mem[tmp];
-		if (!mem) {
+		if (mem == NULL) {
 			cfg->mem[tmp] = mem = isapnp_find_mem(cfg->request, tmp);
-			if (!mem)
+			if (mem == NULL)
 				return -EINVAL;
 		}
 		if (from && mem->res == from) {
 			while (mem->res != to) {
-				if (!mem->res->alt)
+				if (mem->res->alt == NULL)
 					return -EINVAL;
 				mem = mem->res->alt->mem;
 				for (tmp1 = tmp; tmp1 > 0 && mem; tmp1--)
 					mem = mem->next;
 				cfg->mem[tmp] = mem;
-				if (!mem)
+				if (mem == NULL)
 					return -ENOENT;
 				cfg->result.resource[tmp + 8].flags = isapnp_mem_resource_flags(mem);
 			}
@@ -1498,7 +1504,7 @@ static int isapnp_check_port(struct isapnp_cfgtmp *cfg, int port, int size, int
 		tmp = cfg->request->resource[i].start;
 		if (flags & IORESOURCE_AUTO) {		/* auto */
 			xport = cfg->port[i];
-			if (!xport)
+			if (xport == NULL)
 				return 1;
 			if (cfg->result.resource[i].flags & IORESOURCE_AUTO)
 				continue;
@@ -1510,7 +1516,7 @@ static int isapnp_check_port(struct isapnp_cfgtmp *cfg, int port, int size, int
 		if (port == tmp)
 			return 1;
 		xport = isapnp_find_port(cfg->request, i);
-		if (!xport)
+		if (xport == NULL)
 			return 1;
 		if (tmp + xport->size >= port && tmp <= port + xport->size)
 			return 1;
@@ -1524,13 +1530,13 @@ static int isapnp_valid_port(struct isapnp_cfgtmp *cfg, int idx)
 	unsigned long *value1, *value2;
 	struct isapnp_port *port;
 
-	if (!cfg || idx < 0 || idx > 7)
+	if (cfg == NULL || idx < 0 || idx > 7)
 		return -EINVAL;
 	if (!(cfg->result.resource[idx].flags & IORESOURCE_AUTO)) /* don't touch */
 		return 0;
       __again:
       	port = cfg->port[idx];
-      	if (!port)
+      	if (port == NULL)
       		return -EINVAL;
       	value1 = &cfg->result.resource[idx].start;
       	value2 = &cfg->result.resource[idx].end;
@@ -1604,13 +1610,13 @@ static int isapnp_valid_irq(struct isapnp_cfgtmp *cfg, int idx)
 	unsigned long *value1, *value2;
 	struct isapnp_irq *irq;
 
-	if (!cfg || idx < 0 || idx > 1)
+	if (cfg == NULL || idx < 0 || idx > 1)
 		return -EINVAL;
 	if (!(cfg->result.irq_resource[idx].flags & IORESOURCE_AUTO))
 		return 0;
       __again:
       	irq = cfg->irq[idx];
-      	if (!irq)
+      	if (irq == NULL)
       		return -EINVAL;
       	value1 = &cfg->result.irq_resource[idx].start;
       	value2 = &cfg->result.irq_resource[idx].end;
@@ -1677,13 +1683,13 @@ static int isapnp_valid_dma(struct isapnp_cfgtmp *cfg, int idx)
 	unsigned long *value1, *value2;
 	struct isapnp_dma *dma;
 
-	if (!cfg || idx < 0 || idx > 1)
+	if (cfg == NULL || idx < 0 || idx > 1)
 		return -EINVAL;
 	if (!(cfg->result.dma_resource[idx].flags & IORESOURCE_AUTO))	/* don't touch */
 		return 0;
       __again:
       	dma = cfg->dma[idx];
-      	if (!dma)
+      	if (dma == NULL)
       		return -EINVAL;
       	value1 = &cfg->result.dma_resource[idx].start;
       	value2 = &cfg->result.dma_resource[idx].end;
@@ -1753,7 +1759,7 @@ static int isapnp_check_mem(struct isapnp_cfgtmp *cfg, unsigned int addr, unsign
 		tmp = cfg->result.resource[i + 8].start;
 		if (flags & IORESOURCE_AUTO) {		/* auto */
 			xmem = cfg->mem[i];
-			if (!xmem)
+			if (xmem == NULL)
 				return 1;
 			if (cfg->result.resource[i + 8].flags & IORESOURCE_AUTO)
 				continue;
@@ -1764,7 +1770,7 @@ static int isapnp_check_mem(struct isapnp_cfgtmp *cfg, unsigned int addr, unsign
 		if (addr == tmp)
 			return 1;
 		xmem = isapnp_find_mem(cfg->request, i);
-		if (!xmem)
+		if (xmem == NULL)
 			return 1;
 		if (tmp + xmem->size >= addr && tmp <= addr + xmem->size)
 			return 1;
@@ -1778,16 +1784,16 @@ static int isapnp_valid_mem(struct isapnp_cfgtmp *cfg, int idx)
 	unsigned long *value1, *value2;
 	struct isapnp_mem *mem;
 
-	if (!cfg || idx < 0 || idx > 3)
+	if (cfg == NULL || idx < 0 || idx > 3)
 		return -EINVAL;
 	if (!(cfg->result.resource[idx + 8].flags & IORESOURCE_AUTO)) /* don't touch */
 		return 0;
       __again:
       	mem = cfg->mem[idx];
-      	if (!mem)
+      	if (mem == NULL)
       		return -EINVAL;
-      	value1 = &cfg->result.resource[idx].start;
-      	value2 = &cfg->result.resource[idx].end;
+      	value1 = &cfg->result.resource[idx + 8].start;
+      	value2 = &cfg->result.resource[idx + 8].end;
 	if (cfg->result.resource[idx + 8].flags & IORESOURCE_AUTO) {
 		cfg->result.resource[idx + 8].flags &= ~IORESOURCE_AUTO;
 		*value1 = mem->min;
@@ -1798,7 +1804,7 @@ static int isapnp_valid_mem(struct isapnp_cfgtmp *cfg, int idx)
 	do {
 		*value1 += mem->align;
 		*value2 = *value1 + mem->size - 1;
-		if (*value1 >= 8 || !mem->align) {
+		if (*value1 >= mem->max || !mem->align) {
 			if (mem->res && mem->res->alt) {
 				if ((err = isapnp_alternative_switch(cfg, mem->res, mem->res->alt))<0)
 					return err;
@@ -1834,7 +1840,7 @@ static int isapnp_config_activate(struct isapnp_dev *dev)
 	struct isapnp_cfgtmp cfg;
 	int tmp, fauto, err;
 	
-	if (!dev)
+	if (dev == NULL)
 		return -EINVAL;
 	if (dev->active)
 		return -EBUSY;
@@ -1911,7 +1917,7 @@ static int isapnp_config_activate(struct isapnp_dev *dev)
 
 static int isapnp_config_deactivate(struct isapnp_dev *dev)
 {
-	if (!dev || !dev->active)
+	if (dev == NULL || !dev->active)
 		return -EINVAL;
       	isapnp_cfg_begin(dev->bus->number, dev->devfn);
 	isapnp_deactivate(dev->devfn);
@@ -2139,7 +2145,7 @@ __initfunc(int isapnp_init(void))
 #ifdef ISAPNP_REGION_OK
 #ifdef NEW_RESOURCE
 	pidxr_res=request_region(_PIDXR, 1, "isapnp index");
-	if(!pidxr_res) {
+	if(pidxr_res == NULL) {
 #else
 	if (check_region(_PIDXR, 1)) {
 #endif
@@ -2149,7 +2155,7 @@ __initfunc(int isapnp_init(void))
 #endif
 #ifdef NEW_RESOURCE
 	pnpwrp_res=request_region(_PNPWRP, 1, "isapnp write");
-	if(!pnpwrp_res) {
+	if(pnpwrp_res == NULL) {
 #else
 	if (check_region(_PNPWRP, 1)) {
 #endif
@@ -2159,7 +2165,7 @@ __initfunc(int isapnp_init(void))
 	if (isapnp_rdp >= 0x203 && isapnp_rdp <= 0x3ff) {
 #ifdef NEW_RESOURCE
 		isapnp_rdp_res=request_region(isapnp_rdp, 1, "isapnp read");
-		if(!isapnp_rdp_res) {
+		if(isapnp_rdp_res == NULL) {
 #else
 		if (check_region(isapnp_rdp, 1)) {
 #endif
